@@ -28,8 +28,8 @@
 
 #include "compression.h"
 
-#ifdef Q_OS_WIN
-#include <QtZlib/zlib.h>
+#if defined(Q_OS_WIN) && (QT_VERSION < 0x050600 || Q_CC_MSVC)
+#include "QtZlib/zlib.h"
 #else
 #include <zlib.h>
 #endif
