@@ -39,6 +39,8 @@ const int FlippedHorizontallyFlag   = 0x80000000;
 const int FlippedVerticallyFlag     = 0x40000000;
 const int FlippedAntiDiagonallyFlag = 0x20000000;
 
+const int RotatedHexagonal120Flag   = 0x10000000;
+
 /**
  * Default constructor. Use \l insert to initialize the gid mapper
  * incrementally.
@@ -70,14 +72,17 @@ Cell GidMapper::gidToCell(unsigned gid, bool &ok) const
     Cell result;
 
     // Read out the flags
-    result.flippedHorizontally = (gid & FlippedHorizontallyFlag);
-    result.flippedVertically = (gid & FlippedVerticallyFlag);
-    result.flippedAntiDiagonally = (gid & FlippedAntiDiagonallyFlag);
+    result.setFlippedHorizontally(gid & FlippedHorizontallyFlag);
+    result.setFlippedVertically(gid & FlippedVerticallyFlag);
+    result.setFlippedAntiDiagonally(gid & FlippedAntiDiagonallyFlag);
+
+    result.setRotatedHexagonal120(gid & RotatedHexagonal120Flag);
 
     // Clear the flags
     gid &= ~(FlippedHorizontallyFlag |
              FlippedVerticallyFlag |
-             FlippedAntiDiagonallyFlag);
+             FlippedAntiDiagonallyFlag |
+             RotatedHexagonal120Flag);
 
     if (gid == 0) {
         ok = true;
@@ -94,7 +99,7 @@ Cell GidMapper::gidToCell(unsigned gid, bool &ok) const
             int tileId = gid - i.key();
             Tileset *tileset = i.value();
 
-            result.tile = tileset->findOrCreateTile(tileId);
+            result.setTile(tileset, tileId);
 
             ok = true;
         }
@@ -112,7 +117,7 @@ unsigned GidMapper::cellToGid(const Cell &cell) const
     if (cell.isEmpty())
         return 0;
 
-    const Tileset *tileset = cell.tile->tileset();
+    const Tileset *tileset = cell.tileset();
 
     // Find the first GID for the tileset
     QMap<unsigned, Tileset*>::const_iterator i = mFirstGidToTileset.begin();
@@ -123,13 +128,15 @@ unsigned GidMapper::cellToGid(const Cell &cell) const
     if (i == i_end) // tileset not found
         return 0;
 
-    unsigned gid = i.key() + cell.tile->id();
-    if (cell.flippedHorizontally)
+    unsigned gid = i.key() + cell.tileId();
+    if (cell.flippedHorizontally())
         gid |= FlippedHorizontallyFlag;
-    if (cell.flippedVertically)
+    if (cell.flippedVertically())
         gid |= FlippedVerticallyFlag;
-    if (cell.flippedAntiDiagonally)
+    if (cell.flippedAntiDiagonally())
         gid |= FlippedAntiDiagonallyFlag;
+    if (cell.rotatedHexagonal120())
+        gid |= RotatedHexagonal120Flag;
 
     return gid;
 }
