@@ -27,8 +27,7 @@
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef IMAGELAYER_H
-#define IMAGELAYER_H
+#pragma once
 
 #include "tiled_global.h"
 
@@ -50,7 +49,7 @@ public:
     /**
      * Constructor.
      */
-    ImageLayer(const QString &name, int x, int y, int width, int height);
+    ImageLayer(const QString &name, int x, int y);
 
     /**
      * Destructor.
@@ -114,12 +113,14 @@ public:
      */
     bool loadFromImage(const QImage &image, const QString &fileName);
 
+    bool loadFromImage(const QString &fileName);
+
     /**
      * Returns true if no image source has been set.
      */
     bool isEmpty() const override;
 
-    Layer *clone() const override;
+    ImageLayer *clone() const override;
 
 protected:
     ImageLayer *initializeClone(ImageLayer *clone) const;
@@ -130,6 +131,10 @@ private:
     QPixmap mImage;
 };
 
-} // namespace Tiled
 
-#endif // IMAGELAYER_H
+inline bool ImageLayer::loadFromImage(const QString &fileName)
+{
+    return loadFromImage(QImage(fileName), fileName);
+}
+
+} // namespace Tiled
