@@ -52,6 +52,7 @@ Map::Map(Orientation orientation,
     mHexSideLength(0),
     mStaggerAxis(StaggerY),
     mStaggerIndex(StaggerOdd),
+    mDrawMarginsDirty(true),
     mLayerDataFormat(Base64Zlib),
     mNextObjectId(1)
 {
@@ -70,6 +71,7 @@ Map::Map(const Map &map):
     mStaggerIndex(map.mStaggerIndex),
     mBackgroundColor(map.mBackgroundColor),
     mDrawMargins(map.mDrawMargins),
+    mDrawMarginsDirty(map.mDrawMarginsDirty),
     mTilesets(map.mTilesets),
     mLayerDataFormat(map.mLayerDataFormat),
     mNextObjectId(1)
@@ -86,6 +88,14 @@ Map::~Map()
     qDeleteAll(mLayers);
 }
 
+QMargins Map::drawMargins() const
+{
+    if (mDrawMarginsDirty)
+        recomputeDrawMargins();
+
+    return mDrawMargins;
+}
+
 static QMargins maxMargins(const QMargins &a,
                            const QMargins &b)
 {
@@ -119,7 +129,7 @@ QMargins Map::computeLayerOffsetMargins() const
  * Recomputes the draw margins for this map and each of its tilesets. Needed
  * after the tile offset of a tileset has changed for example.
  */
-void Map::recomputeDrawMargins()
+void Map::recomputeDrawMargins() const
 {
     int maxTileSize = 0;
     QMargins offsetMargins;
@@ -144,30 +154,25 @@ void Map::recomputeDrawMargins()
                             offsetMargins.top() + maxTileSize - mTileHeight,
                             offsetMargins.right() + maxTileSize - mTileWidth,
                             offsetMargins.bottom());
+
+    mDrawMarginsDirty = false;
 }
 
 int Map::layerCount(Layer::TypeFlag type) const
 {
     int count = 0;
-    for (Layer *layer : mLayers)
+    LayerIterator iterator(this);
+    while (Layer *layer = iterator.next())
        if (layer->layerType() == type)
            count++;
     return count;
 }
 
-QList<Layer*> Map::layers(Layer::TypeFlag type) const
-{
-    QList<Layer*> layers;
-    for (Layer *layer : mLayers)
-        if (layer->layerType() == type)
-            layers.append(layer);
-    return layers;
-}
-
 QList<ObjectGroup*> Map::objectGroups() const
 {
     QList<ObjectGroup*> layers;
-    for (Layer *layer : mLayers)
+    LayerIterator iterator(this);
+    while (Layer *layer = iterator.next())
         if (ObjectGroup *og = layer->asObjectGroup())
             layers.append(og);
     return layers;
@@ -176,7 +181,8 @@ QList<ObjectGroup*> Map::objectGroups() const
 QList<TileLayer*> Map::tileLayers() const
 {
     QList<TileLayer*> layers;
-    for (Layer *layer : mLayers)
+    LayerIterator iterator(this);
+    while (Layer *layer = iterator.next())
         if (TileLayer *tl = layer->asTileLayer())
             layers.append(tl);
     return layers;
@@ -208,12 +214,8 @@ void Map::adoptLayer(Layer *layer)
 {
     layer->setMap(this);
 
-    if (ObjectGroup *group = layer->asObjectGroup()) {
-        for (MapObject *o : group->objects()) {
-            if (o->id() == 0)
-                o->setId(takeNextObjectId());
-        }
-    }
+    if (ObjectGroup *group = layer->asObjectGroup())
+        initializeObjectIds(*group);
 }
 
 Layer *Map::takeLayerAt(int index)
@@ -223,9 +225,13 @@ Layer *Map::takeLayerAt(int index)
     return layer;
 }
 
-void Map::addTileset(const SharedTileset &tileset)
+bool Map::addTileset(const SharedTileset &tileset)
 {
+    if (mTilesets.contains(tileset))
+        return false;
+
     mTilesets.append(tileset);
+    return true;
 }
 
 void Map::addTilesets(const QSet<SharedTileset> &tilesets)
@@ -236,6 +242,7 @@ void Map::addTilesets(const QSet<SharedTileset> &tilesets)
 
 void Map::insertTileset(int index, const SharedTileset &tileset)
 {
+    Q_ASSERT(!mTilesets.contains(tileset));
     mTilesets.insert(index, tileset);
 }
 
@@ -249,17 +256,27 @@ void Map::removeTilesetAt(int index)
     mTilesets.remove(index);
 }
 
-void Map::replaceTileset(const SharedTileset &oldTileset,
+bool Map::replaceTileset(const SharedTileset &oldTileset,
                          const SharedTileset &newTileset)
 {
+    Q_ASSERT(oldTileset != newTileset);
+
     const int index = mTilesets.indexOf(oldTileset);
     Q_ASSERT(index != -1);
 
-    for (Layer *layer : mLayers)
+    const auto &layers = mLayers;
+    for (Layer *layer : layers) {
         layer->replaceReferencesToTileset(oldTileset.data(),
                                           newTileset.data());
+    }
 
-    mTilesets.replace(index, newTileset);
+    if (mTilesets.contains(newTileset)) {
+        mTilesets.remove(index);
+        return false;
+    } else {
+        mTilesets.replace(index, newTileset);
+        return true;
+    }
 }
 
 bool Map::isTilesetUsed(const Tileset *tileset) const
@@ -271,6 +288,14 @@ bool Map::isTilesetUsed(const Tileset *tileset) const
     return false;
 }
 
+void Map::initializeObjectIds(ObjectGroup &objectGroup)
+{
+    for (MapObject *o : objectGroup) {
+        if (o->id() == 0)
+            o->setId(takeNextObjectId());
+    }
+}
+
 
 QString Tiled::staggerAxisToString(Map::StaggerAxis staggerAxis)
 {
@@ -382,10 +407,3 @@ Map::RenderOrder Tiled::renderOrderFromString(const QString &string)
     }
     return renderOrder;
 }
-
-Map *Map::fromLayer(Layer *layer)
-{
-    Map *result = new Map(Unknown, layer->width(), layer->height(), 0, 0);
-    result->addLayer(layer);
-    return result;
-}
