@@ -32,13 +32,14 @@
 
 #include "compression.h"
 #include "gidmapper.h"
+#include "grouplayer.h"
 #include "imagelayer.h"
 #include "objectgroup.h"
 #include "map.h"
 #include "mapobject.h"
 #include "tile.h"
 #include "tilelayer.h"
-#include "tilesetformat.h"
+#include "tilesetmanager.h"
 #include "terrain.h"
 
 #include <QCoreApplication>
@@ -80,12 +81,15 @@ private:
 
     SharedTileset readTileset();
     void readTilesetTile(Tileset &tileset);
+    void readTilesetGrid(Tileset &tileset);
     void readTilesetImage(Tileset &tileset);
     void readTilesetTerrainTypes(Tileset &tileset);
     ImageReference readImage();
 
-    TileLayer *readLayer();
-    void readLayerData(TileLayer &tileLayer);
+    Layer *tryReadLayer();
+
+    TileLayer *readTileLayer();
+    void readTileLayerData(TileLayer &tileLayer);
     void decodeBinaryLayerData(TileLayer &tileLayer,
                                const QByteArray &data,
                                Map::LayerDataFormat format);
@@ -107,6 +111,10 @@ private:
     ObjectGroup *readObjectGroup();
     MapObject *readObject();
     QPolygonF readPolygon();
+    TextData readObjectText();
+
+    GroupLayer *readGroupLayer();
+
     QVector<Frame> readAnimationFrames();
 
     Properties readProperties();
@@ -247,16 +255,12 @@ Map *MapReaderPrivate::readMap()
         mMap->setBackgroundColor(QColor(bgColorString.toString()));
 
     while (xml.readNextStartElement()) {
-        if (xml.name() == QLatin1String("properties"))
+        if (Layer *layer = tryReadLayer())
+            mMap->addLayer(layer);
+        else if (xml.name() == QLatin1String("properties"))
             mMap->mergeProperties(readProperties());
         else if (xml.name() == QLatin1String("tileset"))
             mMap->addTileset(readTileset());
-        else if (xml.name() == QLatin1String("layer"))
-            mMap->addLayer(readLayer());
-        else if (xml.name() == QLatin1String("objectgroup"))
-            mMap->addLayer(readObjectGroup());
-        else if (xml.name() == QLatin1String("imagelayer"))
-            mMap->addLayer(readImageLayer());
         else
             readUnknownElement();
     }
@@ -272,14 +276,13 @@ Map *MapReaderPrivate::readMap()
                 tileset->loadImage();
         }
 
-        mMap->recomputeDrawMargins();
-
         // Fix up sizes of tile objects
-        for (Layer *layer : mMap->layers()) {
+        LayerIterator iterator(mMap.data());
+        while (Layer *layer = iterator.next()) {
             if (ObjectGroup *objectGroup = layer->asObjectGroup()) {
                 for (MapObject *object : *objectGroup) {
-                    if (!object->cell().isEmpty()) {
-                        const QSizeF &tileSize = object->cell().tile->size();
+                    if (const Tile *tile = object->cell().tile()) {
+                        const QSizeF tileSize = tile->size();
                         if (object->width() == 0)
                             object->setWidth(tileSize.width());
                         if (object->height() == 0)
@@ -311,6 +314,7 @@ SharedTileset MapReaderPrivate::readTileset()
         const int tileSpacing = atts.value(QLatin1String("spacing")).toInt();
         const int margin = atts.value(QLatin1String("margin")).toInt();
         const int columns = atts.value(QLatin1String("columns")).toInt();
+        QStringRef bgColorString = atts.value(QLatin1String("backgroundcolor"));
 
         if (tileWidth < 0 || tileHeight < 0
             || (firstGid == 0 && !mReadingExternalTileset)) {
@@ -322,6 +326,9 @@ SharedTileset MapReaderPrivate::readTileset()
 
             tileset->setColumnCount(columns);
 
+            if (!bgColorString.isEmpty())
+                tileset->setBackgroundColor(QColor(bgColorString.toString()));
+
             while (xml.readNextStartElement()) {
                 if (xml.name() == QLatin1String("tile")) {
                     readTilesetTile(*tileset);
@@ -331,6 +338,8 @@ SharedTileset MapReaderPrivate::readTileset()
                     int y = oa.value(QLatin1String("y")).toInt();
                     tileset->setTileOffset(QPoint(x, y));
                     xml.skipCurrentElement();
+                } else if (xml.name() == QLatin1String("grid")) {
+                    readTilesetGrid(*tileset);
                 } else if (xml.name() == QLatin1String("properties")) {
                     tileset->mergeProperties(readProperties());
                 } else if (xml.name() == QLatin1String("image")) {
@@ -384,6 +393,8 @@ void MapReaderPrivate::readTilesetTile(Tileset &tileset)
 
     Tile *tile = tileset.findOrCreateTile(id);
 
+    tile->setType(atts.value(QLatin1String("type")).toString());
+
     // Read tile quadrant terrain ids
     QString terrain = atts.value(QLatin1String("terrain")).toString();
     if (!terrain.isEmpty()) {
@@ -446,6 +457,25 @@ void MapReaderPrivate::readTilesetTile(Tileset &tileset)
     }
 }
 
+void MapReaderPrivate::readTilesetGrid(Tileset &tileset)
+{
+    Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("grid"));
+
+    const QXmlStreamAttributes atts = xml.attributes();
+
+    const QString orientation = atts.value(QLatin1String("orientation")).toString();
+    const int width = atts.value(QLatin1String("width")).toInt();
+    const int height = atts.value(QLatin1String("height")).toInt();
+
+    tileset.setOrientation(Tileset::orientationFromString(orientation));
+
+    const QSize gridSize(width, height);
+    if (!gridSize.isEmpty())
+        tileset.setGridSize(gridSize);
+
+    xml.skipCurrentElement();
+}
+
 void MapReaderPrivate::readTilesetImage(Tileset &tileset)
 {
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("image"));
@@ -494,6 +524,22 @@ ImageReference MapReaderPrivate::readImage()
     return image;
 }
 
+Layer *MapReaderPrivate::tryReadLayer()
+{
+    Q_ASSERT(xml.isStartElement());
+
+    if (xml.name() == QLatin1String("layer"))
+        return readTileLayer();
+    else if (xml.name() == QLatin1String("objectgroup"))
+        return readObjectGroup();
+    else if (xml.name() == QLatin1String("imagelayer"))
+        return readImageLayer();
+    else if (xml.name() == QLatin1String("group"))
+        return readGroupLayer();
+    else
+        return nullptr;
+}
+
 void MapReaderPrivate::readTilesetTerrainTypes(Tileset &tileset)
 {
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("terraintypes"));
@@ -539,7 +585,7 @@ static void readLayerAttributes(Layer &layer,
     layer.setOffset(offset);
 }
 
-TileLayer *MapReaderPrivate::readLayer()
+TileLayer *MapReaderPrivate::readTileLayer()
 {
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("layer"));
 
@@ -557,7 +603,7 @@ TileLayer *MapReaderPrivate::readLayer()
         if (xml.name() == QLatin1String("properties"))
             tileLayer->mergeProperties(readProperties());
         else if (xml.name() == QLatin1String("data"))
-            readLayerData(*tileLayer);
+            readTileLayerData(*tileLayer);
         else
             readUnknownElement();
     }
@@ -565,7 +611,7 @@ TileLayer *MapReaderPrivate::readLayer()
     return tileLayer;
 }
 
-void MapReaderPrivate::readLayerData(TileLayer &tileLayer)
+void MapReaderPrivate::readTileLayerData(TileLayer &tileLayer)
 {
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("data"));
 
@@ -707,10 +753,8 @@ ObjectGroup *MapReaderPrivate::readObjectGroup()
     const QString name = atts.value(QLatin1String("name")).toString();
     const int x = atts.value(QLatin1String("x")).toInt();
     const int y = atts.value(QLatin1String("y")).toInt();
-    const int width = atts.value(QLatin1String("width")).toInt();
-    const int height = atts.value(QLatin1String("height")).toInt();
 
-    ObjectGroup *objectGroup = new ObjectGroup(name, x, y, width, height);
+    ObjectGroup *objectGroup = new ObjectGroup(name, x, y);
     readLayerAttributes(*objectGroup, atts);
 
     const QString color = atts.value(QLatin1String("color")).toString();
@@ -748,10 +792,8 @@ ImageLayer *MapReaderPrivate::readImageLayer()
     const QString name = atts.value(QLatin1String("name")).toString();
     const int x = atts.value(QLatin1String("x")).toInt();
     const int y = atts.value(QLatin1String("y")).toInt();
-    const int width = atts.value(QLatin1String("width")).toInt();
-    const int height = atts.value(QLatin1String("height")).toInt();
 
-    ImageLayer *imageLayer = new ImageLayer(name, x, y, width, height);
+    ImageLayer *imageLayer = new ImageLayer(name, x, y);
     readLayerAttributes(*imageLayer, atts);
 
     // Image layer pixel position moved from x/y to offsetx/offsety for
@@ -837,6 +879,9 @@ MapObject *MapReaderPrivate::readObject()
         } else if (xml.name() == QLatin1String("ellipse")) {
             xml.skipCurrentElement();
             object->setShape(MapObject::Ellipse);
+        } else if (xml.name() == QLatin1String("text")) {
+            object->setTextData(readObjectText());
+            object->setShape(MapObject::Text);
         } else {
             readUnknownElement();
         }
@@ -858,7 +903,7 @@ QPolygonF MapReaderPrivate::readPolygon()
     QPolygonF polygon;
     bool ok = true;
 
-    foreach (const QString &point, pointsList) {
+    for (const QString &point : pointsList) {
         const int commaPos = point.indexOf(QLatin1Char(','));
         if (commaPos == -1) {
             ok = false;
@@ -882,6 +927,87 @@ QPolygonF MapReaderPrivate::readPolygon()
     return polygon;
 }
 
+static int intAttribute(const QXmlStreamAttributes& atts, const char *name, int def)
+{
+    bool ok = false;
+    int value = atts.value(QLatin1String(name)).toInt(&ok);
+    return ok ? value : def;
+}
+
+TextData MapReaderPrivate::readObjectText()
+{
+    Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("text"));
+
+    const QXmlStreamAttributes atts = xml.attributes();
+
+    TextData textData;
+
+    if (atts.hasAttribute(QLatin1String("fontfamily")))
+        textData.font = QFont(atts.value(QLatin1String("fontfamily")).toString());
+
+    if (atts.hasAttribute(QLatin1String("pixelsize")))
+        textData.font.setPixelSize(atts.value(QLatin1String("pixelsize")).toInt());
+
+    textData.wordWrap = intAttribute(atts, "wrap", 0) == 1;
+    textData.font.setBold(intAttribute(atts, "bold", 0) == 1);
+    textData.font.setItalic(intAttribute(atts, "italic", 0) == 1);
+    textData.font.setUnderline(intAttribute(atts, "underline", 0) == 1);
+    textData.font.setStrikeOut(intAttribute(atts, "strikeout", 0) == 1);
+    textData.font.setKerning(intAttribute(atts, "kerning", 1) == 1);
+
+    QStringRef colorString = atts.value(QLatin1String("color"));
+    if (!colorString.isEmpty())
+        textData.color = QColor(colorString.toString());
+
+    Qt::Alignment alignment = 0;
+
+    QStringRef hAlignString = atts.value(QLatin1String("halign"));
+    if (hAlignString == QLatin1String("center"))
+        alignment |= Qt::AlignHCenter;
+    else if (hAlignString == QLatin1String("right"))
+        alignment |= Qt::AlignRight;
+    else
+        alignment |= Qt::AlignLeft;
+
+    QStringRef vAlignString = atts.value(QLatin1String("valign"));
+    if (vAlignString == QLatin1String("center"))
+        alignment |= Qt::AlignVCenter;
+    else if (vAlignString == QLatin1String("bottom"))
+        alignment |= Qt::AlignBottom;
+    else
+        alignment |= Qt::AlignTop;
+
+    textData.alignment = alignment;
+
+    textData.text = xml.readElementText();
+
+    return textData;
+}
+
+GroupLayer *MapReaderPrivate::readGroupLayer()
+{
+    Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("group"));
+
+    const QXmlStreamAttributes atts = xml.attributes();
+    const QString name = atts.value(QLatin1String("name")).toString();
+    const int x = atts.value(QLatin1String("x")).toInt();
+    const int y = atts.value(QLatin1String("y")).toInt();
+
+    GroupLayer *groupLayer = new GroupLayer(name, x, y);
+    readLayerAttributes(*groupLayer, atts);
+
+    while (xml.readNextStartElement()) {
+        if (Layer *layer = tryReadLayer())
+            groupLayer->addLayer(layer);
+        else if (xml.name() == QLatin1String("properties"))
+            groupLayer->mergeProperties(readProperties());
+        else
+            readUnknownElement();
+    }
+
+    return groupLayer;
+}
+
 QVector<Frame> MapReaderPrivate::readAnimationFrames()
 {
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("animation"));
@@ -1019,5 +1145,5 @@ QString MapReader::resolveReference(const QString &reference,
 SharedTileset MapReader::readExternalTileset(const QString &source,
                                              QString *error)
 {
-    return Tiled::readTileset(source, error);
+    return TilesetManager::instance()->loadTileset(source, error);
 }
