@@ -27,8 +27,7 @@
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef MAPWRITER_H
-#define MAPWRITER_H
+#pragma once
 
 #include "map.h"
 #include "tiled_global.h"
@@ -105,9 +104,9 @@ public:
     bool isDtdEnabled() const;
 
 private:
+    Q_DISABLE_COPY(MapWriter)
+
     Internal::MapWriterPrivate *d;
 };
 
 } // namespace Tiled
-
-#endif // MAPWRITER_H
