@@ -26,8 +26,7 @@
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef STAGGEREDRENDERER_H
-#define STAGGEREDRENDERER_H
+#pragma once
 
 #include "hexagonalrenderer.h"
 
@@ -79,9 +78,7 @@ public:
     StaggeredRenderer(const Map *map) : HexagonalRenderer(map) {}
 
     using HexagonalRenderer::screenToTileCoords;
-    QPointF screenToTileCoords(qreal x, qreal y) const;
+    QPointF screenToTileCoords(qreal x, qreal y) const override;
 };
 
 } // namespace Tiled
-
-#endif // STAGGEREDRENDERER_H
