@@ -1,4 +1,4 @@
-/* 
+/*
  * tilelayer.cpp
  * Copyright 2008-2011, Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
  * Copyright 2009, Jeff Bland <jksb@member.fsf.org>
@@ -29,13 +29,15 @@
 
 #include "tilelayer.h"
 
-#include "map.h"
 #include "tile.h"
+#include "hex.h"
 
 using namespace Tiled;
 
 TileLayer::TileLayer(const QString &name, int x, int y, int width, int height)
-    : Layer(TileLayerType, name, x, y, width, height)
+    : Layer(TileLayerType, name, x, y)
+    , mWidth(width)
+    , mHeight(height)
     , mGrid(width * height)
     , mUsedTilesetsDirty(false)
 {
@@ -115,8 +117,8 @@ void Tiled::TileLayer::setCell(int x, int y, const Cell &cell)
     Cell &existingCell = mGrid[x + y * mWidth];
 
     if (!mUsedTilesetsDirty) {
-        Tileset *oldTileset = existingCell.isEmpty() ? nullptr : existingCell.tile->tileset();
-        Tileset *newTileset = cell.isEmpty() ? nullptr : cell.tile->tileset();
+        Tileset *oldTileset = existingCell.isEmpty() ? nullptr : existingCell.tileset();
+        Tileset *newTileset = cell.tileset();
         if (oldTileset != newTileset) {
             if (oldTileset)
                 mUsedTilesetsDirty = true;
@@ -182,6 +184,25 @@ void TileLayer::setCells(int x, int y, TileLayer *layer,
                 setCell(_x, _y, layer->cellAt(_x - x, _y - y));
 }
 
+/**
+ * Sets the tiles in the given \a area to \a tile. Flipping flags are
+ * preserved.
+ */
+void TileLayer::setTiles(const QRegion &area, Tile *tile)
+{
+    Q_ASSERT(area.subtracted(QRegion(0, 0, mWidth, mHeight)).isEmpty());
+
+    for (const QRect &rect : area.rects()) {
+        for (int x = rect.left(); x <= rect.right(); ++x) {
+            for (int y = rect.top(); y <= rect.bottom(); ++y) {
+                Cell cell = cellAt(x, y);
+                cell.setTile(tile);
+                setCell(x, y, cell);
+            }
+        }
+    }
+}
+
 void TileLayer::erase(const QRegion &area)
 {
     const Cell emptyCell;
@@ -203,11 +224,11 @@ void TileLayer::flip(FlipDirection direction)
             if (direction == FlipHorizontally) {
                 const Cell &source = cellAt(mWidth - x - 1, y);
                 dest = source;
-                dest.flippedHorizontally = !source.flippedHorizontally;
+                dest.setFlippedHorizontally(!source.flippedHorizontally());
             } else if (direction == FlipVertically) {
                 const Cell &source = cellAt(x, mHeight - y - 1);
                 dest = source;
-                dest.flippedVertically = !source.flippedVertically;
+                dest.setFlippedVertically(!source.flippedVertically());
             }
         }
     }
@@ -215,6 +236,42 @@ void TileLayer::flip(FlipDirection direction)
     mGrid = newGrid;
 }
 
+void TileLayer::flipHexagonal(FlipDirection direction)
+{
+    QVector<Cell> newGrid(mWidth * mHeight);
+
+    Q_ASSERT(direction == FlipHorizontally || direction == FlipVertically);
+
+    // for more info see impl "void TileLayer::rotateHexagonal(RotateDirection direction)"
+    static const char flipMaskH[16] = { 8, 6, 5, 4, 12, 2, 1, 0, 0, 14, 13, 12, 4, 10, 9, 8 }; // [0,15]<=>[8,7]; 2<=>5; 1<=>6; [12,3]<=>[4,11]; 14<=>9; 13<=>10;
+    static const char flipMaskV[16] = { 4, 10, 9, 8, 0, 14, 13, 12, 12, 2, 1, 0, 8, 6, 5, 4 }; // [0,15]<=>[4,11]; 2<=>9; 1<=>10; [12,3]<=>[8,7]; 14<=>5; 13<=>6;
+
+    const char (&flipMask)[16] = (direction == FlipHorizontally ? flipMaskH : flipMaskV);
+
+    for (int y = 0; y < mHeight; ++y) {
+        for (int x = 0; x < mWidth; ++x) {
+            const Cell &source = (direction == FlipHorizontally ? cellAt(mWidth - x - 1, y) : cellAt(x, mHeight - y - 1));
+            Cell &dest = newGrid[x + y * mWidth];
+            dest = source;
+
+            unsigned char mask =
+                    (static_cast<unsigned char>(dest.flippedHorizontally()) << 3) |
+                    (static_cast<unsigned char>(dest.flippedVertically()) << 2) |
+                    (static_cast<unsigned char>(dest.flippedAntiDiagonally()) << 1) |
+                    (static_cast<unsigned char>(dest.rotatedHexagonal120()) << 0);
+
+            mask = flipMask[mask];
+
+            dest.setFlippedHorizontally((mask & 8) != 0);
+            dest.setFlippedVertically((mask & 4) != 0);
+            dest.setFlippedAntiDiagonally((mask & 2) != 0);
+            dest.setRotatedHexagonal120((mask & 1) != 0);
+        }
+    }
+
+    mGrid = newGrid;
+}
+
 void TileLayer::rotate(RotateDirection direction)
 {
     static const char rotateRightMask[8] = { 5, 4, 1, 0, 7, 6, 3, 2 };
@@ -233,15 +290,15 @@ void TileLayer::rotate(RotateDirection direction)
             Cell dest = source;
 
             unsigned char mask =
-                    (dest.flippedHorizontally << 2) |
-                    (dest.flippedVertically << 1) |
-                    (dest.flippedAntiDiagonally << 0);
+                    (dest.flippedHorizontally() << 2) |
+                    (dest.flippedVertically() << 1) |
+                    (dest.flippedAntiDiagonally() << 0);
 
             mask = rotateMask[mask];
 
-            dest.flippedHorizontally = (mask & 4) != 0;
-            dest.flippedVertically = (mask & 2) != 0;
-            dest.flippedAntiDiagonally = (mask & 1) != 0;
+            dest.setFlippedHorizontally((mask & 4) != 0);
+            dest.setFlippedVertically((mask & 2) != 0);
+            dest.setFlippedAntiDiagonally((mask & 1) != 0);
 
             if (direction == RotateRight)
                 newGrid[x * newWidth + (mHeight - y - 1)] = dest;
@@ -255,6 +312,99 @@ void TileLayer::rotate(RotateDirection direction)
     mGrid = newGrid;
 }
 
+void TileLayer::rotateHexagonal(RotateDirection direction, Map *map)
+{
+    Map::StaggerIndex staggerIndex = map->staggerIndex();
+    Map::StaggerAxis staggerAxis = map->staggerAxis();
+
+    Hex bottomRight(mWidth, mHeight, staggerIndex, staggerAxis);
+    Hex topRight(mWidth, 0, staggerIndex, staggerAxis);
+    Hex center(mWidth / 2, mHeight / 2, staggerIndex, staggerAxis);
+
+    bottomRight -= center;
+    topRight -= center;
+
+    bottomRight.rotate(RotateRight);
+    topRight.rotate(RotateRight);
+
+    int newWidth = topRight.toStaggered(staggerIndex, staggerAxis).x() * 2 + 2;
+    int newHeight = bottomRight.toStaggered(staggerIndex, staggerAxis).y() * 2 + 2;
+    QVector<Cell> newGrid(newWidth * newHeight);
+
+    Hex newCenter(newWidth / 2, newHeight / 2, staggerIndex, staggerAxis);
+
+    /* https://github.com/bjorn/tiled/pull/1447
+
+  0 or 15     0: None or (Rotated60 | Rotated120 | FlippedVertically | FlippedHorizontally)
+     2       60: Rotated60
+     1      120: Rotated120
+ 12 or 3    180: (FlippedHorizontally | FlippedVertically) or (Rotated60 | Rotated120)
+    14      240: Rotated60 | FlippedHorizontally | FlippedVertically
+    13      300: Rotated120 | FlippedHorizontally | FlippedVertically
+
+  8 or 7      0: FlippedHorizontally or (Rotated60 | Rotated120 | FlippedVertically)
+    10       60: Rotated60 | FlippedHorizontally
+     9      120: Rotated120 | FlippedHorizontally
+  4 or 11   180: (FlippedVertically) or (Rotated60 | Rotated120 | FlippedHorizontally)
+     6      240: Rotated60 | FlippedVertically
+     5      300: Rotated120 | FlippedVertically
+
+    */
+
+    static const char rotateRightMask[16] = { 2, 12, 1, 14, 6, 8, 5, 10, 10,  4, 9, 0, 14,  0, 13,  2 }; // [0,15]->2->1->[12,3]->14->13; [8,7]->10->9->[4,11]->6->5;
+    static const char rotateLeftMask[16]  = { 13, 2, 0,  1, 9, 6, 4,  5,  5, 10, 8, 9,  1, 14, 12, 13 }; // [0,15]->13->14->[12,3]->1->2; [8,7]->5->6->[4,11]->9->10;
+
+    const char (&rotateMask)[16] =
+            (direction == RotateRight) ? rotateRightMask : rotateLeftMask;
+
+    for (int y = 0; y < mHeight; ++y) {
+        for (int x = 0; x < mWidth; ++x) {
+            const Cell &source = cellAt(x, y);
+            Cell dest = source;
+
+            unsigned char mask =
+                    (static_cast<unsigned char>(dest.flippedHorizontally()) << 3) |
+                    (static_cast<unsigned char>(dest.flippedVertically()) << 2) |
+                    (static_cast<unsigned char>(dest.flippedAntiDiagonally()) << 1) |
+                    (static_cast<unsigned char>(dest.rotatedHexagonal120()) << 0);
+
+            mask = rotateMask[mask];
+
+            dest.setFlippedHorizontally((mask & 8) != 0);
+            dest.setFlippedVertically((mask & 4) != 0);
+            dest.setFlippedAntiDiagonally((mask & 2) != 0);
+            dest.setRotatedHexagonal120((mask & 1) != 0);
+
+            Hex rotatedHex(x, y, staggerIndex, staggerAxis);
+            rotatedHex -= center;
+            rotatedHex.rotate(direction);
+            rotatedHex += newCenter;
+
+            QPoint rotatedPoint = rotatedHex.toStaggered(staggerIndex, staggerAxis);
+
+            int index = rotatedPoint.y() * newWidth + rotatedPoint.x();
+
+            newGrid[index] = dest;
+        }
+    }
+
+    mWidth = newWidth;
+    mHeight = newHeight;
+    mGrid = newGrid;
+
+    QRect filledRect = region().boundingRect();
+
+    if (staggerAxis == Map::StaggerY) {
+        if (filledRect.y() & 1)
+            map->invertStaggerIndex();
+    } else {
+        if (filledRect.x() & 1)
+            map->invertStaggerIndex();
+    }
+
+    resize(filledRect.size(), -filledRect.topLeft());
+}
+
 
 QSet<SharedTileset> TileLayer::usedTilesets() const
 {
@@ -262,7 +412,7 @@ QSet<SharedTileset> TileLayer::usedTilesets() const
         QSet<SharedTileset> tilesets;
 
         for (const Cell &cell : mGrid)
-            if (const Tile *tile = cell.tile)
+            if (const Tile *tile = cell.tile())
                 tilesets.insert(tile->sharedTileset());
 
         mUsedTilesets.swap(tilesets);
@@ -283,19 +433,13 @@ bool TileLayer::hasCell(std::function<bool (const Cell &)> condition) const
 
 bool TileLayer::referencesTileset(const Tileset *tileset) const
 {
-    for (const Cell &cell : mGrid) {
-        const Tile *tile = cell.tile;
-        if (tile && tile->tileset() == tileset)
-            return true;
-    }
-    return false;
+    return usedTilesets().contains(tileset->sharedPointer());
 }
 
 void TileLayer::removeReferencesToTileset(Tileset *tileset)
 {
     for (int i = 0, i_end = mGrid.size(); i < i_end; ++i) {
-        const Tile *tile = mGrid.at(i).tile;
-        if (tile && tile->tileset() == tileset)
+        if (mGrid.at(i).tileset() == tileset)
             mGrid.replace(i, Cell());
     }
 
@@ -306,9 +450,8 @@ void TileLayer::replaceReferencesToTileset(Tileset *oldTileset,
                                            Tileset *newTileset)
 {
     for (Cell &cell : mGrid) {
-        const Tile *tile = cell.tile;
-        if (tile && tile->tileset() == oldTileset)
-            cell.tile = newTileset->findOrCreateTile(tile->id());
+        if (cell.tileset() == oldTileset)
+            cell.setTile(newTileset, cell.tileId());
     }
 
     if (mUsedTilesets.remove(oldTileset->sharedPointer()))
@@ -397,7 +540,7 @@ Layer *TileLayer::mergedWith(Layer *other) const
     const QRect unitedBounds = bounds().united(o->bounds());
     const QPoint offset = position() - unitedBounds.topLeft();
 
-    TileLayer *merged = static_cast<TileLayer*>(clone());
+    TileLayer *merged = clone();
     merged->resize(unitedBounds.size(), offset);
     merged->merge(o->position() - unitedBounds.topLeft(), o);
     return merged;
@@ -443,7 +586,7 @@ bool TileLayer::isEmpty() const
  *
  * \sa Layer::clone()
  */
-Layer *TileLayer::clone() const
+TileLayer *TileLayer::clone() const
 {
     return initializeClone(new TileLayer(mName, mX, mY, mWidth, mHeight));
 }
