@@ -27,19 +27,23 @@
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef TILELAYER_H
-#define TILELAYER_H
+#pragma once
 
 #include "tiled_global.h"
 
+#include "map.h"
 #include "layer.h"
 #include "tiled.h"
+#include "tile.h"
+#include "tileset.h"
 
 #include <QMargins>
 #include <QString>
 #include <QVector>
 #include <QSharedPointer>
 
+#include <functional>
+
 namespace Tiled {
 
 class Tile;
@@ -51,43 +55,100 @@ class Cell
 {
 public:
     Cell() :
-        tile(nullptr),
-        flippedHorizontally(false),
-        flippedVertically(false),
-        flippedAntiDiagonally(false)
+        _tileset(nullptr),
+        _tileId(-1),
+        _flippedHorizontally(false),
+        _flippedVertically(false),
+        _flippedAntiDiagonally(false),
+        _rotatedHexagonal120(false)
     {}
 
     explicit Cell(Tile *tile) :
-        tile(tile),
-        flippedHorizontally(false),
-        flippedVertically(false),
-        flippedAntiDiagonally(false)
+        _tileset(tile ? tile->tileset() : nullptr),
+        _tileId(tile ? tile->id() : -1),
+        _flippedHorizontally(false),
+        _flippedVertically(false),
+        _flippedAntiDiagonally(false),
+        _rotatedHexagonal120(false)
     {}
 
-    bool isEmpty() const { return tile == nullptr; }
+    bool isEmpty() const { return _tileset == nullptr; }
 
     bool operator == (const Cell &other) const
     {
-        return tile == other.tile
-                && flippedHorizontally == other.flippedHorizontally
-                && flippedVertically == other.flippedVertically
-                && flippedAntiDiagonally == other.flippedAntiDiagonally;
+        return _tileset == other._tileset
+                && _tileId == other._tileId
+                && _flippedHorizontally == other._flippedHorizontally
+                && _flippedVertically == other._flippedVertically
+                && _flippedAntiDiagonally == other._flippedAntiDiagonally
+                && _rotatedHexagonal120 == other._rotatedHexagonal120;
     }
 
     bool operator != (const Cell &other) const
     {
-        return tile != other.tile
-                || flippedHorizontally != other.flippedHorizontally
-                || flippedVertically != other.flippedVertically
-                || flippedAntiDiagonally != other.flippedAntiDiagonally;
+        return _tileset != other._tileset
+                || _tileId != other._tileId
+                || _flippedHorizontally != other._flippedHorizontally
+                || _flippedVertically != other._flippedVertically
+                || _flippedAntiDiagonally != other._flippedAntiDiagonally
+                || _rotatedHexagonal120 != other._rotatedHexagonal120;
     }
 
-    Tile *tile;
-    bool flippedHorizontally;
-    bool flippedVertically;
-    bool flippedAntiDiagonally;
+    Tileset *tileset() const { return _tileset; }
+    int tileId() const { return _tileId; }
+
+    bool flippedHorizontally() const { return _flippedHorizontally; }
+    bool flippedVertically() const { return _flippedVertically; }
+    bool flippedAntiDiagonally() const { return _flippedAntiDiagonally; }
+
+    bool rotatedHexagonal120() const { return _rotatedHexagonal120; }
+
+    void setFlippedHorizontally(bool f) { _flippedHorizontally = f; }
+    void setFlippedVertically(bool f) { _flippedVertically = f; }
+    void setFlippedAntiDiagonally(bool f) { _flippedAntiDiagonally = f; }
+
+    void setRotatedHexagonal120(bool f) { _rotatedHexagonal120 = f; }
+
+    Tile *tile() const;
+    void setTile(Tile *tile);
+    void setTile(Tileset *tileset, int tileId);
+    bool refersTile(const Tile *tile) const;
+
+private:
+    Tileset *_tileset;
+    int _tileId;
+    bool _flippedHorizontally;
+    bool _flippedVertically;
+    bool _flippedAntiDiagonally;
+
+    bool _rotatedHexagonal120;
 };
 
+inline Tile *Cell::tile() const
+{
+    return _tileset ? _tileset->findTile(_tileId) : nullptr;
+}
+
+inline void Cell::setTile(Tile *tile)
+{
+    if (tile)
+        setTile(tile->tileset(), tile->id());
+    else
+        setTile(nullptr, -1);
+}
+
+inline void Cell::setTile(Tileset *tileset, int tileId)
+{
+    _tileset = tileset;
+    _tileId = tileId;
+}
+
+inline bool Cell::refersTile(const Tile *tile) const
+{
+    return _tileset == tile->tileset() && _tileId == tile->id();
+}
+
+
 /**
  * A tile layer is a grid of cells. Each cell refers to a specific tile, and
  * stores how the tile is flipped.
@@ -104,40 +165,37 @@ public:
     TileLayer(const QString &name, int x, int y, int width, int height);
 
     /**
-     * Returns the maximum tile size of this layer.
+     * Returns the width of this layer.
      */
-    QSize maxTileSize() const { return mMaxTileSize; }
+    int width() const { return mWidth; }
 
     /**
-     * Returns the margins that have to be taken into account while drawing
-     * this tile layer. The margins depend on the maximum tile size and the
-     * offset applied to the tiles.
+     * Returns the height of this layer.
      */
-    QMargins drawMargins() const
-    {
-        return QMargins(mOffsetMargins.left(),
-                        mOffsetMargins.top() + mMaxTileSize.height(),
-                        mOffsetMargins.right() + mMaxTileSize.width(),
-                        mOffsetMargins.bottom());
-    }
-
-    void recomputeDrawMargins();
+    int height() const { return mHeight; }
 
     /**
-     * Returns whether (x, y) is inside this map layer.
+     * Returns the size of this layer.
      */
-    bool contains(int x, int y) const
-    { return x >= 0 && y >= 0 && x < mWidth && y < mHeight; }
+    QSize size() const { return QSize(mWidth, mHeight); }
 
-    bool contains(const QPoint &point) const
-    { return contains(point.x(), point.y()); }
+    void setSize(const QSize &size);
+
+    /**
+     * Returns the bounds of this layer.
+     */
+    QRect bounds() const { return QRect(mX, mY, mWidth, mHeight); }
+
+    QMargins drawMargins() const;
+
+    bool contains(int x, int y) const;
+    bool contains(const QPoint &point) const;
 
     /**
      * Calculates the region of cells in this tile layer for which the given
      * \a condition returns true.
      */
-    template<typename Condition>
-    QRegion region(Condition condition) const;
+    QRegion region(std::function<bool (const Cell &)> condition) const;
 
     /**
      * Calculates the region occupied by the tiles of this layer. Similar to
@@ -145,17 +203,9 @@ public:
      */
     QRegion region() const;
 
-    /**
-     * Returns a read-only reference to the cell at the given coordinates. The
-     * coordinates have to be within this layer.
-     */
     const Cell &cellAt(int x, int y) const;
-
     const Cell &cellAt(const QPoint &point) const;
 
-    /**
-     * Sets the cell at the given coordinates.
-     */
     void setCell(int x, int y, const Cell &cell);
 
     /**
@@ -189,6 +239,8 @@ public:
     void setCells(int x, int y, TileLayer *tileLayer,
                   const QRegion &mask = QRegion());
 
+    void setTiles(const QRegion &area, Tile *tile);
+
     /**
      * Flip this tile layer in the given \a direction. Direction must be
      * horizontal or vertical. This doesn't change the dimensions of the
@@ -196,6 +248,13 @@ public:
      */
     void flip(FlipDirection direction);
 
+    /**
+     * Hexagonal flip this tile layer in the given \a direction. Direction must be
+     * horizontal or vertical. This doesn't change the dimensions of the
+     * tile layer.
+     */
+    void flipHexagonal(FlipDirection direction);
+
     /**
      * Rotate this tile layer by 90 degrees left or right. The tile positions
      * are rotated within the layer, and the tiles themselves are rotated. The
@@ -203,6 +262,14 @@ public:
      */
     void rotate(RotateDirection direction);
 
+    /**
+     * Hexagonal rotate this tile layer by 60 degrees left or right. The tile positions
+     * are rotated within the layer, and the tiles themselves are rotated.
+     * As a temporary measure, a Map* is passed to give information about stagger index
+     * and axis, which affects rotation. The stagger index of this map can change.
+     */
+    void rotateHexagonal(RotateDirection direction, Map *map);
+
     /**
      * Computes and returns the set of tilesets used by this tile layer.
      */
@@ -212,8 +279,7 @@ public:
      * Returns whether this tile layer has any cell for which the given
      * \a condition returns true.
      */
-    template<typename Condition>
-    bool hasCell(Condition condition) const;
+    bool hasCell(std::function<bool (const Cell &)> condition) const;
 
     /**
      * Returns whether this tile layer is referencing the given tileset.
@@ -263,7 +329,7 @@ public:
      */
     bool isEmpty() const override;
 
-    virtual Layer *clone() const override;
+    TileLayer *clone() const override;
 
     // Enable easy iteration over cells with range-based for
     QVector<Cell>::iterator begin() { return mGrid.begin(); }
@@ -275,44 +341,34 @@ protected:
     TileLayer *initializeClone(TileLayer *clone) const;
 
 private:
-    QSize mMaxTileSize;
-    QMargins mOffsetMargins;
+    int mWidth;
+    int mHeight;
     QVector<Cell> mGrid;
+    mutable QSet<SharedTileset> mUsedTilesets;
+    mutable bool mUsedTilesetsDirty;
 };
 
 
-template<typename Condition>
-QRegion TileLayer::region(Condition condition) const
+/**
+ * Sets the size of this layer.
+ */
+inline void TileLayer::setSize(const QSize &size)
 {
-    QRegion region;
-
-    for (int y = 0; y < mHeight; ++y) {
-        for (int x = 0; x < mWidth; ++x) {
-            if (condition(cellAt(x, y))) {
-                const int rangeStart = x;
-                for (++x; x <= mWidth; ++x) {
-                    if (x == mWidth || !condition(cellAt(x, y))) {
-                        const int rangeEnd = x;
-                        region += QRect(rangeStart + mX, y + mY,
-                                        rangeEnd - rangeStart, 1);
-                        break;
-                    }
-                }
-            }
-        }
-    }
-
-    return region;
+    mWidth = size.width();
+    mHeight = size.height();
 }
 
-template<typename Condition>
-bool TileLayer::hasCell(Condition condition) const
+/**
+ * Returns whether (x, y) is inside this map layer.
+ */
+inline bool TileLayer::contains(int x, int y) const
 {
-    for (const Cell &cell : mGrid)
-        if (condition(cell))
-            return true;
+    return x >= 0 && y >= 0 && x < mWidth && y < mHeight;
+}
 
-    return false;
+inline bool TileLayer::contains(const QPoint &point) const
+{
+    return contains(point.x(), point.y());
 }
 
 inline QRegion TileLayer::region() const
@@ -320,6 +376,10 @@ inline QRegion TileLayer::region() const
     return region([] (const Cell &cell) { return !cell.isEmpty(); });
 }
 
+/**
+ * Returns a read-only reference to the cell at the given coordinates. The
+ * coordinates have to be within this layer.
+ */
 inline const Cell &TileLayer::cellAt(int x, int y) const
 {
     Q_ASSERT(contains(x, y));
@@ -334,5 +394,3 @@ inline const Cell &TileLayer::cellAt(const QPoint &point) const
 typedef QSharedPointer<TileLayer> SharedTileLayer;
 
 } // namespace Tiled
-
-#endif // TILELAYER_H
