@@ -28,8 +28,10 @@
  */
 
 #include "tileset.h"
-#include "tile.h"
+
 #include "terrain.h"
+#include "tile.h"
+#include "tilesetformat.h"
 
 #include <QBitmap>
 
@@ -43,8 +45,11 @@ Tileset::Tileset(QString name, int tileWidth, int tileHeight,
     mTileHeight(tileHeight),
     mTileSpacing(tileSpacing),
     mMargin(margin),
+    mOrientation(Orthogonal),
+    mGridSize(tileWidth, tileHeight),
     mColumnCount(0),
     mExpectedColumnCount(0),
+    mExpectedRowCount(0),
     mNextTileId(0),
     mTerrainDistancesDirty(false),
     mLoaded(true)
@@ -59,6 +64,16 @@ Tileset::~Tileset()
     qDeleteAll(mTerrainTypes);
 }
 
+void Tileset::setFormat(TilesetFormat *format)
+{
+    mFormat = format;
+}
+
+TilesetFormat *Tileset::format() const
+{
+    return mFormat;
+}
+
 /**
  * Sets the tile size of this tileset. Affects how image is cut in loadImage.
  *
@@ -104,6 +119,17 @@ Tile *Tileset::findOrCreateTile(int id)
     return mTiles[id] = new Tile(id, this);
 }
 
+/**
+ * Returns the number of tile rows in the tileset image.
+ */
+int Tileset::rowCount() const
+{
+    if (isCollection())
+        return 1;
+
+    return rowCountForHeight(mImageReference.size.height());
+}
+
 /**
  * Sets the transparent color. Pixels with this color will be masked out
  * when loadFromImage() is called.
@@ -124,6 +150,7 @@ void Tileset::setImageReference(const ImageReference &reference)
 {
     mImageReference = reference;
     mExpectedColumnCount = columnCountForWidth(mImageReference.size.width());
+    mExpectedRowCount = rowCountForHeight(mImageReference.size.height());
 }
 
 /**
@@ -284,6 +311,18 @@ int Tileset::columnCountForWidth(int width) const
     return (width - mMargin + mTileSpacing) / (mTileWidth + mTileSpacing);
 }
 
+/**
+ * Returns the row count that this tileset would have if the tileset
+ * image would have the given \a width. This takes into account the tile
+ * size, margin and spacing.
+ */
+int Tileset::rowCountForHeight(int height) const
+{
+    if (mTileHeight <= 0)
+        return 0;
+    return (height - mMargin + mTileSpacing) / (mTileHeight + mTileSpacing);
+}
+
 /**
  * Adds a new terrain type.
  *
@@ -557,6 +596,82 @@ void Tileset::setTileImage(Tile *tile,
     }
 }
 
+void Tileset::swap(Tileset &other)
+{
+    const Properties p = properties();
+    setProperties(other.properties());
+    other.setProperties(p);
+
+    std::swap(mFileName, other.mFileName);
+    std::swap(mImageReference, other.mImageReference);
+    std::swap(mTileWidth, other.mTileWidth);
+    std::swap(mTileHeight, other.mTileHeight);
+    std::swap(mTileSpacing, other.mTileSpacing);
+    std::swap(mMargin, other.mMargin);
+    std::swap(mTileOffset, other.mTileOffset);
+    std::swap(mOrientation, other.mOrientation);
+    std::swap(mGridSize, other.mGridSize);
+    std::swap(mColumnCount, other.mColumnCount);
+    std::swap(mExpectedColumnCount, other.mExpectedColumnCount);
+    std::swap(mExpectedRowCount, other.mExpectedRowCount);
+    std::swap(mTiles, other.mTiles);
+    std::swap(mNextTileId, other.mNextTileId);
+    std::swap(mTerrainTypes, other.mTerrainTypes);
+    std::swap(mTerrainDistancesDirty, other.mTerrainDistancesDirty);
+    std::swap(mLoaded, other.mLoaded);
+    std::swap(mBackgroundColor, other.mBackgroundColor);
+    std::swap(mFormat, other.mFormat);
+
+    // Don't swap mWeakPointer, since it's a reference to this.
+
+    // Update back references from tiles and terrains
+    for (auto tile : mTiles)
+        tile->mTileset = this;
+    for (auto terrain : mTerrainTypes)
+        terrain->mTileset = this;
+
+    for (auto tile : other.mTiles)
+        tile->mTileset = &other;
+    for (auto terrain : other.mTerrainTypes)
+        terrain->mTileset = &other;
+}
+
+SharedTileset Tileset::clone() const
+{
+    SharedTileset c = create(mName, mTileWidth, mTileHeight, mTileSpacing, mMargin);
+    c->setProperties(properties());
+
+    // mFileName stays empty
+    c->mImageReference = mImageReference;
+    c->mTileOffset = mTileOffset;
+    c->mOrientation = mOrientation;
+    c->mGridSize = mGridSize;
+    c->mColumnCount = mColumnCount;
+    c->mExpectedColumnCount = mExpectedColumnCount;
+    c->mExpectedRowCount = mExpectedRowCount;
+    c->mNextTileId = mNextTileId;
+    c->mTerrainDistancesDirty = mTerrainDistancesDirty;
+    c->mLoaded = mLoaded;
+    c->mBackgroundColor = mBackgroundColor;
+    c->mFormat = mFormat;
+
+    QMapIterator<int, Tile*> tileIterator(mTiles);
+    while (tileIterator.hasNext()) {
+        tileIterator.next();
+
+        const int id = tileIterator.key();
+        const Tile *tile = tileIterator.value();
+
+        c->mTiles.insert(id, tile->clone(c.data()));
+    }
+
+    c->mTerrainTypes.reserve(mTerrainTypes.size());
+    for (Terrain *terrain : mTerrainTypes)
+        c->mTerrainTypes.append(terrain->clone(c.data()));
+
+    return c;
+}
+
 /**
  * Sets tile size to the maximum size.
  */
@@ -574,3 +689,25 @@ void Tileset::updateTileSize()
     mTileWidth = maxWidth;
     mTileHeight = maxHeight;
 }
+
+
+QString Tileset::orientationToString(Tileset::Orientation orientation)
+{
+    switch (orientation) {
+    default:
+    case Tileset::Orthogonal:
+        return QLatin1String("orthogonal");
+        break;
+    case Tileset::Isometric:
+        return QLatin1String("isometric");
+        break;
+    }
+}
+
+Tileset::Orientation Tileset::orientationFromString(const QString &string)
+{
+    Orientation orientation = Orthogonal;
+    if (string == QLatin1String("isometric"))
+        orientation = Isometric;
+    return orientation;
+}
