@@ -193,7 +193,7 @@ SharedTileset VariantToMapConverter::toTileset(const QVariant &variant)
     const QVariantMap grid = variantMap[QLatin1String("grid")].toMap();
     const int tileOffsetX = tileOffset[QLatin1String("x")].toInt();
     const int tileOffsetY = tileOffset[QLatin1String("y")].toInt();
-    const int columns = tileOffset[QLatin1String("columns")].toInt();
+    const int columns = variantMap[QLatin1String("columns")].toInt();
     const QString bgColor = variantMap[QLatin1String("backgroundcolor")].toString();
 
     if (tileWidth <= 0 || tileHeight <= 0 ||
@@ -287,8 +287,12 @@ SharedTileset VariantToMapConverter::toTileset(const QVariant &variant)
         }
 
         QVariantMap objectGroupVariant = tileVar[QLatin1String("objectgroup")].toMap();
-        if (!objectGroupVariant.isEmpty())
-            tile->setObjectGroup(toObjectGroup(objectGroupVariant));
+        if (!objectGroupVariant.isEmpty()) {
+            ObjectGroup *objectGroup = toObjectGroup(objectGroupVariant);
+            if (objectGroup)
+                objectGroup->setProperties(extractProperties(objectGroupVariant));
+            tile->setObjectGroup(objectGroup);
+        }
 
         QVariantList frameList = tileVar[QLatin1String("animation")].toList();
         if (!frameList.isEmpty()) {
