@@ -19,8 +19,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef DEFOLDPLUGIN_H
-#define DEFOLDPLUGIN_H
+#pragma once
 
 #include "defoldplugin_global.h"
 #include "tiled.h"
@@ -39,8 +38,10 @@ public:
 
     bool write(const Tiled::Map *map, const QString &fileName) override;
     QString errorString() const override;
+    QString shortName() const override;
     QStringList outputFiles(const Tiled::Map *, const QString &fileName) const override;
 
+
 protected:
     QString nameFilter() const override;
 
@@ -49,5 +50,3 @@ private:
 };
 
 } // namespace Defold
-
-#endif
