@@ -22,10 +22,12 @@
 
 #include "luatablewriter.h"
 
+#include "grouplayer.h"
 #include "imagelayer.h"
 #include "mapobject.h"
 #include "objectgroup.h"
 #include "properties.h"
+#include "savefile.h"
 #include "terrain.h"
 #include "tile.h"
 #include "tilelayer.h"
@@ -33,7 +35,6 @@
 
 #include <QFile>
 #include <QCoreApplication>
-#include <QSaveFile>
 
 /**
  * See below for an explanation of the different formats. One of these needs
@@ -43,16 +44,17 @@
 //#define POLYGON_FORMAT_PAIRS
 //#define POLYGON_FORMAT_OPTIMAL
 
-using namespace Lua;
 using namespace Tiled;
 
+namespace Lua {
+
 LuaPlugin::LuaPlugin()
 {
 }
 
 bool LuaPlugin::write(const Map *map, const QString &fileName)
 {
-    QSaveFile file(fileName);
+    SaveFile file(fileName);
 
     if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
         mError = tr("Could not open file for writing.");
@@ -61,12 +63,12 @@ bool LuaPlugin::write(const Map *map, const QString &fileName)
 
     mMapDir = QFileInfo(fileName).path();
 
-    LuaTableWriter writer(&file);
+    LuaTableWriter writer(file.device());
     writer.writeStartDocument();
     writeMap(writer, map);
     writer.writeEndDocument();
 
-    if (file.error() != QFile::NoError) {
+    if (file.error() != QFileDevice::NoError) {
         mError = file.errorString();
         return false;
     }
@@ -84,11 +86,32 @@ QString LuaPlugin::nameFilter() const
     return tr("Lua files (*.lua)");
 }
 
+QString LuaPlugin::shortName() const
+{
+    return QLatin1String("lua");
+}
+
 QString LuaPlugin::errorString() const
 {
     return mError;
 }
 
+static void writeColor(LuaTableWriter &writer,
+                       const char *name,
+                       const QColor &color)
+{
+    // Example: backgroundcolor = { 255, 200, 100 }
+    writer.writeStartTable(name);
+    writer.setSuppressNewlines(true);
+    writer.writeValue(color.red());
+    writer.writeValue(color.green());
+    writer.writeValue(color.blue());
+    if (color.alpha() != 255)
+        writer.writeValue(color.alpha());
+    writer.writeEndTable();
+    writer.setSuppressNewlines(false);
+}
+
 void LuaPlugin::writeMap(LuaTableWriter &writer, const Map *map)
 {
     writer.writeStartReturnTable();
@@ -119,18 +142,8 @@ void LuaPlugin::writeMap(LuaTableWriter &writer, const Map *map)
     }
 
     const QColor &backgroundColor = map->backgroundColor();
-    if (backgroundColor.isValid()) {
-        // Example: backgroundcolor = { 255, 200, 100 }
-        writer.writeStartTable("backgroundcolor");
-        writer.setSuppressNewlines(true);
-        writer.writeValue(backgroundColor.red());
-        writer.writeValue(backgroundColor.green());
-        writer.writeValue(backgroundColor.blue());
-        if (backgroundColor.alpha() != 255)
-            writer.writeValue(backgroundColor.alpha());
-        writer.writeEndTable();
-        writer.setSuppressNewlines(false);
-    }
+    if (backgroundColor.isValid())
+        writeColor(writer, "backgroundcolor", backgroundColor);
 
     writeProperties(writer, map->properties());
 
@@ -145,23 +158,7 @@ void LuaPlugin::writeMap(LuaTableWriter &writer, const Map *map)
     }
     writer.writeEndTable();
 
-    writer.writeStartTable("layers");
-    for (const Layer *layer : map->layers()) {
-        switch (layer->layerType()) {
-        case Layer::TileLayerType:
-            writeTileLayer(writer,
-                           static_cast<const TileLayer*>(layer),
-                           map->layerDataFormat());
-            break;
-        case Layer::ObjectGroupType:
-            writeObjectGroup(writer, static_cast<const ObjectGroup*>(layer));
-            break;
-        case Layer::ImageLayerType:
-            writeImageLayer(writer, static_cast<const ImageLayer*>(layer));
-            break;
-        }
-    }
-    writer.writeEndTable();
+    writeLayers(writer, map->layers(), map->layerDataFormat());
 
     writer.writeEndTable();
 }
@@ -187,6 +184,8 @@ void LuaPlugin::writeProperties(LuaTableWriter &writer,
 
 static bool includeTile(const Tile *tile)
 {
+    if (!tile->type().isEmpty())
+        return true;
     if (!tile->properties().isEmpty())
         return true;
     if (!tile->imageSource().isEmpty())
@@ -236,12 +235,23 @@ void LuaPlugin::writeTileset(LuaTableWriter &writer, const Tileset *tileset,
                                 tileset->transparentColor().name());
     }
 
+    const QColor &backgroundColor = tileset->backgroundColor();
+    if (backgroundColor.isValid())
+        writeColor(writer, "backgroundcolor", backgroundColor);
+
     const QPoint offset = tileset->tileOffset();
     writer.writeStartTable("tileoffset");
     writer.writeKeyAndValue("x", offset.x());
     writer.writeKeyAndValue("y", offset.y());
     writer.writeEndTable();
 
+    const QSize gridSize = tileset->gridSize();
+    writer.writeStartTable("grid");
+    writer.writeKeyAndValue("orientation", Tileset::orientationToString(tileset->orientation()));
+    writer.writeKeyAndValue("width", gridSize.width());
+    writer.writeKeyAndValue("height", gridSize.height());
+    writer.writeEndTable();
+
     writeProperties(writer, tileset->properties());
 
     writer.writeStartTable("terrains");
@@ -268,6 +278,9 @@ void LuaPlugin::writeTileset(LuaTableWriter &writer, const Tileset *tileset,
         writer.writeStartTable();
         writer.writeKeyAndValue("id", tile->id());
 
+        if (!tile->type().isEmpty())
+            writer.writeKeyAndValue("type", tile->type());
+
         if (!tile->properties().isEmpty())
             writeProperties(writer, tile->properties());
 
@@ -303,8 +316,8 @@ void LuaPlugin::writeTileset(LuaTableWriter &writer, const Tileset *tileset,
             writer.writeStartTable("animation");
             for (const Frame &frame : frames) {
                 writer.writeStartTable();
-                writer.writeKeyAndValue("tileid", QString::number(frame.tileId));
-                writer.writeKeyAndValue("duration", QString::number(frame.duration));
+                writer.writeKeyAndValue("tileid", frame.tileId);
+                writer.writeKeyAndValue("duration", frame.duration);
                 writer.writeEndTable();
             }
             writer.writeEndTable(); // animation
@@ -317,6 +330,30 @@ void LuaPlugin::writeTileset(LuaTableWriter &writer, const Tileset *tileset,
     writer.writeEndTable(); // tileset
 }
 
+void LuaPlugin::writeLayers(LuaTableWriter &writer,
+                            const QList<Layer *> &layers,
+                            Map::LayerDataFormat format)
+{
+    writer.writeStartTable("layers");
+    for (const Layer *layer : layers) {
+        switch (layer->layerType()) {
+        case Layer::TileLayerType:
+            writeTileLayer(writer, static_cast<const TileLayer*>(layer), format);
+            break;
+        case Layer::ObjectGroupType:
+            writeObjectGroup(writer, static_cast<const ObjectGroup*>(layer));
+            break;
+        case Layer::ImageLayerType:
+            writeImageLayer(writer, static_cast<const ImageLayer*>(layer));
+            break;
+        case Layer::GroupLayerType:
+            writeGroupLayer(writer, static_cast<const GroupLayer*>(layer), format);
+            break;
+        }
+    }
+    writer.writeEndTable();
+}
+
 void LuaPlugin::writeTileLayer(LuaTableWriter &writer,
                                const TileLayer *tileLayer,
                                Map::LayerDataFormat format)
@@ -429,6 +466,28 @@ void LuaPlugin::writeImageLayer(LuaTableWriter &writer,
     writer.writeEndTable();
 }
 
+void LuaPlugin::writeGroupLayer(LuaTableWriter &writer,
+                                const GroupLayer *groupLayer,
+                                Map::LayerDataFormat format)
+{
+    writer.writeStartTable();
+
+    writer.writeKeyAndValue("type", "group");
+    writer.writeKeyAndValue("name", groupLayer->name());
+    writer.writeKeyAndValue("visible", groupLayer->isVisible());
+    writer.writeKeyAndValue("opacity", groupLayer->opacity());
+
+    const QPointF offset = groupLayer->offset();
+    writer.writeKeyAndValue("offsetx", offset.x());
+    writer.writeKeyAndValue("offsety", offset.y());
+
+    writeProperties(writer, groupLayer->properties());
+
+    writeLayers(writer, groupLayer->layers(), format);
+
+    writer.writeEndTable();
+}
+
 static const char *toString(MapObject::Shape shape)
 {
     switch (shape) {
@@ -440,6 +499,8 @@ static const char *toString(MapObject::Shape shape)
         return "polyline";
     case MapObject::Ellipse:
         return "ellipse";
+    case MapObject::Text:
+        return "text";
     }
     return "unknown";
 }
@@ -464,82 +525,137 @@ void LuaPlugin::writeMapObject(LuaTableWriter &writer,
 
     writer.writeKeyAndValue("visible", mapObject->isVisible());
 
-    const QPolygonF &polygon = mapObject->polygon();
-    if (!polygon.isEmpty()) {
-        if (mapObject->shape() == MapObject::Polygon)
-            writer.writeStartTable("polygon");
-        else
-            writer.writeStartTable("polyline");
-
-#if defined(POLYGON_FORMAT_FULL)
-        /* This format is the easiest to read and understand:
-         *
-         *  {
-         *    { x = 1, y = 1 },
-         *    { x = 2, y = 2 },
-         *    { x = 3, y = 3 },
-         *    ...
-         *  }
-         */
-        for (const QPointF &point : polygon) {
-            writer.writeStartTable();
-            writer.setSuppressNewlines(true);
-
-            writer.writeKeyAndValue("x", point.x());
-            writer.writeKeyAndValue("y", point.y());
-
-            writer.writeEndTable();
-            writer.setSuppressNewlines(false);
-        }
-#elif defined(POLYGON_FORMAT_PAIRS)
-        /* This is an alternative that takes about 25% less memory.
-         *
-         *  {
-         *    { 1, 1 },
-         *    { 2, 2 },
-         *    { 3, 3 },
-         *    ...
-         *  }
-         */
-        for (const QPointF &point : polygon) {
-            writer.writeStartTable();
-            writer.setSuppressNewlines(true);
-
-            writer.writeValue(point.x());
-            writer.writeValue(point.y());
-
-            writer.writeEndTable();
-            writer.setSuppressNewlines(false);
-        }
-#elif defined(POLYGON_FORMAT_OPTIMAL)
-        /* Writing it out in two tables, one for the x coordinates and one for
-         * the y coordinates. It is a compromise between code readability and
-         * performance. This takes the least amount of memory (60% less than
-         * the first approach).
-         *
-         * x = { 1, 2, 3, ... }
-         * y = { 1, 2, 3, ... }
-         */
-
-        writer.writeStartTable("x");
-        writer.setSuppressNewlines(true);
-        for (const QPointF &point : polygon)
-            writer.writeValue(point.x());
-        writer.writeEndTable();
-        writer.setSuppressNewlines(false);
-
-        writer.writeStartTable("y");
-        writer.setSuppressNewlines(true);
-        for (const QPointF &point : polygon)
-            writer.writeValue(point.y());
-        writer.writeEndTable();
-        writer.setSuppressNewlines(false);
-#endif
-
-        writer.writeEndTable();
+    switch (mapObject->shape()) {
+    case MapObject::Rectangle:
+    case MapObject::Ellipse:
+        break;
+    case MapObject::Polygon:
+    case MapObject::Polyline:
+        writePolygon(writer, mapObject);
+        break;
+    case MapObject::Text:
+        writeTextProperties(writer, mapObject);
+        break;
     }
 
     writeProperties(writer, mapObject->properties());
 
     writer.writeEndTable();
 }
+
+void LuaPlugin::writePolygon(LuaTableWriter &writer, const MapObject *mapObject)
+{
+    if (mapObject->shape() == MapObject::Polygon)
+        writer.writeStartTable("polygon");
+    else
+        writer.writeStartTable("polyline");
+
+#if defined(POLYGON_FORMAT_FULL)
+    /* This format is the easiest to read and understand:
+     *
+     *  {
+     *    { x = 1, y = 1 },
+     *    { x = 2, y = 2 },
+     *    { x = 3, y = 3 },
+     *    ...
+     *  }
+     */
+    for (const QPointF &point : mapObject->polygon()) {
+        writer.writeStartTable();
+        writer.setSuppressNewlines(true);
+
+        writer.writeKeyAndValue("x", point.x());
+        writer.writeKeyAndValue("y", point.y());
+
+        writer.writeEndTable();
+        writer.setSuppressNewlines(false);
+    }
+#elif defined(POLYGON_FORMAT_PAIRS)
+    /* This is an alternative that takes about 25% less memory.
+     *
+     *  {
+     *    { 1, 1 },
+     *    { 2, 2 },
+     *    { 3, 3 },
+     *    ...
+     *  }
+     */
+    for (const QPointF &point : mapObject->polygon()) {
+        writer.writeStartTable();
+        writer.setSuppressNewlines(true);
+
+        writer.writeValue(point.x());
+        writer.writeValue(point.y());
+
+        writer.writeEndTable();
+        writer.setSuppressNewlines(false);
+    }
+#elif defined(POLYGON_FORMAT_OPTIMAL)
+    /* Writing it out in two tables, one for the x coordinates and one for
+     * the y coordinates. It is a compromise between code readability and
+     * performance. This takes the least amount of memory (60% less than
+     * the first approach).
+     *
+     * x = { 1, 2, 3, ... }
+     * y = { 1, 2, 3, ... }
+     */
+
+    writer.writeStartTable("x");
+    writer.setSuppressNewlines(true);
+    for (const QPointF &point : mapObject->polygon())
+        writer.writeValue(point.x());
+    writer.writeEndTable();
+    writer.setSuppressNewlines(false);
+
+    writer.writeStartTable("y");
+    writer.setSuppressNewlines(true);
+    for (const QPointF &point : mapObject->polygon())
+        writer.writeValue(point.y());
+    writer.writeEndTable();
+    writer.setSuppressNewlines(false);
+#endif
+
+    writer.writeEndTable();
+}
+
+void LuaPlugin::writeTextProperties(LuaTableWriter &writer, const MapObject *mapObject)
+{
+    const TextData &textData = mapObject->textData();
+
+    writer.writeKeyAndValue("text", textData.text);
+
+    if (textData.font.family() != QLatin1String("sans-serif"))
+        writer.writeKeyAndValue("fontfamily", textData.font.family());
+    if (textData.font.pixelSize() >= 0 && textData.font.pixelSize() != 16)
+        writer.writeKeyAndValue("pixelsize", textData.font.pixelSize());
+    if (textData.wordWrap)
+        writer.writeKeyAndValue("wrap", textData.wordWrap);
+    if (textData.color != Qt::black)
+        writeColor(writer, "color", textData.color);
+    if (textData.font.bold())
+        writer.writeKeyAndValue("bold", textData.font.bold());
+    if (textData.font.italic())
+        writer.writeKeyAndValue("italic", textData.font.italic());
+    if (textData.font.underline())
+        writer.writeKeyAndValue("underline", textData.font.underline());
+    if (textData.font.strikeOut())
+        writer.writeKeyAndValue("strikeout", textData.font.strikeOut());
+    if (!textData.font.kerning())
+        writer.writeKeyAndValue("kerning", textData.font.kerning());
+
+    if (!textData.alignment.testFlag(Qt::AlignLeft)) {
+        if (textData.alignment.testFlag(Qt::AlignHCenter))
+            writer.writeKeyAndValue("halign", "center");
+        else if (textData.alignment.testFlag(Qt::AlignRight))
+            writer.writeKeyAndValue("halign", "right");
+    }
+
+    if (!textData.alignment.testFlag(Qt::AlignTop)) {
+        if (textData.alignment.testFlag(Qt::AlignVCenter))
+            writer.writeKeyAndValue("valign", "center");
+        else if (textData.alignment.testFlag(Qt::AlignBottom))
+            writer.writeKeyAndValue("valign", "bottom");
+    }
+}
+
+} // namespace Lua
