@@ -22,6 +22,7 @@
 #include "replicaislandplugin.h"
 
 #include "map.h"
+#include "savefile.h"
 #include "tile.h"
 #include "tileset.h"
 #include "tilelayer.h"
@@ -29,7 +30,6 @@
 
 #include <QtEndian>
 #include <QFile>
-#include <QSaveFile>
 
 using namespace ReplicaIsland;
 
@@ -207,6 +207,11 @@ QString ReplicaIslandPlugin::nameFilter() const
     return tr("Replica Island map files (*.bin)");
 }
 
+QString ReplicaIslandPlugin::shortName() const
+{
+    return QLatin1String("replicaisland");
+}
+
 bool ReplicaIslandPlugin::supportsFile(const QString &fileName) const
 {
     // Check the file extension first.
@@ -234,14 +239,14 @@ bool ReplicaIslandPlugin::write(const Tiled::Map *map, const QString &fileName)
     using namespace Tiled;
 
     // Open up a temporary file for saving the level.
-    QSaveFile file(fileName);
+    SaveFile file(fileName);
     if (!file.open(QIODevice::WriteOnly)) {
         mError = tr("Could not open file for writing.");
         return false;
     }
 
     // Create an output stream for serializing data.
-    QDataStream out(&file);
+    QDataStream out(file.device());
     out.setByteOrder(QDataStream::LittleEndian);
     out.setFloatingPointPrecision(QDataStream::SinglePrecision);
 
@@ -304,7 +309,7 @@ bool ReplicaIslandPlugin::writeLayer(QDataStream &out, Tiled::TileLayer *layer)
     // correct tileset for this layer.
     for (int y = 0; y < layer->height(); y++) {
         for (int x = 0; x < layer->width(); x++) {
-            Tile *tile = layer->cellAt(x, y).tile;
+            Tile *tile = layer->cellAt(x, y).tile();
             if (tile)
                 out << static_cast<quint8>(tile->id());
             else
