@@ -51,6 +51,32 @@
 #include <QFocusEvent>
 #include <QStyle>
 #include <QPalette>
+#include <QScreen>
+
+static qreal defaultDpiScale()
+{
+    if (const QScreen *screen = QGuiApplication::primaryScreen())
+        return screen->logicalDotsPerInchX() / 96.0;
+    return 1.0;
+}
+
+static qreal dpiScaled(qreal value)
+{
+#ifdef Q_OS_MAC
+    // On mac the DPI is always 72 so we should not scale it
+    return value;
+#else
+    static const qreal scale = defaultDpiScale();
+    return value * scale;
+#endif
+}
+
+static QSize dpiScaled(QSize value)
+{
+    return QSize(qRound(dpiScaled(value.width())),
+                 qRound(dpiScaled(value.height())));
+}
+
 
 #if QT_VERSION >= 0x040400
 QT_BEGIN_NAMESPACE
@@ -92,10 +118,14 @@ public:
     void setCurrentItem(QtBrowserItem *browserItem, bool block);
     void editItem(QtBrowserItem *browserItem);
 
+    QList<QtBrowserItem*> selectedItems() const;
+
     void slotCurrentBrowserItemChanged(QtBrowserItem *item);
     void slotCurrentTreeItemChanged(QTreeWidgetItem *newItem, QTreeWidgetItem *);
+    void slotItemSelectionChanged();
 
     QTreeWidgetItem *editedItem() const;
+    QtBrowserItem *editedBrowserItem() const;
 
 private:
     void updateItem(QTreeWidgetItem *item);
@@ -108,6 +138,7 @@ private:
     QtPropertyEditorView *m_treeWidget;
 
     bool m_headerVisible;
+    bool m_allowMultiSelection;
     QtTreePropertyBrowser::ResizeMode m_resizeMode;
     class QtPropertyEditorDelegate *m_delegate;
     bool m_markPropertiesWithoutValue;
@@ -409,7 +440,7 @@ void QtPropertyEditorDelegate::drawDisplay(QPainter *painter, const QStyleOption
 QSize QtPropertyEditorDelegate::sizeHint(const QStyleOptionViewItem &option,
             const QModelIndex &index) const
 {
-    return QItemDelegate::sizeHint(option, index) + QSize(3, 4);
+    return QItemDelegate::sizeHint(option, index) + dpiScaled(QSize(3, 4));
 }
 
 bool QtPropertyEditorDelegate::eventFilter(QObject *object, QEvent *event)
@@ -426,6 +457,7 @@ bool QtPropertyEditorDelegate::eventFilter(QObject *object, QEvent *event)
 QtTreePropertyBrowserPrivate::QtTreePropertyBrowserPrivate() :
     m_treeWidget(0),
     m_headerVisible(true),
+    m_allowMultiSelection(false),
     m_resizeMode(QtTreePropertyBrowser::Stretch),
     m_delegate(0),
     m_markPropertiesWithoutValue(false),
@@ -496,6 +528,7 @@ void QtTreePropertyBrowserPrivate::init(QWidget *parent)
     QObject::connect(m_treeWidget, SIGNAL(collapsed(const QModelIndex &)), q_ptr, SLOT(slotCollapsed(const QModelIndex &)));
     QObject::connect(m_treeWidget, SIGNAL(expanded(const QModelIndex &)), q_ptr, SLOT(slotExpanded(const QModelIndex &)));
     QObject::connect(m_treeWidget, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), q_ptr, SLOT(slotCurrentTreeItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)));
+    QObject::connect(m_treeWidget, SIGNAL(itemSelectionChanged()), q_ptr, SLOT(slotItemSelectionChanged()));
 }
 
 QtBrowserItem *QtTreePropertyBrowserPrivate::currentItem() const
@@ -710,11 +743,21 @@ void QtTreePropertyBrowserPrivate::slotCurrentTreeItemChanged(QTreeWidgetItem *n
     m_browserChangedBlocked = false;
 }
 
+void QtTreePropertyBrowserPrivate::slotItemSelectionChanged()
+{
+    emit q_ptr->selectedItemsChanged();
+}
+
 QTreeWidgetItem *QtTreePropertyBrowserPrivate::editedItem() const
 {
     return m_delegate->editedItem();
 }
 
+QtBrowserItem *QtTreePropertyBrowserPrivate::editedBrowserItem() const
+{
+    return m_itemToIndex.value(editedItem());
+}
+
 void QtTreePropertyBrowserPrivate::editItem(QtBrowserItem *browserItem)
 {
     if (QTreeWidgetItem *treeItem = m_indexToItem.value(browserItem, 0)) {
@@ -723,6 +766,18 @@ void QtTreePropertyBrowserPrivate::editItem(QtBrowserItem *browserItem)
     }
 }
 
+QList<QtBrowserItem *> QtTreePropertyBrowserPrivate::selectedItems() const
+{
+    const QList<QTreeWidgetItem*> selectedTreeItems = m_treeWidget->selectedItems();
+    QList<QtBrowserItem*> browserItems;
+
+    for (QTreeWidgetItem *treeItem : selectedTreeItems)
+        if (QtBrowserItem *browserItem = m_itemToIndex.value(treeItem, 0))
+            browserItems.append(browserItem);
+
+    return browserItems;
+}
+
 /*!
     \class QtTreePropertyBrowser
 
@@ -862,6 +917,25 @@ void QtTreePropertyBrowser::setHeaderVisible(bool visible)
     d_ptr->m_treeWidget->header()->setVisible(visible);
 }
 
+/*!
+  \property QtTreePropertyBrowser::allowMultiSelection
+  \brief whether to allow multiple selection items.
+*/
+bool QtTreePropertyBrowser::allowMultiSelection() const
+{
+    return d_ptr->m_allowMultiSelection;
+}
+
+void QtTreePropertyBrowser::setAllowMultiSelection(bool multiSelection)
+{
+    if (d_ptr->m_allowMultiSelection == multiSelection)
+        return;
+
+    d_ptr->m_allowMultiSelection = multiSelection;
+    d_ptr->m_treeWidget->setSelectionMode(multiSelection ? QAbstractItemView::ExtendedSelection
+                                                         : QAbstractItemView::SingleSelection);
+}
+
 /*!
   \enum QtTreePropertyBrowser::ResizeMode
 
@@ -1086,6 +1160,24 @@ void QtTreePropertyBrowser::editItem(QtBrowserItem *item)
     d_ptr->editItem(item);
 }
 
+/*!
+    Returns the current item being edited, if any.
+*/
+QtBrowserItem *QtTreePropertyBrowser::editedItem() const
+{
+    return d_ptr->editedBrowserItem();
+}
+
+/*!
+    Returns the selected items.
+
+    \sa setAllowMultiSelection()
+*/
+QList<QtBrowserItem *> QtTreePropertyBrowser::selectedItems() const
+{
+    return d_ptr->selectedItems();
+}
+
 #if QT_VERSION >= 0x040400
 QT_END_NAMESPACE
 #endif
