@@ -19,8 +19,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef ABOUTDIALOG_H
-#define ABOUTDIALOG_H
+#pragma once
 
 #include <QDialog>
 #include "ui_aboutdialog.h"
@@ -41,5 +40,3 @@ private slots:
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // ABOUTDIALOG_H
