@@ -18,8 +18,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef ABSTRACTOBJECTTOOL_H
-#define ABSTRACTOBJECTTOOL_H
+#pragma once
 
 #include "abstracttool.h"
 
@@ -66,11 +65,13 @@ protected:
 
     MapScene *mapScene() const { return mMapScene; }
     ObjectGroup *currentObjectGroup() const;
+    QList<MapObjectItem*> objectItemsAt(QPointF pos) const;
     MapObjectItem *topMostObjectItemAt(QPointF pos) const;
 
 private slots:
     void duplicateObjects();
     void removeObjects();
+    void resetTileSize();
 
     void flipHorizontally();
     void flipVertically();
@@ -89,5 +90,3 @@ private:
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // ABSTRACTOBJECTTOOL_H
