@@ -64,6 +64,9 @@ AutoMapper::AutoMapper(MapDocument *workingDocument, Map *rules,
 {
     Q_ASSERT(mMapRules);
 
+    TilesetManager *tilesetManager = TilesetManager::instance();
+    tilesetManager->addReferences(rules->tilesets());
+
     if (!setupRuleMapProperties())
         return;
 
@@ -91,30 +94,37 @@ bool AutoMapper::ruleLayerNameUsed(QString ruleLayerName) const
 
 bool AutoMapper::setupRuleMapProperties()
 {
-    Properties properties = mMapRules->properties();
-    foreach (QString key, properties.keys()) {
-        QVariant value = properties.value(key);
+    QMapIterator<QString,QVariant> it(mMapRules->properties());
+    while (it.hasNext()) {
+        it.next();
+
+        const QString &name = it.key();
+        const QVariant &value = it.value();
+
         bool raiseWarning = true;
-        if (key.toLower() == QLatin1String("deletetiles")) {
+
+        if (name.compare(QLatin1String("deletetiles"), Qt::CaseInsensitive) == 0) {
             if (value.canConvert(QVariant::Bool)) {
                 mDeleteTiles = value.toBool();
                 raiseWarning = false;
             }
-        } else if (key.toLower() == QLatin1String("automappingradius")) {
+        } else if (name.compare(QLatin1String("automappingradius"), Qt::CaseInsensitive) == 0) {
             if (value.canConvert(QVariant::Int)) {
                 mAutoMappingRadius = value.toInt();
                 raiseWarning = false;
             }
-        } else if (key.toLower() == QLatin1String("nooverlappingrules")) {
+        } else if (name.compare(QLatin1String("nooverlappingrules"), Qt::CaseInsensitive) == 0) {
             if (value.canConvert(QVariant::Bool)) {
                 mNoOverlappingRules = value.toBool();
                 raiseWarning = false;
             }
         }
-        if (raiseWarning)
+
+        if (raiseWarning) {
             mWarning += tr("'%1': Property '%2' = '%3' does not make sense. "
                            "Ignoring this property.")
-                    .arg(mRulePath, key, value.toString()) + QLatin1Char('\n');
+                    .arg(mRulePath, name, value.toString()) + QLatin1Char('\n');
+        }
     }
     return true;
 }
@@ -127,51 +137,56 @@ bool AutoMapper::setupRuleMapTileLayers()
     Q_ASSERT(!mLayerOutputRegions);
     Q_ASSERT(mInputRules.isEmpty());
     Q_ASSERT(mInputRules.names.isEmpty());
-    Q_ASSERT(mInputRules.indexes.isEmpty());
 
     QString error;
 
-    foreach (Layer *layer, mMapRules->layers()) {
+    for (Layer *layer : mMapRules->layers()) {
         const QString layerName = layer->name();
 
-        if (layerName.startsWith(QLatin1String("regions"),
-                                 Qt::CaseInsensitive)) {
-            bool treatAsBoth = layerName.toLower() == QLatin1String("regions");
-            if (layerName.endsWith(QLatin1String("input"),
-                                 Qt::CaseInsensitive) || treatAsBoth) {
+        if (layerName.startsWith(QLatin1String("regions"), Qt::CaseInsensitive)) {
+            bool inputAndOutput = layerName.compare(QLatin1String("regions"), Qt::CaseInsensitive) == 0;
+
+            if (inputAndOutput || layerName.endsWith(QLatin1String("input"), Qt::CaseInsensitive)) {
                 if (mLayerInputRegions) {
-                    error += tr("'regions_input' layer must not occur more than once.")
-                            + QLatin1Char('\n');
+                    error += tr("'regions_input' layer must not occur more than once.");
+                    error += QLatin1Char('\n');
                 }
-                if (layer->isTileLayer()) {
-                    mLayerInputRegions = layer->asTileLayer();
+                if (TileLayer *tileLayer = layer->asTileLayer()) {
+                    mLayerInputRegions = tileLayer;
                 } else {
-                    error += tr("'regions_*' layers must be tile layers.")
-                            + QLatin1Char('\n');
+                    error += tr("'regions_*' layers must be tile layers.");
+                    error += QLatin1Char('\n');
                 }
             }
-            if (layerName.endsWith(QLatin1String("output"),
-                                 Qt::CaseInsensitive) || treatAsBoth) {
+
+            if (inputAndOutput || layerName.endsWith(QLatin1String("output"), Qt::CaseInsensitive)) {
                 if (mLayerOutputRegions) {
-                    error += tr("'regions_output' layer must not occur more than once.")
-                            + QLatin1Char('\n');
+                    error += tr("'regions_output' layer must not occur more than once.");
+                    error += QLatin1Char('\n');
                 }
-                if (layer->isTileLayer()) {
-                    mLayerOutputRegions = layer->asTileLayer();
+                if (TileLayer *tileLayer = layer->asTileLayer()) {
+                    mLayerOutputRegions = tileLayer;
                 } else {
-                    error += tr("'regions_*' layers must be tile layers.")
-                            + QLatin1Char('\n');
+                    error += tr("'regions_*' layers must be tile layers.");
+                    error += QLatin1Char('\n');
                 }
             }
+
             continue;
         }
 
-        int nameStartPosition = layerName.indexOf(
-                                      QLatin1Char('_')) + 1;
-        // name is all characters behind the underscore (excluded)
-        QString name = layerName.right(layerName.size() - nameStartPosition);
-        // group is all before the underscore (included)
-        QString index = layerName.left(nameStartPosition);
+        int nameStartPosition = layerName.indexOf(QLatin1Char('_')) + 1;
+
+        // both 'rule' and 'output' layers will require and underscore and
+        // rely on the correct position detected of the underscore
+        if (nameStartPosition == 0) {
+            error += tr("Did you forget an underscore in layer '%1'?").arg(layerName);
+            error += QLatin1Char('\n');
+            continue;
+        }
+
+        QString name = layerName.mid(nameStartPosition);    // all characters behind the underscore (excluded)
+        QString index = layerName.left(nameStartPosition);  // all before the underscore (included)
 
         if (index.startsWith(QLatin1String("output"), Qt::CaseInsensitive))
             index.remove(0, 6);
@@ -180,67 +195,54 @@ bool AutoMapper::setupRuleMapTileLayers()
         else if (index.startsWith(QLatin1String("input"), Qt::CaseInsensitive))
             index.remove(0, 5);
 
-        // both 'rule' and 'output' layers will require and underscore and
-        // rely on the correct position detected of the underscore
-        if (nameStartPosition == 0) {
-            error += tr("Did you forget an underscore in layer '%1'?").arg(
-                        layerName) + QLatin1Char('\n');
-            continue;
-        }
-
         if (layerName.startsWith(QLatin1String("input"), Qt::CaseInsensitive)) {
+            bool isNotList = layerName.startsWith(QLatin1String("inputnot"), Qt::CaseInsensitive);
 
-            bool isNotList = layerName.startsWith(QLatin1String("inputnot"),
-                                                  Qt::CaseInsensitive);
+            TileLayer *tileLayer = layer->asTileLayer();
 
-            if (!layer->isTileLayer()) {
-                error += tr("'input_*' and 'inputnot_*' layers must be tile layers.")
-                        + QLatin1Char('\n');
+            if (!tileLayer) {
+                error += tr("'input_*' and 'inputnot_*' layers must be tile layers.");
+                error += QLatin1Char('\n');
                 continue;
             }
 
             mInputRules.names.insert(name);
 
-            if (!mInputRules.indexes.contains(index)) {
-                mInputRules.indexes.insert(index);
+            if (!mInputRules.contains(index))
                 mInputRules.insert(index, InputIndex());
-            }
 
-            if (!mInputRules[index].names.contains(name)) {
-                mInputRules[index].names.insert(name);
-                mInputRules[index].insert(name, InputIndexName());
-            }
+            if (!mInputRules[index].contains(name))
+                mInputRules[index].insert(name, InputConditions());
 
             if (isNotList)
-                mInputRules[index][name].listNo.append(layer->asTileLayer());
+                mInputRules[index][name].listNo.append(tileLayer);
             else
-                mInputRules[index][name].listYes.append(layer->asTileLayer());
+                mInputRules[index][name].listYes.append(tileLayer);
 
             continue;
         }
 
-        if (layerName.startsWith(QLatin1String("output"),
-                                 Qt::CaseInsensitive)) {
+        if (layerName.startsWith(QLatin1String("output"), Qt::CaseInsensitive)) {
             if (layer->isTileLayer())
                 mTouchedTileLayers.insert(name);
-            else
+            else if (layer->isObjectGroup())
                 mTouchedObjectGroups.insert(name);
 
             Layer::TypeFlag type = layer->layerType();
             int layerIndex = mMapWork->indexOfLayer(name, type);
 
             bool found = false;
-            foreach (RuleOutput *translationTable, mLayerList) {
-                if (translationTable->index == index) {
-                    translationTable->insert(layer, layerIndex);
+            for (RuleOutput &translationTable : mLayerList) {
+                if (translationTable.index == index) {
+                    translationTable.insert(layer, layerIndex);
                     found = true;
                     break;
                 }
             }
             if (!found) {
-                mLayerList.append(new RuleOutput());
-                mLayerList.last()->insert(layer, layerIndex);
-                mLayerList.last()->index = index;
+                mLayerList.append(RuleOutput());
+                mLayerList.last().insert(layer, layerIndex);
+                mLayerList.last().index = index;
             }
             continue;
         }
@@ -333,7 +335,7 @@ bool AutoMapper::prepareAutoMap()
     if (!setupCorrectIndexes())
         return false;
 
-    if (!setupTilesets(mMapRules, mMapWork))
+    if (!setupTilesets())
         return false;
 
     return true;
@@ -341,18 +343,19 @@ bool AutoMapper::prepareAutoMap()
 
 bool AutoMapper::setupMissingLayers()
 {
+    QUndoStack *undoStack = mMapDocument->undoStack();
+
     // make sure all needed layers are there:
     foreach (const QString &name, mTouchedTileLayers) {
         if (mMapWork->indexOfLayer(name, Layer::TileLayerType) != -1)
             continue;
 
         const int index =  mMapWork->layerCount();
-        TileLayer *tilelayer = new TileLayer(name, 0, 0,
+        TileLayer *tileLayer = new TileLayer(name, 0, 0,
                                              mMapWork->width(),
                                              mMapWork->height());
-        mMapDocument->undoStack()->push(
-                    new AddLayer(mMapDocument, index, tilelayer));
-        mAddedTileLayers.append(name);
+        undoStack->push(new AddLayer(mMapDocument, index, tileLayer, nullptr));
+        mAddedLayers.append(tileLayer);
     }
 
     foreach (const QString &name, mTouchedObjectGroups) {
@@ -360,12 +363,9 @@ bool AutoMapper::setupMissingLayers()
             continue;
 
         const int index =  mMapWork->layerCount();
-        ObjectGroup *objectGroup = new ObjectGroup(name, 0, 0,
-                                                   mMapWork->width(),
-                                                   mMapWork->height());
-        mMapDocument->undoStack()->push(
-                    new AddLayer(mMapDocument, index, objectGroup));
-        mAddedTileLayers.append(name);
+        ObjectGroup *objectGroup = new ObjectGroup(name, 0, 0);
+        undoStack->push(new AddLayer(mMapDocument, index, objectGroup, nullptr));
+        mAddedLayers.append(objectGroup);
     }
 
     return true;
@@ -373,64 +373,37 @@ bool AutoMapper::setupMissingLayers()
 
 bool AutoMapper::setupCorrectIndexes()
 {
-    // make sure all indexes of the layer translationtables are correct.
-    for (RuleOutput *translationTable : mLayerList) {
-        foreach (Layer *layerKey, translationTable->keys()) {
+    // make sure all indexes of the layer translation tables are correct.
+    for (RuleOutput &translationTable : mLayerList) {
+        foreach (Layer *layerKey, translationTable.keys()) {
             QString name = layerKey->name();
             const int pos = name.indexOf(QLatin1Char('_')) + 1;
             name = name.right(name.length() - pos);
 
-            const int index = translationTable->value(layerKey, -1);
+            const int index = translationTable.value(layerKey, -1);
             if (index >= mMapWork->layerCount() || index == -1 ||
                     name != mMapWork->layerAt(index)->name()) {
 
                 int newIndex = mMapWork->indexOfLayer(name, layerKey->layerType());
                 Q_ASSERT(newIndex != -1);
 
-                translationTable->insert(layerKey, newIndex);
+                translationTable.insert(layerKey, newIndex);
             }
         }
     }
     return true;
 }
 
-// This cannot just be replaced by MapDocument::unifyTileset(Map),
-// because here mAddedTileset is modified.
-bool AutoMapper::setupTilesets(Map *src, Map *dst)
+bool AutoMapper::setupTilesets()
 {
-    const QVector<SharedTileset> &existingTilesets = dst->tilesets();
-    TilesetManager *tilesetManager = TilesetManager::instance();
+    Q_ASSERT(mAddedTilesets.isEmpty());
 
-    // Add tilesets that are not yet part of dst map
-    for (const SharedTileset &tileset : src->tilesets()) {
-        if (existingTilesets.contains(tileset))
-            continue;
+    mMapDocument->unifyTilesets(mMapRules, mAddedTilesets);
 
-        QUndoStack *undoStack = mMapDocument->undoStack();
+    const auto &addedTilesets = mAddedTilesets;
+    for (const SharedTileset &tileset : addedTilesets)
+        mMapDocument->undoStack()->push(new AddTileset(mMapDocument, tileset));
 
-        SharedTileset replacement = tileset->findSimilarTileset(existingTilesets);
-        if (!replacement) {
-            mAddedTilesets.append(tileset);
-            undoStack->push(new AddTileset(mMapDocument, tileset));
-            continue;
-        }
-
-        // Merge the tile properties
-        for (Tile *replacementTile : replacement->tiles()) {
-            if (Tile *originalTile = tileset->findTile(replacementTile->id())) {
-                Properties properties = replacementTile->properties();
-                properties.merge(originalTile->properties());
-                undoStack->push(new ChangeProperties(mMapDocument,
-                                                     tr("Tile"),
-                                                     replacementTile,
-                                                     properties));
-            }
-        }
-        src->replaceTileset(tileset, replacement);
-
-        tilesetManager->addReference(replacement);
-        tilesetManager->removeReference(tileset);
-    }
     return true;
 }
 
@@ -452,18 +425,19 @@ void AutoMapper::autoMap(QRegion *where)
     // delete all the relevant area, if the property "DeleteTiles" is set
     if (mDeleteTiles) {
         const QRegion setLayersRegion = getSetLayersRegion();
-        for (RuleOutput *translationTable : mLayerList) {
-            foreach (Layer *layer, translationTable->keys()) {
-                const int index = translationTable->value(layer);
+        for (RuleOutput &translationTable : mLayerList) {
+            foreach (Layer *layer, translationTable.keys()) {
+                const int index = translationTable.value(layer);
                 Layer *dstLayer = mMapWork->layerAt(index);
                 const QRegion region = setLayersRegion.intersected(*where);
                 TileLayer *dstTileLayer = dstLayer->asTileLayer();
-                if (dstTileLayer)
+                if (dstTileLayer) {
                     dstTileLayer->erase(region);
-                else
+                } else {
                     eraseRegionObjectGroup(mMapDocument,
                                            dstLayer->asObjectGroup(),
                                            region);
+                }
             }
         }
     }
@@ -472,20 +446,21 @@ void AutoMapper::autoMap(QRegion *where)
     // This needs to be done, so you can rely on the order of the rules at all
     // locations
     QRegion ret;
-    foreach (const QRect &rect, where->rects())
+    foreach (const QRect &rect, where->rects()) {
         for (int i = 0; i < mRulesInput.size(); ++i) {
             // at the moment the parallel execution does not work yet
             // TODO: make multithreading available!
             // either by dividing the rules or the region to multiple threads
             ret = ret.united(applyRule(i, rect));
         }
+    }
     *where = where->united(ret);
 }
 
-const QRegion AutoMapper::getSetLayersRegion()
+QRegion AutoMapper::getSetLayersRegion() const
 {
     QRegion result;
-    foreach (const QString &name, mInputRules.names) {
+    for (const QString &name : mInputRules.names) {
         const int index = mMapWork->indexOfLayer(name, Layer::TileLayerType);
         if (index == -1)
             continue;
@@ -507,9 +482,9 @@ QRect AutoMapper::applyRule(const int ruleIndex, const QRect &where)
     if (mLayerList.isEmpty())
         return ret;
 
-    const QRegion ruleInput = mRulesInput.at(ruleIndex);
-    const QRegion ruleOutput = mRulesOutput.at(ruleIndex);
-    QRect rbr = ruleInput.boundingRect();
+    const QRegion &ruleInputRegion = mRulesInput.at(ruleIndex);
+    const QRegion &ruleOutputRegion = mRulesOutput.at(ruleIndex);
+    QRect rbr = ruleInputRegion.boundingRect();
 
     // Since the rule itself is translated, we need to adjust the borders of the
     // loops. Decrease the size at all sides by one: There must be at least one
@@ -530,43 +505,50 @@ QRect AutoMapper::applyRule(const int ruleIndex, const QRect &where)
 
     for (int y = minY; y <= maxY; ++y)
     for (int x = minX; x <= maxX; ++x) {
-        bool anymatch = false;
-        foreach (const QString &index, mInputRules.indexes) {
-            const InputIndex &ii = mInputRules[index];
+        bool anyMatch = false;
 
+        const auto &inputRules = mInputRules;
+        for (const InputIndex &inputIndex : inputRules) {
             bool allLayerNamesMatch = true;
-            foreach (const QString &name, ii.names) {
+
+            QMapIterator<QString, InputConditions> inputIndexIterator(inputIndex);
+            while (inputIndexIterator.hasNext()) {
+                inputIndexIterator.next();
+
+                const QString &name = inputIndexIterator.key();
+                const InputConditions &conditions = inputIndexIterator.value();
+
                 const int i = mMapWork->indexOfLayer(name, Layer::TileLayerType);
                 if (i == -1) {
                     allLayerNamesMatch = false;
                 } else {
                     const TileLayer *setLayer = mMapWork->layerAt(i)->asTileLayer();
                     allLayerNamesMatch &= compareLayerTo(setLayer,
-                                                         ii[name].listYes,
-                                                         ii[name].listNo,
-                                                         ruleInput,
+                                                         conditions.listYes,
+                                                         conditions.listNo,
+                                                         ruleInputRegion,
                                                          QPoint(x, y));
                 }
             }
             if (allLayerNamesMatch) {
-                anymatch = true;
+                anyMatch = true;
                 break;
             }
         }
 
-        if (anymatch) {
+        if (anyMatch) {
             // choose by chance which group of rule_layers should be used:
             const int r = qrand() % mLayerList.size();
-            const RuleOutput *translationTable = mLayerList.at(r);
+            const RuleOutput &translationTable = mLayerList.at(r);
 
             if (!mNoOverlappingRules) {
-                copyMapRegion(ruleOutput, QPoint(x, y), translationTable);
+                copyMapRegion(ruleOutputRegion, QPoint(x, y), translationTable);
                 ret = ret.united(rbr.translated(QPoint(x, y)));
                 continue;
             }
 
             bool missmatch = false;
-            QList<Layer*> layers = translationTable->keys();
+            const QList<Layer*> layers = translationTable.keys();
 
             // check if there are no overlaps within this rule.
             QVector<QRegion> ruleRegionInLayer;
@@ -574,15 +556,17 @@ QRect AutoMapper::applyRule(const int ruleIndex, const QRect &where)
                 Layer *layer = layers.at(i);
 
                 QRegion appliedPlace;
-                TileLayer *tileLayer = layer->asTileLayer();
-                if (tileLayer)
-                    appliedPlace = tileLayer->region();
-                else
-                    appliedPlace = tileRegionOfObjectGroup(layer->asObjectGroup());
 
-                ruleRegionInLayer.append(appliedPlace.intersected(ruleOutput));
-                if (appliedRegions.at(i).intersects(
-                            ruleRegionInLayer[i].translated(x, y))) {
+                if (TileLayer *tileLayer = layer->asTileLayer())
+                    appliedPlace = tileLayer->region();
+                else if (ObjectGroup *objectGroup = layer->asObjectGroup())
+                    appliedPlace = tileRegionOfObjectGroup(objectGroup);
+                else
+                    continue;
+
+                ruleRegionInLayer.append(appliedPlace.intersected(ruleOutputRegion));
+
+                if (appliedRegions.at(i).intersects(ruleRegionInLayer.at(i).translated(x, y))) {
                     missmatch = true;
                     break;
                 }
@@ -590,12 +574,10 @@ QRect AutoMapper::applyRule(const int ruleIndex, const QRect &where)
             if (missmatch)
                 continue;
 
-            copyMapRegion(ruleOutput, QPoint(x, y), translationTable);
+            copyMapRegion(ruleOutputRegion, QPoint(x, y), translationTable);
             ret = ret.united(rbr.translated(QPoint(x, y)));
-            for (int i = 0; i < translationTable->size(); ++i) {
-                appliedRegions[i] +=
-                        ruleRegionInLayer[i].translated(x, y);
-            }
+            for (int i = 0; i < translationTable.size(); ++i)
+                appliedRegions[i] += ruleRegionInLayer[i].translated(x, y);
         }
     }
 
@@ -610,7 +592,7 @@ static QVector<Cell> cellsInRegion(const QVector<TileLayer*> &list,
                                    const QRegion &r)
 {
     QVector<Cell> cells;
-    foreach (const TileLayer *tilelayer, list) {
+    for (const TileLayer *tilelayer : list) {
         foreach (const QRect &rect, r.rects()) {
             for (int x = rect.left(); x <= rect.right(); ++x) {
                 for (int y = rect.top(); y <= rect.bottom(); ++y) {
@@ -695,8 +677,6 @@ static bool compareLayerTo(const TileLayer *setLayer,
         return false;
 
     QVector<Cell> cells;
-    if (listYes.isEmpty())
-        cells = cellsInRegion(listNo, ruleRegion);
     if (listNo.isEmpty())
         cells = cellsInRegion(listYes, ruleRegion);
 
@@ -710,8 +690,18 @@ static bool compareLayerTo(const TileLayer *setLayer,
                 bool matchListYes = false;
                 bool matchListNo  = false;
 
-                if (!setLayer->contains(x + offset.x(), y + offset.y()))
-                    return false;
+
+                if (!setLayer->contains(x + offset.x(), y + offset.y())) {
+                    foreach (const TileLayer *comparedTileLayer, listYes) {
+                        if (!comparedTileLayer->contains(x, y))
+                            return false;
+
+                        const Cell &c2 = comparedTileLayer->cellAt(x, y);
+                        if (!c2.isEmpty())
+                            return false;
+                    }
+                    continue;
+                }
 
                 const Cell &c1 = setLayer->cellAt(x + offset.x(),
                                                   y + offset.y());
@@ -722,8 +712,7 @@ static bool compareLayerTo(const TileLayer *setLayer,
                 // if there is given no tile at all in the listYes layers,
                 // consider all tiles valid.
 
-                foreach (const TileLayer *comparedTileLayer, listYes) {
-
+                for (const TileLayer *comparedTileLayer : listYes) {
                     if (!comparedTileLayer->contains(x, y))
                         return false;
 
@@ -734,8 +723,7 @@ static bool compareLayerTo(const TileLayer *setLayer,
                     if (!c2.isEmpty() && c1 == c2)
                         matchListYes = true;
                 }
-                foreach (const TileLayer *comparedTileLayer, listNo) {
-
+                for (const TileLayer *comparedTileLayer : listNo) {
                     if (!comparedTileLayer->contains(x, y))
                         return false;
 
@@ -777,11 +765,12 @@ static bool compareLayerTo(const TileLayer *setLayer,
 }
 
 void AutoMapper::copyMapRegion(const QRegion &region, QPoint offset,
-                               const RuleOutput *layerTranslation)
+                               const RuleOutput &layerTranslation)
 {
-    for (auto it = layerTranslation->begin(), end = layerTranslation->end(); it != end; ++it) {
+    for (auto it = layerTranslation.begin(), end = layerTranslation.end(); it != end; ++it) {
         Layer *from = it.key();
-        Layer *to = mMapWork->layerAt(layerTranslation->value(from));
+        Layer *to = mMapWork->layerAt(it.value());
+
         foreach (const QRect &rect, region.rects()) {
             if (TileLayer *fromTileLayer = from->asTileLayer()) {
                 TileLayer *toTileLayer = to->asTileLayer();
@@ -801,10 +790,22 @@ void AutoMapper::copyMapRegion(const QRegion &region, QPoint offset,
                 Q_ASSERT(false);
             }
         }
+
+        // Copy any custom properties set on the output layer
+        if (!from->properties().isEmpty()) {
+            Properties mergedProperties = to->properties();
+            mergedProperties.merge(from->properties());
+
+            if (mergedProperties != to->properties()) {
+                QUndoStack *undoStack = mMapDocument->undoStack();
+                undoStack->push(new ChangeProperties(mMapDocument, QString(),
+                                                     to, mergedProperties));
+            }
+        }
     }
 }
 
-void AutoMapper::copyTileRegion(TileLayer *srcLayer, int srcX, int srcY,
+void AutoMapper::copyTileRegion(const TileLayer *srcLayer, int srcX, int srcY,
                                 int width, int height,
                                 TileLayer *dstLayer, int dstX, int dstY)
 {
@@ -828,7 +829,7 @@ void AutoMapper::copyTileRegion(TileLayer *srcLayer, int srcX, int srcY,
     }
 }
 
-void AutoMapper::copyObjectRegion(ObjectGroup *srcLayer, int srcX, int srcY,
+void AutoMapper::copyObjectRegion(const ObjectGroup *srcLayer, int srcX, int srcY,
                                   int width, int height,
                                   ObjectGroup *dstLayer, int dstX, int dstY)
 {
@@ -857,7 +858,10 @@ void AutoMapper::cleanAll()
 
 void AutoMapper::cleanTilesets()
 {
-    foreach (const SharedTileset &tileset, mAddedTilesets) {
+    QUndoStack *undoStack = mMapDocument->undoStack();
+
+    const auto &addedTilesets = mAddedTilesets;
+    for (const SharedTileset &tileset : addedTilesets) {
         if (mMapWork->isTilesetUsed(tileset.data()))
             continue;
 
@@ -865,38 +869,34 @@ void AutoMapper::cleanTilesets()
         if (index == -1)
             continue;
 
-        QUndoStack *undo = mMapDocument->undoStack();
-        undo->push(new RemoveTileset(mMapDocument, index));
+        undoStack->push(new RemoveTileset(mMapDocument, index));
     }
+
     mAddedTilesets.clear();
 }
 
 void AutoMapper::cleanTileLayers()
 {
-    foreach (const QString &tilelayerName, mAddedTileLayers) {
-        const int layerIndex = mMapWork->indexOfLayer(tilelayerName,
-                                                      Layer::TileLayerType);
-        if (layerIndex == -1)
-            continue;
+    QUndoStack *undoStack = mMapDocument->undoStack();
 
-        const Layer *layer = mMapWork->layerAt(layerIndex);
+    const auto &addedLayers = mAddedLayers;
+    for (Layer *layer : addedLayers) {
         if (!layer->isEmpty())
             continue;
 
-        QUndoStack *undo = mMapDocument->undoStack();
-        undo->push(new RemoveLayer(mMapDocument, layerIndex));
+        const int index = layer->siblingIndex();
+        GroupLayer *parentLayer = layer->parentLayer();
+
+        undoStack->push(new RemoveLayer(mMapDocument, index, parentLayer));
     }
-    mAddedTileLayers.clear();
+
+    mAddedLayers.clear();
 }
 
 void AutoMapper::cleanUpRulesMap()
 {
     cleanTilesets();
 
-    // mMapRules can be empty, when in prepareLoad the very first stages fail.
-    if (!mMapRules)
-        return;
-
     TilesetManager *tilesetManager = TilesetManager::instance();
     tilesetManager->removeReferences(mMapRules->tilesets());
 
@@ -912,10 +912,9 @@ void AutoMapper::cleanUpRuleMapLayers()
 {
     cleanTileLayers();
 
-    qDeleteAll(mLayerList);
     mLayerList.clear();
 
-    // do not delete mLayerRuleRegions, it is owned by the rulesmap
+    // do not delete these, they are owned by the rules map
     mLayerInputRegions = nullptr;
     mLayerOutputRegions = nullptr;
     mInputRules.clear();
