@@ -33,12 +33,13 @@ namespace Tiled {
 namespace Internal {
 
 void eraseRegionObjectGroup(MapDocument *mapDocument,
-                                        ObjectGroup *layer,
-                                        const QRegion &where)
+                            ObjectGroup *layer,
+                            const QRegion &where)
 {
     QUndoStack *undo = mapDocument->undoStack();
 
-    foreach (MapObject *obj, layer->objects()) {
+    const auto objects = layer->objects();
+    for (MapObject *obj : objects) {
         // TODO: we are checking bounds, which is only correct for rectangles and
         // tile objects. polygons and polylines are not covered correctly by this
         // erase method (we are in fact deleting too many objects)
@@ -63,10 +64,10 @@ void eraseRegionObjectGroup(MapDocument *mapDocument,
     }
 }
 
-QRegion tileRegionOfObjectGroup(ObjectGroup *layer)
+QRegion tileRegionOfObjectGroup(const ObjectGroup *layer)
 {
     QRegion ret;
-    foreach (MapObject *obj, layer->objects()) {
+    for (MapObject *obj : layer->objects()) {
         // TODO: we are using bounds, which is only correct for rectangles and
         // tile objects. polygons and polylines are not probably covering less
         // tiles.
@@ -75,11 +76,11 @@ QRegion tileRegionOfObjectGroup(ObjectGroup *layer)
     return ret;
 }
 
-const QList<MapObject*> objectsInRegion(ObjectGroup *layer,
+const QList<MapObject*> objectsInRegion(const ObjectGroup *layer,
                                         const QRegion &where)
 {
     QList<MapObject*> ret;
-    foreach (MapObject *obj, layer->objects()) {
+    for (MapObject *obj : layer->objects()) {
         // TODO: we are checking bounds, which is only correct for rectangles and
         // tile objects. polygons and polylines are not covered correctly by this
         // erase method (we are in fact deleting too many objects)
@@ -95,5 +96,5 @@ const QList<MapObject*> objectsInRegion(ObjectGroup *layer,
     return ret;
 }
 
-}
-}
+} // namespace Internal
+} // namespace Tiled
