@@ -19,8 +19,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef BRUSHITEM_H
-#define BRUSHITEM_H
+#pragma once
 
 #include "tilelayer.h"
 
@@ -97,5 +96,3 @@ inline QRegion BrushItem::tileRegion() const
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // BRUSHITEM_H
