@@ -18,8 +18,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef CHANGEMAPOBJECTSORDER_H
-#define CHANGEMAPOBJECTSORDER_H
+#pragma once
 
 #include <QUndoCommand>
 
@@ -38,7 +37,8 @@ public:
                           ObjectGroup *objectGroup,
                           int from,
                           int to,
-                          int count);
+                          int count,
+                          QUndoCommand *parent = nullptr);
 
     void undo() override;
     void redo() override;
@@ -53,5 +53,3 @@ private:
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // CHANGEMAPOBJECTSORDER_H
