@@ -18,8 +18,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef CHANGEMAPPROPERTY_H
-#define CHANGEMAPPROPERTY_H
+#pragma once
 
 #include "map.h"
 
@@ -125,5 +124,3 @@ private:
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // CHANGEMAPPROPERTY_H
