@@ -18,8 +18,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef CHANGEPROPERTIES_H
-#define CHANGEPROPERTIES_H
+#pragma once
 
 #include "object.h"
 
@@ -30,7 +29,7 @@
 namespace Tiled {
 namespace Internal {
 
-class MapDocument;
+class Document;
 
 class ChangeProperties : public QUndoCommand
 {
@@ -38,12 +37,12 @@ public:
     /**
      * Constructs a new 'Change Properties' command.
      *
-     * @param mapDocument  the map document of the object's map
+     * @param document     the document owning the object
      * @param kind         the kind of properties (Map, Layer, Object, etc.)
      * @param object       the object of which the properties should be changed
      * @param newProperties the new properties that should be applied
      */
-    ChangeProperties(MapDocument *mapDocument,
+    ChangeProperties(Document *document,
                      const QString &kind,
                      Object *object,
                      const Properties &newProperties,
@@ -55,7 +54,7 @@ public:
 private:
     void swapProperties();
 
-    MapDocument *mMapDocument;
+    Document *mDocument;
     Object *mObject;
     Properties mNewProperties;
 };
@@ -66,13 +65,13 @@ public:
     /**
      * Constructs a new 'Set Property' command.
      *
-     * @param mapDocument  the map document of the object's map
+     * @param document     the document owning the objects
      * @param objects      the objects of which the property should be changed
      * @param name         the name of the property to be changed
      * @param value        the new value of the property
      * @param type         the (new) type ot the property to be changed
      */
-    SetProperty(MapDocument *mapDocument,
+    SetProperty(Document *document,
                 const QList<Object*> &objects,
                 const QString &name,
                 const QVariant &value,
@@ -87,7 +86,7 @@ private:
         bool existed;
     };
     QVector<ObjectProperty> mProperties;
-    MapDocument *mMapDocument;
+    Document *mDocument;
     QList<Object*> mObjects;
     QString mName;
     QVariant mValue;
@@ -99,11 +98,11 @@ public:
     /**
      * Constructs a new 'Remove Property' command.
      *
-     * @param mapDocument  the map document of the object's map
+     * @param document     the document owning the objects
      * @param objects      the objects from which the property should be removed
      * @param name         the name of the property to be removed
      */
-    RemoveProperty(MapDocument *mapDocument,
+    RemoveProperty(Document *document,
                    const QList<Object*> &objects,
                    const QString &name,
                    QUndoCommand *parent = nullptr);
@@ -112,7 +111,7 @@ public:
     void redo() override;
 
 private:
-    MapDocument *mMapDocument;
+    Document *mDocument;
     QList<Object*> mObjects;
     QVector<QVariant> mPreviousValues;
     QString mName;
@@ -124,12 +123,12 @@ public:
     /**
      * Constructs a new 'Rename Property' command.
      *
-     * @param mapDocument  the map document of the object's map
+     * @param document     the document owning the object
      * @param object       the object of which the property should be renamed
      * @param oldName      the old name of the property
      * @param newName      the new name of the property
      */
-    RenameProperty(MapDocument *mapDocument,
+    RenameProperty(Document *document,
                    const QList<Object*> &objects,
                    const QString &oldName,
                    const QString &newName);
@@ -137,5 +136,3 @@ public:
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // CHANGEPROPERTIES_H
