@@ -18,8 +18,9 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef CLIPBOARDMANAGER_H
-#define CLIPBOARDMANAGER_H
+#pragma once
+
+#include "properties.h"
 
 #include <QObject>
 
@@ -43,37 +44,17 @@ class ClipboardManager : public QObject
     Q_OBJECT
 
 public:
-    /**
-     * Returns the clipboard manager instance. Creates the instance when it
-     * doesn't exist yet.
-     */
     static ClipboardManager *instance();
-
-    /**
-     * Deletes the clipboard manager instance if it exists.
-     */
     static void deleteInstance();
 
-    /**
-     * Returns whether the clipboard has a map.
-     */
-    bool hasMap() const { return mHasMap; }
-
-    /**
-     * Retrieves the map from the clipboard. Returns 0 when there was no map or
-     * loading failed.
-     */
+    bool hasMap() const;
     Map *map() const;
+    void setMap(const Map &map);
 
-    /**
-     * Sets the given map on the clipboard.
-     */
-    void setMap(const Map *map);
+    bool hasProperties() const;
+    Properties properties() const;
+    void setProperties(const Properties &properties);
 
-    /**
-     * Convenience method to copy the current selection to the clipboard.
-     * Deals with either tile selection or object selection.
-     */
     void copySelection(const MapDocument *mapDocument);
 
     enum PasteFlag {
@@ -84,23 +65,17 @@ public:
     Q_DECLARE_FLAGS(PasteFlags, PasteFlag)
     Q_FLAGS(PasteFlags)
 
-    /**
-     * Convenience method that deals with some of the logic related to pasting
-     * a group of objects.
-     */
     void pasteObjectGroup(const ObjectGroup *objectGroup,
                           MapDocument *mapDocument,
                           const MapView *view,
                           PasteFlags flags = PasteDefault);
 
 signals:
-    /**
-     * Emitted when whether the clip has a map changed.
-     */
     void hasMapChanged();
+    void hasPropertiesChanged();
 
 private slots:
-    void updateHasMap();
+    void update();
 
 private:
     ClipboardManager();
@@ -109,13 +84,28 @@ private:
 
     QClipboard *mClipboard;
     bool mHasMap;
+    bool mHasProperties;
 
     static ClipboardManager *mInstance;
 };
 
+/**
+ * Returns whether the clipboard has a map.
+ */
+inline bool ClipboardManager::hasMap() const
+{
+    return mHasMap;
+}
+
+/**
+ * Returns whether the clipboard holds some custom properties.
+ */
+inline bool ClipboardManager::hasProperties() const
+{
+    return mHasProperties;
+}
+
 Q_DECLARE_OPERATORS_FOR_FLAGS(ClipboardManager::PasteFlags)
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // CLIPBOARDMANAGER_H
