@@ -26,7 +26,6 @@
 
 #include <QDir>
 #include <QMessageBox>
-#include <QSettings>
 
 using namespace Tiled;
 using namespace Tiled::Internal;
@@ -36,18 +35,26 @@ QString Command::finalCommand() const
     QString finalCommand = command;
 
     // Perform variable replacement
-    if (MapDocument *mapDocument = DocumentManager::instance()->currentDocument()) {
-        const QString fileName = mapDocument->fileName();
+    if (Document *document = DocumentManager::instance()->currentDocument()) {
+        const QString fileName = document->fileName();
 
         finalCommand.replace(QLatin1String("%mapfile"),
                              QString(QLatin1String("\"%1\"")).arg(fileName));
 
-        if (const Layer *layer = mapDocument->currentLayer()) {
-            finalCommand.replace(QLatin1String("%layername"),
-                                 QString(QLatin1String("\"%1\"")).arg(layer->name()));
+        QFileInfo fileInfo(fileName);
+        QString mapPath = fileInfo.absolutePath();
+        finalCommand.replace(
+            QLatin1String("%mappath"),
+            QString(QLatin1String("\"%1\"")).arg(mapPath));
+
+        if (MapDocument *mapDocument = qobject_cast<MapDocument*>(document)) {
+            if (const Layer *layer = mapDocument->currentLayer()) {
+                finalCommand.replace(QLatin1String("%layername"),
+                                     QString(QLatin1String("\"%1\"")).arg(layer->name()));
+            }
         }
 
-        if (MapObject *currentObject = dynamic_cast<MapObject *>(mapDocument->currentObject())) {
+        if (MapObject *currentObject = dynamic_cast<MapObject *>(document->currentObject())) {
             finalCommand.replace(QLatin1String("%objecttype"),
                                  QString(QLatin1String("\"%1\"")).arg(currentObject->type()));
             finalCommand.replace(QLatin1String("%objectid"),
@@ -60,13 +67,10 @@ QString Command::finalCommand() const
 
 void Command::execute(bool inTerminal) const
 {
-    // Save if save option is unset or true
-    QSettings settings;
-    QVariant variant = settings.value(QLatin1String("saveBeforeExecute"), true);
-    if (variant.toBool()) {
-        MapDocument *document = DocumentManager::instance()->currentDocument();
+    if (saveBeforeExecute) {
+        Document *document = DocumentManager::instance()->currentDocument();
         if (document)
-            document->save();
+            document->save(document->fileName());
     }
 
     // Start the process
@@ -79,6 +83,8 @@ QVariant Command::toQVariant() const
     hash[QLatin1String("Enabled")] = isEnabled;
     hash[QLatin1String("Name")] = name;
     hash[QLatin1String("Command")] = command;
+    hash[QLatin1String("Shortcut")] = shortcut;
+    hash[QLatin1String("SaveBeforeExecute")] = saveBeforeExecute;
     return hash;
 }
 
@@ -89,6 +95,8 @@ Command Command::fromQVariant(const QVariant &variant)
     const QString namePref = QLatin1String("Name");
     const QString commandPref = QLatin1String("Command");
     const QString enablePref = QLatin1String("Enabled");
+    const QString shortcutPref = QLatin1String("Shortcut");
+    const QString saveBeforeExecutePref = QLatin1String("SaveBeforeExecute");
 
     Command command;
     if (hash.contains(enablePref))
@@ -97,6 +105,10 @@ Command Command::fromQVariant(const QVariant &variant)
         command.name = hash[namePref].toString();
     if (hash.contains(commandPref))
         command.command = hash[commandPref].toString();
+    if (hash.contains(shortcutPref))
+        command.shortcut = hash[shortcutPref].value<QKeySequence>();
+    if (hash.contains(saveBeforeExecutePref))
+        command.saveBeforeExecute = hash[saveBeforeExecutePref].toBool();
 
     return command;
 }
