@@ -18,10 +18,10 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef COMMAND_H
-#define COMMAND_H
+#pragma once
 
 #include <QString>
+#include <QKeySequence>
 #include <QProcess>
 #include <QVariant>
 
@@ -36,14 +36,20 @@ struct Command
 {
     Command(bool isEnabled = true,
             QString name = QString(),
-            QString command = QString())
+            QString command = QString(),
+            QKeySequence shortcut = QKeySequence(),
+            bool saveBeforeExecute = true)
         : isEnabled(isEnabled)
         , name(std::move(name))
-        , command(std::move(command)) {}
+        , command(std::move(command))
+        , shortcut(shortcut)
+        , saveBeforeExecute(saveBeforeExecute) {}
 
     bool isEnabled;
     QString name;
     QString command;
+    QKeySequence shortcut;
+    bool saveBeforeExecute;
 
     /**
      * Returns the final command with replaced tokens.
@@ -90,5 +96,3 @@ private:
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // COMMAND_H
