@@ -18,8 +18,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef CREATEMULTIPOINTBJECTTOOL_H
-#define CREATEMULTIPOINTBJECTTOOL_H
+#pragma once
 
 #include "createobjecttool.h"
 
@@ -35,7 +34,7 @@ public:
     CreateMultipointObjectTool(QObject *parent);
     ~CreateMultipointObjectTool();
 
-    void startNewMapObject(const QPointF &pos, ObjectGroup *objectGroup) override;
+    bool startNewMapObject(const QPointF &pos, ObjectGroup *objectGroup) override;
 
 protected:
     void mouseMovedWhileCreatingObject(const QPointF &pos,
@@ -49,5 +48,3 @@ private:
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // CREATEMULTIPOINTBJECTTOOL_H
