@@ -18,8 +18,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef CREATEOBJECTTOOL_H
-#define CREATEOBJECTTOOL_H
+#pragma once
 
 #include "abstractobjecttool.h"
 
@@ -37,12 +36,7 @@ class CreateObjectTool : public AbstractObjectTool
     Q_OBJECT
 
 public:
-    enum CreationMode {
-        CreateTile,
-        CreateGeometry
-    };
-
-    CreateObjectTool(CreationMode mode, QObject *parent = nullptr);
+    CreateObjectTool(QObject *parent = nullptr);
     ~CreateObjectTool();
 
     void activate(MapScene *scene) override;
@@ -69,7 +63,7 @@ protected:
     virtual void mouseReleasedWhileCreatingObject(QGraphicsSceneMouseEvent *event);
 
 
-    virtual void startNewMapObject(const QPointF &pos, ObjectGroup *objectGroup);
+    virtual bool startNewMapObject(const QPointF &pos, ObjectGroup *objectGroup);
     virtual MapObject *createNewMapObject() = 0;
     virtual void cancelNewMapObject();
     virtual void finishNewMapObject();
@@ -80,10 +74,7 @@ protected:
     MapObjectItem *mNewMapObjectItem;
     MapObjectItem *mOverlayPolygonItem;
     Tile *mTile;
-    CreationMode mMode;
 };
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // CREATEOBJECTTOOL_H
