@@ -19,28 +19,40 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef DOCUMENT_MANAGER_H
-#define DOCUMENT_MANAGER_H
+#pragma once
+
+#include "document.h"
+#include "tileset.h"
 
 #include <QList>
 #include <QObject>
 #include <QPair>
 #include <QPointF>
+#include <QSet>
 
 class QTabWidget;
 class QUndoGroup;
+class QStackedLayout;
+class QTabBar;
 
 namespace Tiled {
 
-class Tileset;
+class FileSystemWatcher;
 
 namespace Internal {
 
 class AbstractTool;
-class FileSystemWatcher;
+class BrokenLinksModel;
+class BrokenLinksWidget;
+class Document;
+class Editor;
+class FileChangedWarning;
 class MapDocument;
+class MapEditor;
 class MapScene;
 class MapView;
+class TilesetDocument;
+class TilesetDocumentsModel;
 
 /**
  * This class controls the open documents.
@@ -59,33 +71,34 @@ public:
      */
     QWidget *widget() const;
 
+    void setEditor(Document::DocumentType documentType, Editor *editor);
+    Editor *editor(Document::DocumentType documentType) const;
+    void deleteEditor(Document::DocumentType documentType);
+    QList<Editor*> editors() const;
+
+    Editor *currentEditor() const;
+
+    void saveState();
+    void restoreState();
+
     /**
      * Returns the undo group that combines the undo stacks of all opened map
      * documents.
      *
-     * @see MapDocument::undoStack()
+     * @see Document::undoStack()
      */
     QUndoGroup *undoGroup() const { return mUndoGroup; }
 
     /**
      * Returns the current map document, or 0 when there is none.
      */
-    MapDocument *currentDocument() const;
+    Document *currentDocument() const;
 
     /**
      * Returns the map view of the current document, or 0 when there is none.
      */
     MapView *currentMapView() const;
 
-    /**
-     * Returns the map scene of the current document, or 0 when there is none.
-     */
-    MapScene *currentMapScene() const;
-
-    /**
-     * Returns the map view that displays the given document, or 0 when there
-     * is none.
-     */
     MapView *viewForDocument(MapDocument *mapDocument) const;
 
     /**
@@ -103,12 +116,18 @@ public:
      * Switches to the map document at the given \a index.
      */
     void switchToDocument(int index);
-    void switchToDocument(MapDocument *mapDocument);
+    bool switchToDocument(Document *document);
 
     /**
-     * Adds the new or opened \a mapDocument to the document manager.
+     * Adds the new or opened \a document to the document manager.
      */
-    void addDocument(MapDocument *mapDocument);
+    void addDocument(Document *document);
+
+    bool isDocumentModified(Document *document) const;
+    bool isDocumentChangedOnDisk(Document *document) const;
+
+    bool saveDocument(Document *document, const QString &fileName);
+    bool saveDocumentAs(Document *document);
 
     /**
      * Closes the current map document. Will not ask the user whether to save
@@ -149,20 +168,34 @@ public:
     /**
      * Returns all open map documents.
      */
-    QList<MapDocument*> documents() const { return mDocuments; }
+    const QList<Document*> &documents() const { return mDocuments; }
+
+    TilesetDocumentsModel *tilesetDocumentsModel() const;
+
+    TilesetDocument *findTilesetDocument(const SharedTileset &tileset) const;
+    TilesetDocument *findTilesetDocument(const QString &fileName) const;
+
+    /**
+     * Opens the document for the given \a tileset.
+     */
+    void openTileset(const SharedTileset &tileset);
 
     /**
      * Centers the current map on the tile coordinates \a x, \a y.
      */
-    void centerViewOn(qreal x, qreal y);
-    void centerViewOn(const QPointF &pos)
-    { centerViewOn(pos.x(), pos.y()); }
+    void centerMapViewOn(qreal x, qreal y);
+    void centerMapViewOn(const QPointF &pos)
+    { centerMapViewOn(pos.x(), pos.y()); }
 
 signals:
+    void fileOpenRequested();
+    void fileOpenRequested(const QString &path);
+    void fileSaveRequested();
+
     /**
      * Emitted when the current displayed map document changed.
      */
-    void currentDocumentChanged(MapDocument *mapDocument);
+    void currentDocumentChanged(Document *document);
 
     /**
      * Emitted when the user requested the document at \a index to be closed.
@@ -172,35 +205,44 @@ signals:
     /**
      * Emitted when a document is about to be closed.
      */
-    void documentAboutToClose(MapDocument *document);
+    void documentAboutToClose(Document *document);
 
     /**
      * Emitted when an error occurred while reloading the map.
      */
     void reloadError(const QString &error);
 
+    void tilesetDocumentAdded(TilesetDocument *tilesetDocument);
+    void tilesetDocumentRemoved(TilesetDocument *tilesetDocument);
+
 public slots:
     void switchToLeftDocument();
     void switchToRightDocument();
 
-    void setSelectedTool(AbstractTool *tool);
+    void openFile();
+    void openFile(const QString &path);
+    void saveFile();
 
 private slots:
     void currentIndexChanged();
     void fileNameChanged(const QString &fileName,
                          const QString &oldFileName);
-    void updateDocumentTab();
+    void modifiedChanged();
+    void updateDocumentTab(Document *document);
     void documentSaved();
     void documentTabMoved(int from, int to);
     void tabContextMenuRequested(const QPoint &pos);
 
+    void tilesetAdded(int index, Tileset *tileset);
+    void tilesetRemoved(Tileset *tileset);
+    void tilesetReplaced(int index, Tileset *tileset, Tileset *oldTileset);
+
+    void tilesetNameChanged(Tileset *tileset);
+
     void fileChanged(const QString &fileName);
+    void hideChangedWarning();
 
-    void reloadRequested();
-
-    void cursorChanged(const QCursor &cursor);
-
-    void tilesetChanged(Tileset *tileset);
+    void tilesetImagesChanged(Tileset *tileset);
 
 private:
     DocumentManager(QObject *parent = nullptr);
@@ -208,18 +250,36 @@ private:
 
     bool askForAdjustment(const Tileset &tileset);
 
-    QList<MapDocument*> mDocuments;
+    void addToTilesetDocument(const SharedTileset &tileset, MapDocument *mapDocument);
+    void removeFromTilesetDocument(const SharedTileset &tileset, MapDocument *mapDocument);
+
+    QList<Document*> mDocuments;
+    TilesetDocumentsModel *mTilesetDocumentsModel;
+
+    QWidget *mWidget;
+    QWidget *mNoEditorWidget;
+    QTabBar *mTabBar;
+    FileChangedWarning *mFileChangedWarning;
+    BrokenLinksModel *mBrokenLinksModel;
+    BrokenLinksWidget *mBrokenLinksWidget;
+    QStackedLayout *mEditorStack;
+    MapEditor *mMapEditor;
+
+    QHash<Document::DocumentType, Editor*> mEditorForType;
 
-    QTabWidget *mTabWidget;
     QUndoGroup *mUndoGroup;
-    AbstractTool *mSelectedTool;
-    MapView *mViewWithTool;
     FileSystemWatcher *mFileSystemWatcher;
+    QSet<Document*> mDocumentsChangedOnDisk;
+
+    QMap<SharedTileset, TilesetDocument*> mTilesetToDocument;
 
     static DocumentManager *mInstance;
 };
 
+inline TilesetDocumentsModel *DocumentManager::tilesetDocumentsModel() const
+{
+    return mTilesetDocumentsModel;
+}
+
 } // namespace Tiled::Internal
 } // namespace Tiled
-
-#endif // DOCUMENT_MANAGER_H
