@@ -57,6 +57,7 @@ ExportAsImageDialog::ExportAsImageDialog(MapDocument *mapDocument,
     , mCurrentScale(currentScale)
 {
     mUi->setupUi(this);
+    resize(Utils::dpiScaled(size()));
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
     QPushButton *saveButton = mUi->buttonBox->button(QDialogButtonBox::Save);
@@ -216,12 +217,15 @@ void ExportAsImageDialog::accept()
 
     painter.translate(margins.left(), margins.top());
 
-    for (const Layer *layer : mMapDocument->map()->layers()) {
-        if (visibleLayersOnly && !layer->isVisible())
+    LayerIterator iterator(mMapDocument->map());
+    while (Layer *layer = iterator.next()) {
+        if (visibleLayersOnly && layer->isHidden())
             continue;
 
-        painter.setOpacity(layer->opacity());
-        painter.translate(layer->offset());
+        const auto offset = layer->totalOffset();
+
+        painter.setOpacity(layer->effectiveOpacity());
+        painter.translate(offset);
 
         switch (layer->layerType()) {
         case Layer::TileLayerType: {
@@ -261,9 +265,13 @@ void ExportAsImageDialog::accept()
             renderer->drawImageLayer(&painter, imageLayer);
             break;
         }
+
+        case Layer::GroupLayerType:
+            // Recursion handled by LayerIterator
+            break;
         }
 
-        painter.translate(-layer->offset());
+        painter.translate(-offset);
     }
 
     if (drawTileGrid) {
