@@ -1,6 +1,6 @@
 /*
  * mapscene.cpp
- * Copyright 2008-2014, Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
+ * Copyright 2008-2017, Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
  * Copyright 2008, Roderic Morris <roderic@ccs.neu.edu>
  * Copyright 2009, Edward Hutchins <eah1@yahoo.com>
  * Copyright 2010, Jeff Bland <jksb@member.fsf.org>
@@ -25,6 +25,8 @@
 
 #include "abstracttool.h"
 #include "containerhelpers.h"
+#include "grouplayer.h"
+#include "grouplayeritem.h"
 #include "map.h"
 #include "mapdocument.h"
 #include "mapobject.h"
@@ -74,21 +76,18 @@ MapScene::MapScene(QObject *parent):
             this, &MapScene::updateDefaultBackgroundColor);
 
     TilesetManager *tilesetManager = TilesetManager::instance();
-    connect(tilesetManager, SIGNAL(tilesetChanged(Tileset*)),
-            this, SLOT(tilesetChanged(Tileset*)));
-    connect(tilesetManager, SIGNAL(repaintTileset(Tileset*)),
-            this, SLOT(tilesetChanged(Tileset*)));
+    connect(tilesetManager, &TilesetManager::tilesetImagesChanged,
+            this, &MapScene::repaintTileset);
+    connect(tilesetManager, &TilesetManager::repaintTileset,
+            this, &MapScene::repaintTileset);
 
     Preferences *prefs = Preferences::instance();
-    connect(prefs, SIGNAL(showGridChanged(bool)), SLOT(setGridVisible(bool)));
-    connect(prefs, SIGNAL(showTileObjectOutlinesChanged(bool)),
-            SLOT(setShowTileObjectOutlines(bool)));
-    connect(prefs, SIGNAL(objectTypesChanged()), SLOT(syncAllObjectItems()));
-    connect(prefs, SIGNAL(highlightCurrentLayerChanged(bool)),
-            SLOT(setHighlightCurrentLayer(bool)));
-    connect(prefs, SIGNAL(gridColorChanged(QColor)), SLOT(update()));
-    connect(prefs, SIGNAL(objectLineWidthChanged(qreal)),
-            SLOT(setObjectLineWidth(qreal)));
+    connect(prefs, &Preferences::showGridChanged, this, &MapScene::setGridVisible);
+    connect(prefs, &Preferences::showTileObjectOutlinesChanged, this, &MapScene::setShowTileObjectOutlines);
+    connect(prefs, &Preferences::objectTypesChanged, this, &MapScene::syncAllObjectItems);
+    connect(prefs, &Preferences::highlightCurrentLayerChanged, this, &MapScene::setHighlightCurrentLayer);
+    connect(prefs, SIGNAL(gridColorChanged(QColor)), this, SLOT(update()));
+    connect(prefs, &Preferences::objectLineWidthChanged, this, &MapScene::setObjectLineWidth);
 
     mDarkRectangle->setPen(Qt::NoPen);
     mDarkRectangle->setBrush(Qt::black);
@@ -128,40 +127,40 @@ void MapScene::setMapDocument(MapDocument *mapDocument)
         renderer->setObjectLineWidth(mObjectLineWidth);
         renderer->setFlag(ShowTileObjectOutlines, mShowTileObjectOutlines);
 
-        connect(mMapDocument, SIGNAL(mapChanged()),
-                this, SLOT(mapChanged()));
+        connect(mMapDocument, &MapDocument::mapChanged,
+                this, &MapScene::mapChanged);
         connect(mMapDocument, &MapDocument::regionChanged,
                 this, &MapScene::repaintRegion);
-        connect(mMapDocument, SIGNAL(tileLayerDrawMarginsChanged(TileLayer*)),
-                this, SLOT(tileLayerDrawMarginsChanged(TileLayer*)));
-        connect(mMapDocument, SIGNAL(layerAdded(int)),
-                this, SLOT(layerAdded(int)));
-        connect(mMapDocument, SIGNAL(layerRemoved(int)),
-                this, SLOT(layerRemoved(int)));
-        connect(mMapDocument, SIGNAL(layerChanged(int)),
-                this, SLOT(layerChanged(int)));
-        connect(mMapDocument, SIGNAL(objectGroupChanged(ObjectGroup*)),
-                this, SLOT(objectGroupChanged(ObjectGroup*)));
-        connect(mMapDocument, SIGNAL(imageLayerChanged(ImageLayer*)),
-                this, SLOT(imageLayerChanged(ImageLayer*)));
-        connect(mMapDocument, SIGNAL(currentLayerIndexChanged(int)),
-                this, SLOT(currentLayerIndexChanged()));
+        connect(mMapDocument, &MapDocument::tileLayerDrawMarginsChanged,
+                this, &MapScene::tileLayerDrawMarginsChanged);
+        connect(mMapDocument, &MapDocument::layerAdded,
+                this, &MapScene::layerAdded);
+        connect(mMapDocument, &MapDocument::layerRemoved,
+                this, &MapScene::layerRemoved);
+        connect(mMapDocument, &MapDocument::layerChanged,
+                this, &MapScene::layerChanged);
+        connect(mMapDocument, &MapDocument::objectGroupChanged,
+                this, &MapScene::objectGroupChanged);
+        connect(mMapDocument, &MapDocument::imageLayerChanged,
+                this, &MapScene::imageLayerChanged);
+        connect(mMapDocument, &MapDocument::currentLayerChanged,
+                this, &MapScene::currentLayerChanged);
         connect(mMapDocument, &MapDocument::tilesetTileOffsetChanged,
                 this, &MapScene::adaptToTilesetTileSizeChanges);
         connect(mMapDocument, &MapDocument::tileImageSourceChanged,
                 this, &MapScene::adaptToTileSizeChanges);
         connect(mMapDocument, &MapDocument::tilesetReplaced,
                 this, &MapScene::tilesetReplaced);
-        connect(mMapDocument, SIGNAL(objectsInserted(ObjectGroup*,int,int)),
-                this, SLOT(objectsInserted(ObjectGroup*,int,int)));
-        connect(mMapDocument, SIGNAL(objectsRemoved(QList<MapObject*>)),
-                this, SLOT(objectsRemoved(QList<MapObject*>)));
-        connect(mMapDocument, SIGNAL(objectsChanged(QList<MapObject*>)),
-                this, SLOT(objectsChanged(QList<MapObject*>)));
-        connect(mMapDocument, SIGNAL(objectsIndexChanged(ObjectGroup*,int,int)),
-                this, SLOT(objectsIndexChanged(ObjectGroup*,int,int)));
-        connect(mMapDocument, SIGNAL(selectedObjectsChanged()),
-                this, SLOT(updateSelectedObjectItems()));
+        connect(mMapDocument, &MapDocument::objectsInserted,
+                this, &MapScene::objectsInserted);
+        connect(mMapDocument, &MapDocument::objectsRemoved,
+                this, &MapScene::objectsRemoved);
+        connect(mMapDocument, &MapDocument::objectsChanged,
+                this, &MapScene::objectsChanged);
+        connect(mMapDocument, &MapDocument::objectsIndexChanged,
+                this, &MapScene::objectsIndexChanged);
+        connect(mMapDocument, &MapDocument::selectedObjectsChanged,
+                this, &MapScene::updateSelectedObjectItems);
     }
 
     refreshScene();
@@ -201,21 +200,13 @@ void MapScene::refreshScene()
     updateSceneRect();
 
     const Map *map = mMapDocument->map();
-    mLayerItems.resize(map->layerCount());
 
     if (map->backgroundColor().isValid())
         setBackgroundBrush(map->backgroundColor());
     else
         setBackgroundBrush(mDefaultBackgroundColor);
 
-    int layerIndex = 0;
-    for (Layer *layer : map->layers()) {
-        QGraphicsItem *layerItem = createLayerItem(layer);
-        layerItem->setZValue(layerIndex);
-        addItem(layerItem);
-        mLayerItems[layerIndex] = layerItem;
-        ++layerIndex;
-    }
+    createLayerItems(map->layers());
 
     TileSelectionItem *tileSelectionItem = new TileSelectionItem(mMapDocument);
     tileSelectionItem->setZValue(10000 - 2);
@@ -228,13 +219,28 @@ void MapScene::refreshScene()
     updateCurrentLayerHighlight();
 }
 
-QGraphicsItem *MapScene::createLayerItem(Layer *layer)
+void MapScene::createLayerItems(const QList<Layer *> &layers)
 {
-    QGraphicsItem *layerItem = nullptr;
+    int layerIndex = 0;
 
-    if (TileLayer *tl = layer->asTileLayer()) {
-        layerItem = new TileLayerItem(tl, mMapDocument);
-    } else if (ObjectGroup *og = layer->asObjectGroup()) {
+    for (Layer *layer : layers) {
+        LayerItem *layerItem = createLayerItem(layer);
+        layerItem->setZValue(layerIndex);
+        ++layerIndex;
+    }
+}
+
+LayerItem *MapScene::createLayerItem(Layer *layer)
+{
+    LayerItem *layerItem = nullptr;
+
+    switch (layer->layerType()) {
+    case Layer::TileLayerType:
+        layerItem = new TileLayerItem(static_cast<TileLayer*>(layer), mMapDocument);
+        break;
+
+    case Layer::ObjectGroupType: {
+        auto og = static_cast<ObjectGroup*>(layer);
         const ObjectGroup::DrawOrder drawOrder = og->drawOrder();
         ObjectGroupItem *ogItem = new ObjectGroupItem(og);
         int objectIndex = 0;
@@ -250,13 +256,32 @@ QGraphicsItem *MapScene::createLayerItem(Layer *layer)
             ++objectIndex;
         }
         layerItem = ogItem;
-    } else if (ImageLayer *il = layer->asImageLayer()) {
-        layerItem = new ImageLayerItem(il, mMapDocument);
+        break;
+    }
+
+    case Layer::ImageLayerType:
+        layerItem = new ImageLayerItem(static_cast<ImageLayer*>(layer), mMapDocument);
+        break;
+
+    case Layer::GroupLayerType:
+        layerItem = new GroupLayerItem(static_cast<GroupLayer*>(layer));
+        break;
     }
 
     Q_ASSERT(layerItem);
 
     layerItem->setVisible(layer->isVisible());
+
+    if (layer->parentLayer())
+        layerItem->setParentItem(mLayerItems.value(layer->parentLayer()));
+    else
+        addItem(layerItem);
+
+    mLayerItems.insert(layer, layerItem);
+
+    if (GroupLayer *groupLayer = layer->asGroupLayer())
+        createLayerItems(groupLayer->layers());
+
     return layerItem;
 }
 
@@ -288,29 +313,41 @@ void MapScene::updateCurrentLayerHighlight()
     if (!mMapDocument)
         return;
 
-    const int currentLayerIndex = mMapDocument->currentLayerIndex();
+    const auto currentLayer = mMapDocument->currentLayer();
 
-    if (!mHighlightCurrentLayer || currentLayerIndex == -1) {
-        mDarkRectangle->setVisible(false);
+    if (!mHighlightCurrentLayer || !currentLayer) {
+        if (mDarkRectangle->isVisible()) {
+            mDarkRectangle->setVisible(false);
 
-        // Restore opacity for all layers
-        for (int i = 0; i < mLayerItems.size(); ++i) {
-            const Layer *layer = mMapDocument->map()->layerAt(i);
-            mLayerItems.at(i)->setOpacity(layer->opacity());
+            // Restore opacity for all layers
+            const auto layerItems = mLayerItems;
+            for (auto layerItem : layerItems)
+                layerItem->setOpacity(layerItem->layer()->opacity());
         }
 
         return;
     }
 
     // Darken layers below the current layer
-    mDarkRectangle->setZValue(currentLayerIndex - 0.5);
+    const int siblingIndex = currentLayer->siblingIndex();
+    const auto parentLayer = currentLayer->parentLayer();
+    const auto parentItem = mLayerItems.value(parentLayer);
+
+    mDarkRectangle->setParentItem(parentItem);
+    mDarkRectangle->setZValue(siblingIndex - 0.5);
     mDarkRectangle->setVisible(true);
 
-    // Set layers above the current layer to half opacity
-    for (int i = 1; i < mLayerItems.size(); ++i) {
-        const Layer *layer = mMapDocument->map()->layerAt(i);
-        const qreal multiplier = (currentLayerIndex < i) ? opacityFactor : 1;
-        mLayerItems.at(i)->setOpacity(layer->opacity() * multiplier);
+    // Set layers above the current layer to reduced opacity
+    LayerIterator iterator(mMapDocument->map());
+    qreal multiplier = 1;
+
+    while (Layer *layer = iterator.next()) {
+        GroupLayer *groupLayer = layer->asGroupLayer();
+        if (!groupLayer)
+            mLayerItems.value(layer)->setOpacity(layer->opacity() * multiplier);
+
+        if (layer == currentLayer)
+            multiplier = opacityFactor;
     }
 }
 
@@ -327,7 +364,7 @@ void MapScene::repaintRegion(const QRegion &region, Layer *layer)
                             margins.right(),
                             margins.bottom());
 
-        boundingRect.translate(layer->offset());
+        boundingRect.translate(layer->totalOffset());
 
         update(boundingRect);
     }
@@ -362,7 +399,7 @@ void MapScene::disableSelectedTool()
     mActiveTool = nullptr;
 }
 
-void MapScene::currentLayerIndexChanged()
+void MapScene::currentLayerChanged()
 {
     updateCurrentLayerHighlight();
 
@@ -394,7 +431,7 @@ void MapScene::mapChanged()
         setBackgroundBrush(mDefaultBackgroundColor);
 }
 
-void MapScene::tilesetChanged(Tileset *tileset)
+void MapScene::repaintTileset(Tileset *tileset)
 {
     if (!mMapDocument)
         return;
@@ -405,42 +442,74 @@ void MapScene::tilesetChanged(Tileset *tileset)
 
 void MapScene::tileLayerDrawMarginsChanged(TileLayer *tileLayer)
 {
-    const int index = mMapDocument->map()->layers().indexOf(tileLayer);
-    TileLayerItem *item = static_cast<TileLayerItem*>(mLayerItems.at(index));
+    TileLayerItem *item = static_cast<TileLayerItem*>(mLayerItems.value(tileLayer));
     item->syncWithTileLayer();
 }
 
-void MapScene::layerAdded(int index)
+void MapScene::layerAdded(Layer *layer)
 {
-    Layer *layer = mMapDocument->map()->layerAt(index);
-    QGraphicsItem *layerItem = createLayerItem(layer);
-    addItem(layerItem);
-    mLayerItems.insert(index, layerItem);
+    createLayerItem(layer);
 
     int z = 0;
-    for (QGraphicsItem *item : mLayerItems)
-        item->setZValue(z++);
+    for (auto sibling : layer->siblings())
+        mLayerItems.value(sibling)->setZValue(z++);
 }
 
-void MapScene::layerRemoved(int index)
+void MapScene::layerRemoved(Layer *layer)
 {
-    delete mLayerItems.at(index);
-    mLayerItems.remove(index);
+    delete mLayerItems.take(layer);
+}
+
+// Returns whether layerB is drawn above layerA
+static bool isAbove(Layer *layerA, Layer *layerB)
+{
+    int depthA = layerA->depth();
+    int depthB = layerB->depth();
+
+    // Make sure to start comparing at a common depth
+    while (depthA > 0 && depthA > depthB) {
+        layerA = layerA->parentLayer();
+        --depthA;
+    }
+    while (depthB > 0 && depthB > depthA) {
+        layerB = layerB->parentLayer();
+        --depthB;
+    }
+
+    // One of the layers is a child of the other
+    if (layerA == layerB)
+        return false;
+
+    // Move upwards until the layers have the same parent
+    while (true) {
+        GroupLayer *parentA = layerA->parentLayer();
+        GroupLayer *parentB = layerB->parentLayer();
+
+        if (parentA == parentB) {
+            const auto &layers = layerA->siblings();
+            const int indexA = layers.indexOf(layerA);
+            const int indexB = layers.indexOf(layerB);
+            return indexB > indexA;
+        }
+
+        layerA = parentA;
+        layerB = parentB;
+    }
 }
 
 /**
  * A layer has changed. This can mean that the layer visibility, opacity or
  * offset changed.
  */
-void MapScene::layerChanged(int index)
+void MapScene::layerChanged(Layer *layer)
 {
-    const Layer *layer = mMapDocument->map()->layerAt(index);
-    QGraphicsItem *layerItem = mLayerItems.at(index);
+    QGraphicsItem *layerItem = mLayerItems.value(layer);
+    Q_ASSERT(layerItem);
 
     layerItem->setVisible(layer->isVisible());
 
     qreal multiplier = 1;
-    if (mHighlightCurrentLayer && mMapDocument->currentLayerIndex() < index)
+    if (mHighlightCurrentLayer && isAbove(mMapDocument->currentLayer(), layer))
         multiplier = opacityFactor;
 
     layerItem->setOpacity(layer->opacity() * multiplier);
@@ -468,8 +537,7 @@ void MapScene::objectGroupChanged(ObjectGroup *objectGroup)
  */
 void MapScene::imageLayerChanged(ImageLayer *imageLayer)
 {
-    const int index = mMapDocument->map()->layers().indexOf(imageLayer);
-    ImageLayerItem *item = static_cast<ImageLayerItem*>(mLayerItems.at(index));
+    ImageLayerItem *item = static_cast<ImageLayerItem*>(mLayerItems.value(imageLayer));
 
     item->syncWithImageLayer();
     item->update();
@@ -489,7 +557,7 @@ void MapScene::adaptToTilesetTileSizeChanges(Tileset *tileset)
 
     for (MapObjectItem *item : mObjectItems) {
         const Cell &cell = item->mapObject()->cell();
-        if (!cell.isEmpty() && cell.tile->tileset() == tileset)
+        if (cell.tileset() == tileset)
             item->syncWithMapObject();
     }
 }
@@ -504,7 +572,7 @@ void MapScene::adaptToTileSizeChanges(Tile *tile)
 
     for (MapObjectItem *item : mObjectItems) {
         const Cell &cell = item->mapObject()->cell();
-        if (cell.tile == tile)
+        if (cell.tile() == tile)
             item->syncWithMapObject();
     }
 }
@@ -555,7 +623,7 @@ void MapScene::objectsInserted(ObjectGroup *objectGroup, int first, int last)
 void MapScene::objectsRemoved(const QList<MapObject*> &objects)
 {
     for (MapObject *o : objects) {
-        ObjectItems::iterator i = mObjectItems.find(o);
+        auto i = mObjectItems.find(o);
         Q_ASSERT(i != mObjectItems.end());
 
         mSelectedObjectItems.remove(i.value());
@@ -680,7 +748,7 @@ void MapScene::drawForeground(QPainter *painter, const QRectF &rect)
 
     // Take into account the offset of the current layer
     if (Layer *layer = mMapDocument->currentLayer()) {
-        offset = layer->offset();
+        offset = layer->totalOffset();
         painter->translate(offset);
     }
 
