@@ -23,6 +23,7 @@
 #include "flexiblescrollbar.h"
 #include "mapscene.h"
 #include "preferences.h"
+#include "utils.h"
 #include "zoomable.h"
 
 #include <QApplication>
@@ -34,9 +35,10 @@
 #include <QScrollBar>
 
 #ifndef QT_NO_OPENGL
-#include <QGLWidget>
+#include <QOpenGLWidget>
 #endif
 
+using namespace Tiled;
 using namespace Tiled::Internal;
 
 MapView::MapView(QWidget *parent, Mode mode)
@@ -100,15 +102,18 @@ void MapView::adjustScale(qreal scale)
 void MapView::setUseOpenGL(bool useOpenGL)
 {
 #ifndef QT_NO_OPENGL
-    if (useOpenGL && QGLFormat::hasOpenGL()) {
-        if (!qobject_cast<QGLWidget*>(viewport())) {
-            QGLFormat format = QGLFormat::defaultFormat();
-            format.setDepth(false); // No need for a depth buffer
-            format.setSampleBuffers(true); // Enable anti-aliasing
-            setViewport(new QGLWidget(format));
+    if (useOpenGL) {
+        if (!qobject_cast<QOpenGLWidget*>(viewport())) {
+            QSurfaceFormat format = QSurfaceFormat::defaultFormat();
+            format.setDepthBufferSize(0);   // No need for a depth buffer
+            format.setSamples(4);           // Enable anti-aliasing
+
+            QOpenGLWidget *openGLWidget = new QOpenGLWidget(this);
+            openGLWidget->setFormat(format);
+            setViewport(openGLWidget);
         }
     } else {
-        if (qobject_cast<QGLWidget*>(viewport()))
+        if (qobject_cast<QOpenGLWidget*>(viewport()))
             setViewport(nullptr);
     }
 
@@ -190,6 +195,14 @@ bool MapView::event(QEvent *e)
             if (pinch->changeFlags() & QPinchGesture::ScaleFactorChanged)
                 handlePinchGesture(pinch);
         }
+    } else if (e->type() == QEvent::ShortcutOverride) {
+        auto keyEvent = static_cast<QKeyEvent*>(e);
+        if (Utils::isZoomInShortcut(keyEvent) ||
+                Utils::isZoomOutShortcut(keyEvent) ||
+                Utils::isResetZoomShortcut(keyEvent)) {
+            e->accept();
+            return true;
+        }
     }
 
     return QGraphicsView::event(e);
@@ -202,11 +215,31 @@ void MapView::hideEvent(QHideEvent *event)
     QGraphicsView::hideEvent(event);
 }
 
+void MapView::keyPressEvent(QKeyEvent *event)
+{
+    if (Utils::isZoomInShortcut(event)) {
+        mZoomable->zoomIn();
+        return;
+    }
+    if (Utils::isZoomOutShortcut(event)) {
+        mZoomable->zoomOut();
+        return;
+    }
+    if (Utils::isResetZoomShortcut(event)) {
+        mZoomable->resetZoom();
+        return;
+    }
+    return QGraphicsView::keyPressEvent(event);
+}
+
 /**
  * Override to support zooming in and out using the mouse wheel.
  */
 void MapView::wheelEvent(QWheelEvent *event)
 {
+    auto *hBar = static_cast<FlexibleScrollBar*>(horizontalScrollBar());
+    auto *vBar = static_cast<FlexibleScrollBar*>(verticalScrollBar());
+
     if (event->modifiers() & Qt::ControlModifier
         && event->orientation() == Qt::Vertical)
     {
@@ -215,8 +248,8 @@ void MapView::wheelEvent(QWheelEvent *event)
 
         // This works around problems with automatic alignment of scenes that
         // are smaller than the view, which seems to be impossible to disable.
-        static_cast<FlexibleScrollBar*>(horizontalScrollBar())->allowNextRangeChange();
-        static_cast<FlexibleScrollBar*>(verticalScrollBar())->allowNextRangeChange();
+        hBar->allowNextRangeChange();
+        vBar->allowNextRangeChange();
 
         mZoomable->handleWheelDelta(event->delta());
 
@@ -227,11 +260,34 @@ void MapView::wheelEvent(QWheelEvent *event)
         return;
     }
 
-    QGraphicsView::wheelEvent(event);
+    // By default, the scroll area forwards the wheel events to the scroll
+    // bars, which apply their bounds. This custom wheel handling is here to
+    // override the bounds checking.
+    //
+    // This also disables QGraphicsSceneWheelEvent, but Tiled does not rely
+    // on that event.
 
-    // When scrolling the mouse does not move, but the view below it does.
-    // This affects the mouse scene position, which needs to be updated.
-    mLastMouseScenePos = mapToScene(viewport()->mapFromGlobal(mLastMousePos));
+    QPoint pixels = event->pixelDelta();
+
+    if (pixels.isNull()) {
+        QPointF steps = event->angleDelta() / 8.0 / 15.0;
+        int lines = QApplication::wheelScrollLines();
+        pixels.setX(int(steps.x() * lines * hBar->singleStep()));
+        pixels.setY(int(steps.y() * lines * vBar->singleStep()));
+    } else {
+        pixels = Utils::dpiScaled(pixels);
+    }
+
+    if (!pixels.isNull()) {
+        int horizontalValue = hBar->value() + (isRightToLeft() ? pixels.x() : -pixels.x());
+        int verticalValue = vBar->value() - pixels.y();
+        hBar->forceSetValue(horizontalValue);
+        vBar->forceSetValue(verticalValue);
+
+        // When scrolling the mouse does not move, but the view below it does.
+        // This affects the mouse scene position, which needs to be updated.
+        mLastMouseScenePos = mapToScene(viewport()->mapFromGlobal(mLastMousePos));
+    }
 }
 
 /**
@@ -288,7 +344,7 @@ void MapView::mouseMoveEvent(QMouseEvent *event)
 
 void MapView::handlePinchGesture(QPinchGesture *pinch)
 {
-    setTransformationAnchor(QGraphicsView::NoAnchor);
+    setTransformationAnchor(QGraphicsView::AnchorUnderMouse);
 
     mZoomable->handlePinchGesture(pinch);
 
