@@ -18,8 +18,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef MOVEMAPOBJECT_H
-#define MOVEMAPOBJECT_H
+#pragma once
 
 #include <QUndoCommand>
 #include <QPointF>
@@ -37,12 +36,14 @@ class MoveMapObject : public QUndoCommand
 public:
     MoveMapObject(MapDocument *mapDocument,
                   MapObject *mapObject,
-                  const QPointF &oldPos);
+                  const QPointF &oldPos,
+                  QUndoCommand *parent = nullptr);
 
     MoveMapObject(MapDocument *mapDocument,
                   MapObject *mapObject,
                   const QPointF &newPos,
-                  const QPointF &oldPos);
+                  const QPointF &oldPos,
+                  QUndoCommand *parent = nullptr);
 
     void undo() override;
     void redo() override;
@@ -56,5 +57,3 @@ private:
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // MOVEMAPOBJECT_H
