@@ -21,14 +21,17 @@
 #include "objectsdock.h"
 
 #include "documentmanager.h"
+#include "grouplayer.h"
 #include "map.h"
-#include "mapobject.h"
 #include "mapdocument.h"
 #include "mapdocumentactionhandler.h"
+#include "mapobject.h"
 #include "mapobjectmodel.h"
 #include "objectgroup.h"
 #include "preferences.h"
+#include "reversingproxymodel.h"
 #include "utils.h"
+#include "eyevisibilitydelegate.h"
 
 #include <QApplication>
 #include <QBoxLayout>
@@ -42,6 +45,7 @@
 #include <QUrl>
 
 static const char FIRST_SECTION_SIZE_KEY[] = "ObjectsDock/FirstSectionSize";
+static const char VISIBLE_SECTIONS_KEY[] = "ObjectsDock/VisibleSections";
 
 using namespace Tiled;
 using namespace Tiled::Internal;
@@ -56,15 +60,13 @@ ObjectsDock::ObjectsDock(QWidget *parent)
     mActionObjectProperties = new QAction(this);
     mActionObjectProperties->setIcon(QIcon(QLatin1String(":/images/16x16/document-properties.png")));
 
-    Utils::setThemeIcon(mActionObjectProperties, "document-properties");
-
     connect(mActionObjectProperties, SIGNAL(triggered()), SLOT(objectProperties()));
 
     MapDocumentActionHandler *handler = MapDocumentActionHandler::instance();
 
     QWidget *widget = new QWidget(this);
     QVBoxLayout *layout = new QVBoxLayout(widget);
-    layout->setMargin(5);
+    layout->setMargin(0);
     layout->setSpacing(0);
     layout->addWidget(mObjectsView);
 
@@ -76,15 +78,26 @@ ObjectsDock::ObjectsDock(QWidget *parent)
     mActionMoveToGroup = new QAction(this);
     mActionMoveToGroup->setIcon(QIcon(QLatin1String(":/images/16x16/layer-object.png")));
 
+    mActionMoveUp = new QAction(this);
+    mActionMoveUp->setIcon(QIcon(QLatin1String(":/images/16x16/go-up.png")));
+    mActionMoveDown = new QAction(this);
+    mActionMoveDown->setIcon(QIcon(QLatin1String(":/images/16x16/go-down.png")));
+
+    Utils::setThemeIcon(mActionObjectProperties, "document-properties");
+    Utils::setThemeIcon(mActionMoveUp, "go-up");
+    Utils::setThemeIcon(mActionMoveDown, "go-down");
+
     QToolBar *toolBar = new QToolBar;
     toolBar->setFloatable(false);
     toolBar->setMovable(false);
-    toolBar->setIconSize(QSize(16, 16));
+    toolBar->setIconSize(Utils::smallIconSize());
 
     toolBar->addAction(mActionNewLayer);
     toolBar->addAction(handler->actionDuplicateObjects());
     toolBar->addAction(handler->actionRemoveObjects());
 
+    toolBar->addAction(mActionMoveUp);
+    toolBar->addAction(mActionMoveDown);
     toolBar->addAction(mActionMoveToGroup);
     QToolButton *button;
     button = dynamic_cast<QToolButton*>(toolBar->widgetForAction(mActionMoveToGroup));
@@ -101,14 +114,29 @@ ObjectsDock::ObjectsDock(QWidget *parent)
     setWidget(widget);
     retranslateUi();
 
-    connect(DocumentManager::instance(), SIGNAL(documentAboutToClose(MapDocument*)),
-            SLOT(documentAboutToClose(MapDocument*)));
+    connect(DocumentManager::instance(), &DocumentManager::documentAboutToClose,
+            this, &ObjectsDock::documentAboutToClose);
+
+    connect(mActionMoveUp, &QAction::triggered, this, &ObjectsDock::moveObjectsUp);
+    connect(mActionMoveDown, &QAction::triggered, this, &ObjectsDock::moveObjectsDown);
+}
+
+void ObjectsDock::moveObjectsUp()
+{
+    if (mMapDocument)
+        mMapDocument->moveObjectsUp(mMapDocument->selectedObjects());
+}
+
+void ObjectsDock::moveObjectsDown()
+{
+    if (mMapDocument)
+        mMapDocument->moveObjectsDown(mMapDocument->selectedObjects());
 }
 
 void ObjectsDock::setMapDocument(MapDocument *mapDoc)
 {
     if (mMapDocument) {
-        saveExpandedGroups(mMapDocument);
+        saveExpandedGroups();
         mMapDocument->disconnect(this);
     }
 
@@ -117,7 +145,7 @@ void ObjectsDock::setMapDocument(MapDocument *mapDoc)
     mObjectsView->setMapDocument(mapDoc);
 
     if (mMapDocument) {
-        restoreExpandedGroups(mMapDocument);
+        restoreExpandedGroups();
         connect(mMapDocument, SIGNAL(selectedObjectsChanged()),
                 this, SLOT(updateActions()));
     }
@@ -143,27 +171,35 @@ void ObjectsDock::retranslateUi()
 
     mActionNewLayer->setToolTip(tr("Add Object Layer"));
     mActionObjectProperties->setToolTip(tr("Object Properties"));
+    mActionMoveUp->setToolTip(tr("Move Objects Up"));
+    mActionMoveDown->setToolTip(tr("Move Objects Down"));
 
     updateActions();
 }
 
 void ObjectsDock::updateActions()
 {
-    int count = mMapDocument ? mMapDocument->selectedObjects().count() : 0;
-    bool enabled = count > 0;
-    mActionObjectProperties->setEnabled(count == 1);
+    int selectedObjectsCount = 0;
+    int objectGroupCount = 0;
 
-    if (mMapDocument && (mMapDocument->map()->objectGroupCount() < 2))
-        enabled = false;
-    mActionMoveToGroup->setEnabled(enabled);
-    mActionMoveToGroup->setToolTip(tr("Move %n Object(s) to Layer", "", count));
+    if (mMapDocument) {
+        selectedObjectsCount = mMapDocument->selectedObjects().count();
+        objectGroupCount = mMapDocument->map()->objectGroupCount();
+    }
+
+    mActionObjectProperties->setEnabled(selectedObjectsCount > 0);
+    mActionMoveToGroup->setEnabled(selectedObjectsCount > 0 && objectGroupCount >= 2);
+    mActionMoveToGroup->setToolTip(tr("Move %n Object(s) to Layer", "", selectedObjectsCount));
+    mActionMoveUp->setEnabled(selectedObjectsCount > 0);
+    mActionMoveDown->setEnabled(selectedObjectsCount > 0);
 }
 
 void ObjectsDock::aboutToShowMoveToMenu()
 {
     mMoveToMenu->clear();
 
-    foreach (ObjectGroup *objectGroup, mMapDocument->map()->objectGroups()) {
+    const auto &objectGroups = mMapDocument->map()->objectGroups();
+    for (ObjectGroup *objectGroup : objectGroups) {
         QAction *action = mMoveToMenu->addAction(objectGroup->name());
         action->setData(QVariant::fromValue(objectGroup));
     }
@@ -182,28 +218,38 @@ void ObjectsDock::objectProperties()
     const QList<MapObject *> &selectedObjects = mMapDocument->selectedObjects();
     MapObject *mapObject = selectedObjects.first();
     mMapDocument->setCurrentObject(mapObject);
-    mMapDocument->emitEditCurrentObject();
+    emit mMapDocument->editCurrentObject();
 }
 
-void ObjectsDock::saveExpandedGroups(MapDocument *mapDoc)
+void ObjectsDock::saveExpandedGroups()
 {
-    mExpandedGroups[mapDoc].clear();
-    foreach (ObjectGroup *og, mapDoc->map()->objectGroups()) {
-        if (mObjectsView->isExpanded(mObjectsView->model()->index(og)))
-            mExpandedGroups[mapDoc].append(og);
+    mExpandedGroups[mMapDocument].clear();
+
+    const auto proxyModel = static_cast<QAbstractProxyModel*>(mObjectsView->model());
+    const auto &objectGroups = mMapDocument->map()->objectGroups();
+
+    for (ObjectGroup *og : objectGroups) {
+        const QModelIndex sourceIndex = mMapDocument->mapObjectModel()->index(og);
+        const QModelIndex index = proxyModel->mapFromSource(sourceIndex);
+        if (mObjectsView->isExpanded(index))
+            mExpandedGroups[mMapDocument].append(og);
     }
 }
 
-void ObjectsDock::restoreExpandedGroups(MapDocument *mapDoc)
+void ObjectsDock::restoreExpandedGroups()
 {
-    foreach (ObjectGroup *og, mExpandedGroups[mapDoc])
-        mObjectsView->setExpanded(mObjectsView->model()->index(og), true);
-    mExpandedGroups[mapDoc].clear();
+    const auto objectGroups = mExpandedGroups.take(mMapDocument);
+    for (ObjectGroup *og : objectGroups) {
+        const QModelIndex sourceIndex = mMapDocument->mapObjectModel()->index(og);
+        const QModelIndex index = static_cast<QAbstractProxyModel*>(mObjectsView->model())->mapFromSource(sourceIndex);
+        mObjectsView->setExpanded(index, true);
+    }
 }
 
-void ObjectsDock::documentAboutToClose(MapDocument *mapDocument)
+void ObjectsDock::documentAboutToClose(Document *document)
 {
-    mExpandedGroups.remove(mapDocument);
+    if (MapDocument *mapDocument = qobject_cast<MapDocument*>(document))
+        mExpandedGroups.remove(mapDocument);
 }
 
 ///// ///// ///// ///// /////
@@ -211,9 +257,12 @@ void ObjectsDock::documentAboutToClose(MapDocument *mapDocument)
 ObjectsView::ObjectsView(QWidget *parent)
     : QTreeView(parent)
     , mMapDocument(nullptr)
+    , mProxyModel(new ReversingProxyModel(this))
     , mSynching(false)
 {
     setUniformRowHeights(true);
+    setModel(mProxyModel);
+    setItemDelegate(new EyeVisibilityDelegate(this));
 
     setSelectionBehavior(QAbstractItemView::SelectRows);
     setSelectionMode(QAbstractItemView::ExtendedSelection);
@@ -223,11 +272,14 @@ ObjectsView::ObjectsView(QWidget *parent)
 
     connect(header(), SIGNAL(sectionResized(int,int,int)),
             this, SLOT(onSectionResized(int)));
+
+    header()->setContextMenuPolicy(Qt::CustomContextMenu);
+    connect(header(), &QWidget::customContextMenuRequested, this, &ObjectsView::showCustomMenu);
 }
 
 QSize ObjectsView::sizeHint() const
 {
-    return QSize(130, 100);
+    return Utils::dpiScaled(QSize(130, 100));
 }
 
 void ObjectsView::setMapDocument(MapDocument *mapDoc)
@@ -241,7 +293,7 @@ void ObjectsView::setMapDocument(MapDocument *mapDoc)
     mMapDocument = mapDoc;
 
     if (mMapDocument) {
-        setModel(mMapDocument->mapObjectModel());
+        mProxyModel->setSourceModel(mMapDocument->mapObjectModel());
 
         const QSettings *settings = Preferences::instance()->settings();
         const int firstSectionSize =
@@ -251,30 +303,49 @@ void ObjectsView::setMapDocument(MapDocument *mapDoc)
         connect(mMapDocument, SIGNAL(selectedObjectsChanged()),
                 this, SLOT(selectedObjectsChanged()));
 
+        restoreVisibleSections();
         synchronizeSelectedItems();
     } else {
-        setModel(nullptr);
+        mProxyModel->setSourceModel(nullptr);
     }
 }
 
-MapObjectModel *ObjectsView::model() const
+MapObjectModel *ObjectsView::mapObjectModel() const
 {
-    return static_cast<MapObjectModel*>(QTreeView::model());
+    return mMapDocument ? mMapDocument->mapObjectModel() : nullptr;
 }
 
-void ObjectsView::onPressed(const QModelIndex &index)
+bool ObjectsView::event(QEvent *event)
 {
-    if (MapObject *mapObject = model()->toMapObject(index))
-        mMapDocument->setCurrentObject(mapObject);
-    else if (ObjectGroup *objectGroup = model()->toObjectGroup(index))
-        mMapDocument->setCurrentObject(objectGroup);
+    if (event->type() == QEvent::ShortcutOverride) {
+        if (static_cast<QKeyEvent *>(event)->key() == Qt::Key_Tab) {
+            if (indexWidget(currentIndex())) {
+                event->accept();
+                return true;
+            }
+        }
+    }
+
+    return QTreeView::event(event);
 }
 
-void ObjectsView::onActivated(const QModelIndex &index)
+void ObjectsView::onPressed(const QModelIndex &proxyIndex)
 {
-    if (MapObject *mapObject = model()->toMapObject(index)) {
+    const QModelIndex index = mProxyModel->mapToSource(proxyIndex);
+
+    if (MapObject *mapObject = mapObjectModel()->toMapObject(index))
         mMapDocument->setCurrentObject(mapObject);
-        mMapDocument->emitEditCurrentObject();
+    else if (Layer *layer = mapObjectModel()->toLayer(index))
+        mMapDocument->setCurrentObject(layer);
+}
+
+void ObjectsView::onActivated(const QModelIndex &proxyIndex)
+{
+    const QModelIndex index = mProxyModel->mapToSource(proxyIndex);
+
+    if (MapObject *mapObject = mapObjectModel()->toMapObject(index)) {
+        mMapDocument->setCurrentObject(mapObject);
+        emit mMapDocument->editCurrentObject();
     }
 }
 
@@ -296,33 +367,23 @@ void ObjectsView::selectionChanged(const QItemSelection &selected,
     if (!mMapDocument || mSynching)
         return;
 
-    const QModelIndexList selectedRows = selectionModel()->selectedRows();
-    int currentLayerIndex = -1;
+    const QModelIndexList selectedProxyRows = selectionModel()->selectedRows();
 
     QList<MapObject*> selectedObjects;
-    for (const QModelIndex &index : selectedRows) {
-        if (ObjectGroup *og = model()->toLayer(index)) {
-            int index = mMapDocument->map()->layers().indexOf(og);
-            if (currentLayerIndex == -1)
-                currentLayerIndex = index;
-            else if (currentLayerIndex != index)
-                currentLayerIndex = -2;
-        }
-        if (MapObject *o = model()->toMapObject(index))
+    for (const QModelIndex &proxyIndex : selectedProxyRows) {
+        const QModelIndex index = mProxyModel->mapToSource(proxyIndex);
+
+        if (MapObject *o = mapObjectModel()->toMapObject(index))
             selectedObjects.append(o);
     }
 
-    // Switch the current object layer if only one object layer (and/or its objects)
-    // are included in the current selection.
-    if (currentLayerIndex >= 0 && currentLayerIndex != mMapDocument->currentLayerIndex())
-        mMapDocument->setCurrentLayerIndex(currentLayerIndex);
-
     if (selectedObjects != mMapDocument->selectedObjects()) {
         mSynching = true;
         if (selectedObjects.count() == 1) {
             const MapObject *o = selectedObjects.first();
             const QPointF center = o->bounds().center();
-            DocumentManager::instance()->centerViewOn(center);
+            const QPointF offset = o->objectGroup()->totalOffset();
+            DocumentManager::instance()->centerMapViewOn(center + offset);
         }
         mMapDocument->setSelectedObjects(selectedObjects);
         mSynching = false;
@@ -339,7 +400,53 @@ void ObjectsView::selectedObjectsChanged()
     const QList<MapObject *> &selectedObjects = mMapDocument->selectedObjects();
     if (selectedObjects.count() == 1) {
         MapObject *o = selectedObjects.first();
-        scrollTo(model()->index(o));
+        scrollTo(mProxyModel->mapFromSource(mapObjectModel()->index(o)));
+    }
+}
+
+void ObjectsView::setColumnVisibility(bool visible)
+{
+    QAction *action = qobject_cast<QAction*>(sender());
+    if (!action)
+        return;
+
+    int column = action->data().toInt();
+    header()->setSectionHidden(column, !visible);
+
+    QSettings *settings = Preferences::instance()->settings();
+    QVariantList visibleSections;
+    for (int i = 0; i < mProxyModel->columnCount(); i++) {
+        if (!header()->isSectionHidden(i))
+            visibleSections.append(i);
+    }
+    settings->setValue(QLatin1String(VISIBLE_SECTIONS_KEY), visibleSections);
+}
+
+void ObjectsView::showCustomMenu(const QPoint &point)
+{
+    Q_UNUSED(point)
+    QMenu contextMenu(this);
+    QAbstractItemModel *model = mProxyModel->sourceModel();
+    for (int i = 0; i < model->columnCount(); i++) {
+        if (i == MapObjectModel::Name)
+            continue;
+        QAction *action = new QAction(model->headerData(i, Qt::Horizontal).toString(), &contextMenu);
+        action->setCheckable(true);
+        action->setChecked(!header()->isSectionHidden(i));
+        action->setData(i);
+        connect(action, &QAction::triggered, this, &ObjectsView::setColumnVisibility);
+        contextMenu.addAction(action);
+    }
+    contextMenu.exec(QCursor::pos());
+}
+
+void ObjectsView::restoreVisibleSections()
+{
+    QSettings *settings = Preferences::instance()->settings();
+    QVariantList visibleSections = settings->value(QLatin1String(VISIBLE_SECTIONS_KEY),
+                                                 QVariantList() << MapObjectModel::Name << MapObjectModel::Type).toList();
+    for (int i = 0; i < mProxyModel->columnCount(); i++) {
+        header()->setSectionHidden(i, !visibleSections.contains(i));
     }
 }
 
@@ -348,11 +455,10 @@ void ObjectsView::synchronizeSelectedItems()
     Q_ASSERT(!mSynching);
     Q_ASSERT(mMapDocument);
 
-    const QList<MapObject *> &selectedObjects = mMapDocument->selectedObjects();
     QItemSelection itemSelection;
 
-    for (MapObject *o : selectedObjects) {
-        QModelIndex index = model()->index(o);
+    for (MapObject *o : mMapDocument->selectedObjects()) {
+        QModelIndex index = mProxyModel->mapFromSource(mapObjectModel()->index(o));
         itemSelection.select(index, index);
     }
 
