@@ -18,15 +18,17 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef TILED_INTERNAL_OBJECTSELECTIONITEM_H
-#define TILED_INTERNAL_OBJECTSELECTIONITEM_H
+#pragma once
 
 #include <QGraphicsObject>
 #include <QHash>
 
 namespace Tiled {
 
+class GroupLayer;
+class Layer;
 class MapObject;
+class Tile;
 
 namespace Internal {
 
@@ -49,13 +51,14 @@ public:
 private slots:
     void selectedObjectsChanged();
     void mapChanged();
-    void layerAdded(int index);
-    void layerAboutToBeRemoved(int index);
-    void layerChanged(int index);
+    void layerAdded(Layer *layer);
+    void layerAboutToBeRemoved(GroupLayer *parentLayer, int index);
+    void layerChanged(Layer *layer);
     void syncOverlayItems(const QList<MapObject *> &objects);
     void updateObjectLabelColors();
     void objectsAdded(const QList<MapObject*> &objects);
     void objectsRemoved(const QList<MapObject*> &objects);
+    void tileTypeChanged(Tile *tile);
 
     void objectLabelVisibilityChanged();
 
@@ -70,5 +73,3 @@ private:
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // TILED_INTERNAL_OBJECTSELECTIONITEM_H
