@@ -86,7 +86,7 @@ void ColorDelegate::paint(QPainter *painter,
 QSize ColorDelegate::sizeHint(const QStyleOptionViewItem &,
                               const QModelIndex &) const
 {
-    return QSize(50, 20);
+    return Utils::dpiScaled(QSize(50, 20));
 }
 
 
@@ -99,6 +99,7 @@ ObjectTypesEditor::ObjectTypesEditor(QWidget *parent)
     , mUpdating(false)
 {
     mUi->setupUi(this);
+    resize(Utils::dpiScaled(size()));
 
     mUi->objectTypesTable->setModel(mObjectTypesModel);
     mUi->objectTypesTable->setItemDelegateForColumn(1, new ColorDelegate(this));
@@ -137,12 +138,12 @@ ObjectTypesEditor::ObjectTypesEditor(QWidget *parent)
     Utils::setThemeIcon(mRemovePropertyAction, "remove");
 
     QToolBar *objectTypesToolBar = new QToolBar(this);
-    objectTypesToolBar->setIconSize(QSize(16, 16));
+    objectTypesToolBar->setIconSize(Utils::smallIconSize());
     objectTypesToolBar->addAction(mAddObjectTypeAction);
     objectTypesToolBar->addAction(mRemoveObjectTypeAction);
 
     QToolBar *propertiesToolBar = new QToolBar(this);
-    propertiesToolBar->setIconSize(QSize(16, 16));
+    propertiesToolBar->setIconSize(Utils::smallIconSize());
     propertiesToolBar->addAction(mAddPropertyAction);
     propertiesToolBar->addAction(mRemovePropertyAction);
     propertiesToolBar->addAction(mRenamePropertyAction);
@@ -177,10 +178,12 @@ ObjectTypesEditor::ObjectTypesEditor(QWidget *parent)
     connect(mUi->actionExport, SIGNAL(triggered()),
             SLOT(exportObjectTypes()));
 
-    connect(mObjectTypesModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-            SLOT(applyObjectTypes()));
-    connect(mObjectTypesModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-            SLOT(applyObjectTypes()));
+    connect(mObjectTypesModel, &ObjectTypesModel::dataChanged,
+            this, &ObjectTypesEditor::applyObjectTypes);
+    connect(mObjectTypesModel, &ObjectTypesModel::rowsInserted,
+            this, &ObjectTypesEditor::applyObjectTypes);
+    connect(mObjectTypesModel, &ObjectTypesModel::rowsRemoved,
+            this, &ObjectTypesEditor::applyObjectTypes);
 
     connect(mVariantManager, &QtVariantPropertyManager::valueChanged,
             this, &ObjectTypesEditor::propertyValueChanged);
@@ -241,8 +244,7 @@ void ObjectTypesEditor::addObjectType()
                QItemSelectionModel::ClearAndSelect |
                QItemSelectionModel::Rows);
     sm->setCurrentIndex(newIndex, QItemSelectionModel::Current);
-    mUi->objectTypesTable->setFocus();
-    mUi->objectTypesTable->scrollTo(newIndex);
+    mUi->objectTypesTable->edit(newIndex);
 }
 
 void ObjectTypesEditor::selectedObjectTypesChanged()
@@ -282,12 +284,12 @@ void ObjectTypesEditor::applyObjectTypes()
     if (!objectTypesDir.exists())
         objectTypesDir.mkpath(QLatin1String("."));
 
-    ObjectTypesWriter writer;
-    if (!writer.writeObjectTypes(objectTypesFile, objectTypes)) {
+    ObjectTypesSerializer serializer;
+    if (!serializer.writeObjectTypes(objectTypesFile, objectTypes)) {
         QMessageBox::critical(this, tr("Error Writing Object Types"),
                               tr("Error writing to %1:\n%2")
                               .arg(prefs->objectTypesFile(),
-                                   writer.errorString()));
+                                   serializer.errorString()));
     }
 }
 
@@ -327,7 +329,7 @@ void ObjectTypesEditor::chooseObjectTypesFile()
     const QString fileName =
             QFileDialog::getOpenFileName(this, tr("Choose Object Types File"),
                                          startPath,
-                                         tr("Object Types files (*.xml)"),
+                                         tr("Object Types files (*.xml *.json)"),
                                          nullptr,
                                          QFileDialog::DontConfirmOverwrite);
 
@@ -339,12 +341,11 @@ void ObjectTypesEditor::chooseObjectTypesFile()
     ObjectTypes objectTypes;
 
     if (QFile::exists(fileName)) {
-        ObjectTypesReader reader;
-        objectTypes = reader.readObjectTypes(fileName);
+        ObjectTypesSerializer serializer;
 
-        if (!reader.errorString().isEmpty()) {
+        if (!serializer.readObjectTypes(fileName, objectTypes)) {
             QMessageBox::critical(this, tr("Error Reading Object Types"),
-                                  reader.errorString());
+                                  serializer.errorString());
             return;
         }
     }
@@ -361,16 +362,16 @@ void ObjectTypesEditor::importObjectTypes()
     const QString fileName =
             QFileDialog::getOpenFileName(this, tr("Import Object Types"),
                                          lastPath,
-                                         tr("Object Types files (*.xml)"));
+                                         tr("Object Types files (*.xml *.json)"));
     if (fileName.isEmpty())
         return;
 
     prefs->setLastPath(Preferences::ObjectTypesFile, fileName);
 
-    ObjectTypesReader reader;
-    const ObjectTypes objectTypes = reader.readObjectTypes(fileName);
+    ObjectTypesSerializer serializer;
+    ObjectTypes objectTypes;
 
-    if (reader.errorString().isEmpty()) {
+    if (serializer.readObjectTypes(fileName, objectTypes)) {
         ObjectTypes currentTypes = mObjectTypesModel->objectTypes();
         for (const ObjectType &type : objectTypes) {
             auto it = std::find_if(currentTypes.begin(), currentTypes.end(), [&type](ObjectType &existingType) {
@@ -388,7 +389,7 @@ void ObjectTypesEditor::importObjectTypes()
         mObjectTypesModel->setObjectTypes(currentTypes);
     } else {
         QMessageBox::critical(this, tr("Error Reading Object Types"),
-                              reader.errorString());
+                              serializer.errorString());
     }
 
     applyObjectTypes();
@@ -405,16 +406,16 @@ void ObjectTypesEditor::exportObjectTypes()
     const QString fileName =
             QFileDialog::getSaveFileName(this, tr("Export Object Types"),
                                          lastPath,
-                                         tr("Object Types files (*.xml)"));
+                                         tr("Object Types files (*.xml *.json)"));
     if (fileName.isEmpty())
         return;
 
     prefs->setLastPath(Preferences::ObjectTypesFile, fileName);
 
-    ObjectTypesWriter writer;
-    if (!writer.writeObjectTypes(fileName, prefs->objectTypes())) {
+    ObjectTypesSerializer serializer;
+    if (!serializer.writeObjectTypes(fileName, prefs->objectTypes())) {
         QMessageBox::critical(this, tr("Error Writing Object Types"),
-                              writer.errorString());
+                              serializer.errorString());
     }
 }
 
