@@ -39,7 +39,7 @@ PaintTileLayer::PaintTileLayer(MapDocument *mapDocument,
     : QUndoCommand(parent)
     , mMapDocument(mapDocument)
     , mTarget(target)
-    , mSource(static_cast<TileLayer*>(source->clone()))
+    , mSource(source->clone())
     , mX(x)
     , mY(y)
     , mPaintedRegion(source->region().translated(QPoint(x, y) - source->position()))
@@ -61,7 +61,7 @@ PaintTileLayer::PaintTileLayer(MapDocument *mapDocument,
     : QUndoCommand(parent)
     , mMapDocument(mapDocument)
     , mTarget(target)
-    , mSource(static_cast<TileLayer*>(source->clone()))
+    , mSource(source->clone())
     , mX(x)
     , mY(y)
     , mPaintedRegion(paintRegion)
