@@ -18,8 +18,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef TILED_INTERNAL_PLUGINLISTMODEL_H
-#define TILED_INTERNAL_PLUGINLISTMODEL_H
+#pragma once
 
 #include <QAbstractListModel>
 #include <QIcon>
@@ -49,5 +48,3 @@ private:
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // TILED_INTERNAL_PLUGINLISTMODEL_H
