@@ -1,6 +1,7 @@
 /*
  * preferences.cpp
  * Copyright 2009-2011, Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
+ * Copyright 2016, Mamed Ibrahimov <ibramlab@gmail.com>
  *
  * This file is part of Tiled.
  *
@@ -24,9 +25,11 @@
 #include "languagemanager.h"
 #include "mapdocument.h"
 #include "pluginmanager.h"
+#include "savefile.h"
 #include "tilesetmanager.h"
 
 #include <QDebug>
+#include <QDir>
 #include <QFileInfo>
 #include <QSettings>
 #include <QStandardPaths>
@@ -59,11 +62,14 @@ Preferences::Preferences()
     mMapRenderOrder = static_cast<Map::RenderOrder>
             (intValue("MapRenderOrder", Map::RightDown));
     mDtdEnabled = boolValue("DtdEnabled");
+    mSafeSavingEnabled = boolValue("SafeSavingEnabled", true);
     mReloadTilesetsOnChange = boolValue("ReloadTilesets", true);
     mStampsDirectory = stringValue("StampsDirectory");
     mObjectTypesFile = stringValue("ObjectTypesFile");
     mSettings->endGroup();
 
+    SaveFile::setSafeSavingEnabled(mSafeSavingEnabled);
+
     // Retrieve interface settings
     mSettings->beginGroup(QLatin1String("Interface"));
     mShowGrid = boolValue("ShowGrid", true);
@@ -71,6 +77,7 @@ Preferences::Preferences()
     mShowTileAnimations = boolValue("ShowTileAnimations", true);
     mSnapToGrid = boolValue("SnapToGrid");
     mSnapToFineGrid = boolValue("SnapToFineGrid");
+    mSnapToPixels = boolValue("SnapToPixels");
     mGridColor = colorValue("GridColor", Qt::black);
     mGridFine = intValue("GridFine", 4);
     mObjectLineWidth = realValue("ObjectLineWidth", 2);
@@ -87,16 +94,22 @@ Preferences::Preferences()
     mApplicationStyle = static_cast<ApplicationStyle>
             (intValue("ApplicationStyle", TiledStyle));
 #endif
+
+    // Backwards compatibility check since 'FusionStyle' was removed from the
+    // preferences dialog.
+    if (mApplicationStyle == FusionStyle)
+        mApplicationStyle = TiledStyle;
+
     mBaseColor = colorValue("BaseColor", Qt::lightGray);
     mSelectionColor = colorValue("SelectionColor", QColor(48, 140, 198));
     mSettings->endGroup();
 
     // Retrieve defined object types
-    ObjectTypesReader objectTypesReader;
-    mObjectTypes = objectTypesReader.readObjectTypes(objectTypesFile());
+    ObjectTypesSerializer objectTypesSerializer;
+    bool success = objectTypesSerializer.readObjectTypes(objectTypesFile(), mObjectTypes);
 
     // For backwards compatibilty, read in object types from settings
-    if (!objectTypesReader.errorString().isEmpty()) {
+    if (!success) {
         mSettings->beginGroup(QLatin1String("ObjectTypes"));
         const QStringList names = mSettings->value(QLatin1String("Names")).toStringList();
         const QStringList colors = mSettings->value(QLatin1String("Colors")).toStringList();
@@ -137,6 +150,7 @@ Preferences::Preferences()
     // Keeping track of some usage information
     mSettings->beginGroup(QLatin1String("Install"));
     mFirstRun = mSettings->value(QLatin1String("FirstRun")).toDate();
+    mPatreonDialogTime = mSettings->value(QLatin1String("PatreonDialogTime")).toDate();
     mRunCount = intValue("RunCount", 0) + 1;
     mIsPatron = boolValue("IsPatron");
     mCheckForUpdates = boolValue("CheckForUpdates");
@@ -144,6 +158,13 @@ Preferences::Preferences()
         mFirstRun = QDate::currentDate();
         mSettings->setValue(QLatin1String("FirstRun"), mFirstRun.toString(Qt::ISODate));
     }
+    if (!mSettings->contains(QLatin1String("PatreonDialogTime"))) {
+        mPatreonDialogTime = mFirstRun.addMonths(1);
+        const QDate today(QDate::currentDate());
+        if (mPatreonDialogTime.daysTo(today) >= 0)
+            mPatreonDialogTime = today.addDays(2);
+        mSettings->setValue(QLatin1String("PatreonDialogTime"), mPatreonDialogTime.toString(Qt::ISODate));
+    }
     mSettings->setValue(QLatin1String("RunCount"), mRunCount);
     mSettings->endGroup();
 
@@ -253,6 +274,16 @@ void Preferences::setSnapToFineGrid(bool snapToFineGrid)
     emit snapToFineGridChanged(mSnapToFineGrid);
 }
 
+void Preferences::setSnapToPixels(bool snapToPixels)
+{
+    if (mSnapToPixels == snapToPixels)
+        return;
+
+    mSnapToPixels = snapToPixels;
+    mSettings->setValue(QLatin1String("Interface/SnapToPixels"), mSnapToPixels);
+    emit snapToPixelsChanged(mSnapToPixels);
+}
+
 void Preferences::setGridColor(QColor gridColor)
 {
     if (mGridColor == gridColor)
@@ -345,6 +376,13 @@ void Preferences::setDtdEnabled(bool enabled)
     mSettings->setValue(QLatin1String("Storage/DtdEnabled"), enabled);
 }
 
+void Preferences::setSafeSavingEnabled(bool enabled)
+{
+    mSafeSavingEnabled = enabled;
+    mSettings->setValue(QLatin1String("Storage/SafeSavingEnabled"), enabled);
+    SaveFile::setSafeSavingEnabled(enabled);
+}
+
 QString Preferences::language() const
 {
     return mLanguage;
@@ -360,6 +398,7 @@ void Preferences::setLanguage(const QString &language)
                         mLanguage);
 
     LanguageManager::instance()->installTranslators();
+    emit languageChanged();
 }
 
 bool Preferences::reloadTilesetsOnChange() const
@@ -436,9 +475,9 @@ QString Preferences::lastPath(FileType fileType) const
 
     if (path.isEmpty()) {
         DocumentManager *documentManager = DocumentManager::instance();
-        MapDocument *mapDocument = documentManager->currentDocument();
-        if (mapDocument)
-            path = QFileInfo(mapDocument->fileName()).path();
+        Document *document = documentManager->currentDocument();
+        if (document)
+            path = QFileInfo(document->fileName()).path();
     }
 
     if (path.isEmpty()) {
@@ -454,6 +493,9 @@ QString Preferences::lastPath(FileType fileType) const
  */
 void Preferences::setLastPath(FileType fileType, const QString &path)
 {
+    if (path.isEmpty())
+        return;
+
     mSettings->setValue(lastPathKey(fileType), path);
 }
 
@@ -489,6 +531,69 @@ void Preferences::setPatron(bool isPatron)
     emit isPatronChanged();
 }
 
+bool Preferences::shouldShowPatreonDialog() const
+{
+    if (mIsPatron)
+        return false;
+    if (mRunCount < 7)
+        return false;
+    if (!mPatreonDialogTime.isValid())
+        return false;
+
+    return mPatreonDialogTime.daysTo(QDate::currentDate()) >= 0;
+}
+
+void Preferences::setPatreonDialogReminder(const QDate &date)
+{
+    if (date.isValid())
+        setPatron(false);
+    mPatreonDialogTime = date;
+    mSettings->setValue(QLatin1String("Install/PatreonDialogTime"), mPatreonDialogTime.toString(Qt::ISODate));
+}
+
+QStringList Preferences::recentFiles() const
+{
+    QVariant v = mSettings->value(QLatin1String("recentFiles/fileNames"));
+    return v.toStringList();
+}
+
+QString Preferences::fileDialogStartLocation() const
+{
+    QStringList files = recentFiles();
+    return (!files.isEmpty()) ? QFileInfo(files.first()).path() : QString();
+}
+
+/**
+ * Adds the given file to the recent files list.
+ */
+void Preferences::addRecentFile(const QString &fileName)
+{
+    // Remember the file by its absolute file path (not the canonical one,
+    // which avoids unexpected paths when symlinks are involved).
+    const QString absoluteFilePath = QDir::cleanPath(QFileInfo(fileName).absoluteFilePath());
+
+    if (absoluteFilePath.isEmpty())
+        return;
+
+    QStringList files = recentFiles();
+    files.removeAll(absoluteFilePath);
+    files.prepend(absoluteFilePath);
+    while (files.size() > MaxRecentFiles)
+        files.removeLast();
+
+    mSettings->beginGroup(QLatin1String("recentFiles"));
+    mSettings->setValue(QLatin1String("fileNames"), files);
+    mSettings->endGroup();
+
+    emit recentFilesChanged();
+}
+
+void Preferences::clearRecentFiles()
+{
+    mSettings->remove(QLatin1String("recentFiles/fileNames"));
+    emit recentFilesChanged();
+}
+
 void Preferences::setCheckForUpdates(bool on)
 {
     if (mCheckForUpdates == on)
@@ -571,11 +676,7 @@ qreal Preferences::realValue(const char *key, qreal defaultValue) const
 
 static QString dataLocation()
 {
-#if QT_VERSION >= 0x050400
     return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
-#else
-    return QStandardPaths::writableLocation(QStandardPaths::DataLocation);
-#endif
 }
 
 QString Preferences::stampsDirectory() const
