@@ -1,6 +1,7 @@
 /*
  * preferences.h
  * Copyright 2009-2011, Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
+ * Copyright 2016, Mamed Ibrahimov <ibramlab@gmail.com>
  *
  * This file is part of Tiled.
  *
@@ -18,8 +19,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef PREFERENCES_H
-#define PREFERENCES_H
+#pragma once
 
 #include <QColor>
 #include <QDate>
@@ -50,6 +50,7 @@ public:
     bool showTileAnimations() const { return mShowTileAnimations; }
     bool snapToGrid() const { return mSnapToGrid; }
     bool snapToFineGrid() const { return mSnapToFineGrid; }
+    bool snapToPixels() const { return mSnapToPixels; }
     QColor gridColor() const { return mGridColor; }
     int gridFine() const { return mGridFine; }
     qreal objectLineWidth() const { return mObjectLineWidth; }
@@ -90,6 +91,9 @@ public:
     bool dtdEnabled() const;
     void setDtdEnabled(bool enabled);
 
+    bool safeSavingEnabled() const;
+    void setSafeSavingEnabled(bool enabled);
+
     QString language() const;
     void setLanguage(const QString &language);
 
@@ -129,6 +133,14 @@ public:
     bool isPatron() const;
     void setPatron(bool isPatron);
 
+    bool shouldShowPatreonDialog() const;
+    void setPatreonDialogReminder(const QDate &date);
+
+    enum { MaxRecentFiles = 8 };
+    QStringList recentFiles() const;
+    QString fileDialogStartLocation() const;
+    void addRecentFile(const QString &fileName);
+
     bool openLastFilesOnStartup() const;
 
     bool checkForUpdates() const;
@@ -146,6 +158,7 @@ public slots:
     void setShowTileAnimations(bool enabled);
     void setSnapToGrid(bool snapToGrid);
     void setSnapToFineGrid(bool snapToFineGrid);
+    void setSnapToPixels(bool snapToPixels);
     void setGridColor(QColor gridColor);
     void setGridFine(int gridFine);
     void setObjectLineWidth(qreal lineWidth);
@@ -155,12 +168,15 @@ public slots:
     void setOpenLastFilesOnStartup(bool load);
     void setPluginEnabled(const QString &fileName, bool enabled);
 
+    void clearRecentFiles();
+
 signals:
     void showGridChanged(bool showGrid);
     void showTileObjectOutlinesChanged(bool enabled);
     void showTileAnimationsChanged(bool enabled);
     void snapToGridChanged(bool snapToGrid);
     void snapToFineGridChanged(bool snapToFineGrid);
+    void snapToPixelsChanged(bool snapToPixels);
     void gridColorChanged(QColor gridColor);
     void gridFineChanged(int gridFine);
     void objectLineWidthChanged(qreal lineWidth);
@@ -174,12 +190,17 @@ signals:
 
     void useOpenGLChanged(bool useOpenGL);
 
+    void languageChanged();
+
     void objectTypesChanged();
 
     void mapsDirectoryChanged();
     void stampsDirectoryChanged(const QString &stampsDirectory);
 
     void isPatronChanged();
+
+    void recentFilesChanged();
+
     void checkForUpdatesChanged();
 
 private:
@@ -199,6 +220,7 @@ private:
     bool mShowTileAnimations;
     bool mSnapToGrid;
     bool mSnapToFineGrid;
+    bool mSnapToPixels;
     QColor mGridColor;
     int mGridFine;
     qreal mObjectLineWidth;
@@ -213,6 +235,7 @@ private:
     Map::LayerDataFormat mLayerDataFormat;
     Map::RenderOrder mMapRenderOrder;
     bool mDtdEnabled;
+    bool mSafeSavingEnabled;
     QString mLanguage;
     bool mReloadTilesetsOnChange;
     bool mUseOpenGL;
@@ -225,6 +248,7 @@ private:
     QString mObjectTypesFile;
 
     QDate mFirstRun;
+    QDate mPatreonDialogTime;
     int mRunCount;
     bool mIsPatron;
     bool mCheckForUpdates;
@@ -248,6 +272,11 @@ inline QColor Preferences::selectionColor() const
     return mSelectionColor;
 }
 
+inline bool Preferences::safeSavingEnabled() const
+{
+    return mSafeSavingEnabled;
+}
+
 inline Preferences::ObjectLabelVisiblity Preferences::objectLabelVisibility() const
 {
     return mObjectLabelVisibility;
@@ -280,5 +309,3 @@ inline bool Preferences::openLastFilesOnStartup() const
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // PREFERENCES_H
