@@ -18,12 +18,14 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef RENAMELAYER_H
-#define RENAMELAYER_H
+#pragma once
 
 #include <QUndoCommand>
 
 namespace Tiled {
+
+class Layer;
+
 namespace Internal {
 
 class MapDocument;
@@ -32,7 +34,7 @@ class RenameLayer : public QUndoCommand
 {
 public:
     RenameLayer(MapDocument *mapDocument,
-                int layerIndex,
+                Layer *layer,
                 const QString &name);
 
     void undo() override;
@@ -42,11 +44,9 @@ private:
     void swapName();
 
     MapDocument *mMapDocument;
-    int mLayerIndex;
+    Layer *mLayer;
     QString mName;
 };
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // RENAMELAYER_H
