@@ -18,11 +18,12 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef RESIZEDIALOG_H
-#define RESIZEDIALOG_H
+#pragma once
 
 #include <QDialog>
 
+#include <functional>
+
 namespace Ui {
 class ResizeDialog;
 }
@@ -41,11 +42,13 @@ public:
 
     void setOldSize(const QSize &size);
 
-    const QSize &newSize() const;
-    const QPoint &offset() const;
+    QSize newSize() const;
+    QPoint offset() const;
 
     bool removeObjects() const;
 
+    void setMiniMapRenderer(std::function<QImage (QSize)> renderer);
+
 private slots:
     void removeObjectsToggled(bool removeObjects);
     void updateOffsetBounds(const QRect &bounds);
@@ -56,5 +59,3 @@ private:
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // RESIZEDIALOG_H
