@@ -18,8 +18,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef ROTATEMAPOBJECT_H
-#define ROTATEMAPOBJECT_H
+#pragma once
 
 #include <QUndoCommand>
 
@@ -55,5 +54,3 @@ private:
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // ROTATEMAPOBJECT_H
