@@ -71,7 +71,7 @@ void StampBrush::tilePositionChanged(const QPoint &pos)
             editedRegion |= doPaint(Mergeable | SuppressRegionEdited);
         }
 
-        mapDocument()->emitRegionEdited(editedRegion, currentTileLayer());
+        emit mapDocument()->regionEdited(editedRegion, currentTileLayer());
     } else {
         updatePreview();
     }
@@ -199,8 +199,8 @@ void StampBrush::updateRandomList()
         for (int x = 0; x < tileLayer->width(); x++) {
             for (int y = 0; y < tileLayer->height(); y++) {
                 const Cell &cell = tileLayer->cellAt(x, y);
-                if (!cell.isEmpty())
-                    mRandomCellPicker.add(cell, cell.tile->probability());
+                if (const Tile *tile = cell.tile())
+                    mRandomCellPicker.add(cell, tile->probability());
             }
         }
     }
@@ -263,6 +263,16 @@ void StampBrush::endCapture()
                              map->tileWidth(),
                              map->tileHeight());
 
+        //gets if the relative stagger should be the same as the base layer
+        int staggerIndexOffSet;
+        if (tileLayer->map()->staggerAxis() == Map::StaggerX)
+            staggerIndexOffSet = captured.x() % 2;
+        else
+            staggerIndexOffSet = captured.y() % 2;
+
+        stamp->setStaggerAxis(map->staggerAxis());
+        stamp->setStaggerIndex((Map::StaggerIndex)((map->staggerIndex() + staggerIndexOffSet) % 2));
+
         // Add tileset references to map
         foreach (const SharedTileset &tileset, capture->usedTilesets())
             stamp->addTileset(tileset);
@@ -329,7 +339,7 @@ QRegion StampBrush::doPaint(int flags)
 
     QRegion editedRegion = preview->region();
     if (! (flags & SuppressRegionEdited))
-        mapDocument()->emitRegionEdited(editedRegion, tileLayer);
+        emit mapDocument()->regionEdited(editedRegion, tileLayer);
     return editedRegion;
 }
 
@@ -358,7 +368,7 @@ void StampBrush::drawPreviewLayer(const QVector<QPoint> &list)
             return;
 
         QRegion paintedRegion;
-        foreach (const QPoint p, list)
+        for (const QPoint p : list)
             paintedRegion += QRect(p, QSize(1, 1));
 
         QRect bounds = paintedRegion.boundingRect();
@@ -380,6 +390,7 @@ void StampBrush::drawPreviewLayer(const QVector<QPoint> &list)
         QRegion paintedRegion;
         QVector<PaintOperation> operations;
         QHash<TileLayer *, QRegion> regionCache;
+        QHash<TileLayer *, TileLayer *> shiftedCopies;
 
         for (const QPoint &p : list) {
             const TileStampVariation variation = mStamp.randomVariation();
@@ -387,6 +398,55 @@ void StampBrush::drawPreviewLayer(const QVector<QPoint> &list)
 
             TileLayer *stamp = variation.tileLayer();
 
+            Map::StaggerAxis mapStaggerAxis = mapDocument()->map()->staggerAxis();
+            Map::StaggerIndex mapStaggerIndex = mapDocument()->map()->staggerIndex();
+            Map::StaggerIndex stampStaggerIndex = variation.map->staggerIndex();
+
+            //if staggered map, makes sure stamp stays the same
+            if (mapDocument()->map()->isStaggered()
+                    && ((mapStaggerAxis == Map::StaggerY)? stamp->height() > 1 : stamp->width() > 1)) {
+
+                if (mapStaggerAxis == Map::StaggerY) {
+                    bool topIsOdd = (p.y() - stamp->height() / 2) & 1;
+
+                    if ((stampStaggerIndex == mapStaggerIndex) == topIsOdd) {
+                        TileLayer *shiftedStamp = shiftedCopies.value(stamp);
+                        if (!shiftedStamp) {
+                            shiftedStamp = stamp->clone();
+                            shiftedCopies.insert(stamp, shiftedStamp);
+
+                            shiftedStamp->resize(QSize(shiftedStamp->width() + 1, shiftedStamp->height()), QPoint());
+
+                            for (int y = (stampStaggerIndex + 1) & 1; y < shiftedStamp->height(); y += 2) {
+                                for (int x = shiftedStamp->width() - 2; x >= 0; --x)
+                                    shiftedStamp->setCell(x + 1, y, shiftedStamp->cellAt(x, y));
+                                shiftedStamp->setCell(0, y, Cell());
+                            }
+                        }
+                        stamp = shiftedStamp;
+                    }
+                } else {
+                    bool leftIsOdd = (p.x() - stamp->width() / 2) & 1;
+
+                    if ((stampStaggerIndex == mapStaggerIndex) == leftIsOdd) {
+                        TileLayer *shiftedStamp = shiftedCopies.value(stamp);
+                        if (!shiftedStamp) {
+                            shiftedStamp = stamp->clone();
+                            shiftedCopies.insert(stamp, shiftedStamp);
+
+                            shiftedStamp->resize(QSize(shiftedStamp->width(), shiftedStamp->height() + 1), QPoint());
+
+                            for (int x = (stampStaggerIndex + 1) & 1; x < shiftedStamp->width(); x += 2) {
+                                for (int y = shiftedStamp->height() - 2; y >= 0; --y)
+                                    shiftedStamp->setCell(x, y + 1, shiftedStamp->cellAt(x, y));
+                                shiftedStamp->setCell(x, 0, Cell());
+                            }
+                        }
+                        stamp = shiftedStamp;
+                    }
+                }
+            }
+
             QRegion stampRegion;
             if (regionCache.contains(stamp)) {
                 stampRegion = regionCache.value(stamp);
@@ -416,6 +476,8 @@ void StampBrush::drawPreviewLayer(const QVector<QPoint> &list)
         for (const PaintOperation &op : operations)
             preview->merge(op.pos - bounds.topLeft(), op.stamp);
 
+        qDeleteAll(shiftedCopies);
+
         mPreviewLayer = preview;
     }
 }
