@@ -76,11 +76,15 @@ static QRectF cellRect(const MapRenderer &renderer,
                        const Cell &cell,
                        const QPointF &tileCoords)
 {
-    QPointF pixelCoords = renderer.tileToScreenCoords(tileCoords);
-    QPointF offset = cell.tile->tileset()->tileOffset();
-    QSize size = cell.tile->size();
+    const Tile *tile = cell.tile();
+    if (!tile)
+        return QRectF();
 
-    if (cell.flippedAntiDiagonally)
+    QPointF pixelCoords = renderer.tileToScreenCoords(tileCoords);
+    QPointF offset = tile->offset();
+    QSize size = tile->size();
+
+    if (cell.flippedAntiDiagonally())
         std::swap(size.rwidth(), size.rheight());
 
     // This is a correction needed because tileToScreenCoords does not return
@@ -102,7 +106,7 @@ static QRect computeMapRect(const MapRenderer &renderer)
             continue;
 
         const TileLayer *tileLayer = static_cast<const TileLayer*>(layer);
-        const QPointF offset = tileLayer->offset();
+        const QPointF offset = tileLayer->totalOffset();
 
         for (int y = 0; y < tileLayer->height(); ++y) {
             for (int x = 0; x < tileLayer->width(); ++x) {
@@ -156,12 +160,15 @@ QImage ThumbnailRenderer::render(const QSize &size) const
 
     mRenderer->setPainterScale(scale);
 
-    for (const Layer *layer : mMap->layers()) {
-        if (mVisibleLayersOnly && !layer->isVisible())
+    LayerIterator iterator(mMap);
+    while (const Layer *layer = iterator.next()) {
+        if (mVisibleLayersOnly && layer->isHidden())
             continue;
 
-        painter.setOpacity(layer->opacity());
-        painter.translate(layer->offset());
+        const auto offset = layer->totalOffset();
+
+        painter.setOpacity(layer->effectiveOpacity());
+        painter.translate(offset);
 
         switch (layer->layerType()) {
         case Layer::TileLayerType: {
@@ -201,9 +208,13 @@ QImage ThumbnailRenderer::render(const QSize &size) const
             mRenderer->drawImageLayer(&painter, imageLayer);
             break;
         }
+        case Layer::GroupLayerType: {
+            // Recursion handled by LayerIterator
+            break;
+        }
         }
 
-        painter.translate(-layer->offset());
+        painter.translate(-offset);
     }
 
     return image;
