@@ -20,11 +20,11 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef TILESETDOCK_H
-#define TILESETDOCK_H
+#pragma once
 
 #include "tileset.h"
 
+#include <QAbstractItemModel>
 #include <QDockWidget>
 #include <QList>
 #include <QMap>
@@ -49,7 +49,10 @@ class Tileset;
 
 namespace Internal {
 
+class Document;
 class MapDocument;
+class TilesetDocument;
+class TilesetDocumentsFilterModel;
 class TilesetView;
 class TileStamp;
 class Zoomable;
@@ -96,9 +99,7 @@ signals:
     /**
      * Emitted when files are dropped at the tileset dock.
      */
-    void tilesetsDropped(const QStringList &paths);
-
-    void newTileset();
+    void localFilesDropped(const QStringList &paths);
 
 protected:
     void changeEvent(QEvent *e) override;
@@ -115,47 +116,53 @@ private slots:
     void updateCurrentTiles();
     void indexPressed(const QModelIndex &index);
 
-    void tilesetAdded(int index, Tileset *tileset);
     void tilesetChanged(Tileset *tileset);
-    void tilesetRemoved(Tileset *tileset);
-    void tilesetMoved(int from, int to);
-    void tilesetReplaced(int index, Tileset *tileset);
-    void tilesetNameChanged(Tileset *tileset);
+    void tilesetFileNameChanged(const QString &fileName);
 
     void tileImageSourceChanged(Tile *tile);
     void tileAnimationChanged(Tile *tile);
 
     void removeTileset();
     void removeTileset(int index);
-    void moveTileset(int from, int to);
 
-    void editTilesetProperties();
-    void importTileset();
+    void newTileset();
+    void editTileset();
+    void embedTileset();
     void exportTileset();
 
-    void editTerrain();
-    void addTiles();
-    void removeTiles();
-
-    void documentAboutToClose(MapDocument *mapDocument);
-
     void refreshTilesetMenu();
 
+    void swapTiles(Tile *tileA, Tile *tileB);
+    void changeSelectedMapObjectsTile(Tile *tile);
+
 private:
     void setCurrentTile(Tile *tile);
     void setCurrentTiles(TileLayer *tiles);
     void retranslateUi();
 
+    void onTilesetRowsInserted(const QModelIndex &parent, int first, int last);
+    void onTilesetRowsAboutToBeRemoved(const QModelIndex &parent, int first, int last);
+    void onTilesetRowsMoved(const QModelIndex &parent, int start, int end, const QModelIndex &destination, int row);
+    void onTilesetLayoutChanged(const QList<QPersistentModelIndex> &parents, QAbstractItemModel::LayoutChangeHint hint);
+    void onTilesetDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight);
+
+    void onTabMoved(int from, int to);
+
     Tileset *currentTileset() const;
     TilesetView *currentTilesetView() const;
     TilesetView *tilesetViewAt(int index) const;
 
+    void createTilesetView(int index, TilesetDocument *tilesetDocument);
+    void deleteTilesetView(int index);
+    void moveTilesetView(int from, int to);
     void setupTilesetModel(TilesetView *view, Tileset *tileset);
 
     MapDocument *mMapDocument;
 
     // Shared tileset references because the dock wants to add new tiles
     QVector<SharedTileset> mTilesets;
+    QList<TilesetDocument *> mTilesetDocuments;
+    TilesetDocumentsFilterModel *mTilesetDocumentsFilterModel;
 
     QTabBar *mTabBar;
     QStackedWidget *mViewStack;
@@ -165,22 +172,16 @@ private:
     const Terrain *mTerrain;
 
     QAction *mNewTileset;
-    QAction *mImportTileset;
+    QAction *mEmbedTileset;
     QAction *mExportTileset;
-    QAction *mPropertiesTileset;
+    QAction *mEditTileset;
     QAction *mDeleteTileset;
-    QAction *mEditTerrain;
-    QAction *mAddTiles;
-    QAction *mRemoveTiles;
-
-    QMap<MapDocument *, QString> mCurrentTilesets;
 
     QToolButton *mTilesetMenuButton;
     QMenu *mTilesetMenu; //opens on click of mTilesetMenu
     QActionGroup *mTilesetActionGroup;
     QSignalMapper *mTilesetMenuMapper; //needed due to dynamic content
 
-    Zoomable *mZoomable;
     QComboBox *mZoomComboBox;
 
     bool mEmittingStampCaptured;
@@ -189,5 +190,3 @@ private:
 
 } // namespace Internal
 } // namespace Tiled
-
-#endif // TILESETDOCK_H
