@@ -22,15 +22,18 @@
 
 #include "changetileterrain.h"
 #include "map.h"
-#include "mapdocument.h"
 #include "preferences.h"
+#include "stylehelper.h"
+#include "terrain.h"
 #include "tile.h"
 #include "tileset.h"
+#include "tilesetdocument.h"
 #include "tilesetmodel.h"
 #include "utils.h"
 #include "zoomable.h"
 
 #include <QAbstractItemDelegate>
+#include <QApplication>
 #include <QCoreApplication>
 #include <QGesture>
 #include <QGestureEvent>
@@ -100,8 +103,6 @@ static void paintCorners(QPainter *painter,
                          unsigned corners,
                          const QRect &rect)
 {
-    // FIXME: This only works right for orthogonal maps right now
-
     const int hx = rect.width() / 2;
     const int hy = rect.height() / 2;
 
@@ -172,6 +173,17 @@ static void paintCorners(QPainter *painter,
     }
 }
 
+static void setCosmeticPen(QPainter *painter, const QBrush &brush, qreal width)
+{
+#if QT_VERSION >= 0x050600
+    QPen pen(brush, width * painter->device()->devicePixelRatioF());
+#else
+    QPen pen(brush, width * painter->device()->devicePixelRatio());
+#endif
+    pen.setCosmetic(true);
+    painter->setPen(pen);
+}
+
 static void paintTerrainOverlay(QPainter *painter,
                                 unsigned terrain,
                                 int terrainTypeId,
@@ -184,7 +196,7 @@ static void paintTerrainOverlay(QPainter *painter,
 
     // Draw the "any terrain" background
     painter->setBrush(QColor(128, 128, 128, 100));
-    painter->setPen(QPen(Qt::gray, 2));
+    setCosmeticPen(painter, Qt::gray, 2);
     paintCorners(painter, invertCorners(terrainCorners(terrain, -1)), rect);
 
     if (terrainTypeId != -1) {
@@ -193,19 +205,41 @@ static void paintTerrainOverlay(QPainter *painter,
         // Draw the shadow
         painter->translate(1, 1);
         painter->setBrush(Qt::NoBrush);
-        painter->setPen(QPen(Qt::black, 2));
+        setCosmeticPen(painter, Qt::black, 2);
         paintCorners(painter, corners, rect);
 
         // Draw the foreground
         painter->translate(-1, -1);
         painter->setBrush(QColor(color.red(), color.green(), color.blue(), 100));
-        painter->setPen(QPen(color, 2));
+        setCosmeticPen(painter, color, 2);
         paintCorners(painter, corners, rect);
     }
 
     painter->restore();
 }
 
+static QTransform tilesetGridTransform(const Tileset &tileset, QPoint tileCenter)
+{
+    QTransform transform;
+
+    if (tileset.orientation() == Tileset::Isometric) {
+        const QSize gridSize = tileset.gridSize();
+
+        transform.translate(tileCenter.x(), tileCenter.y());
+
+        const auto ratio = (double) gridSize.height() / gridSize.width();
+        const auto scaleX = 1.0 / sqrt(2.0);
+        const auto scaleY = scaleX * ratio;
+        transform.scale(scaleX, scaleY);
+
+        transform.rotate(45.0);
+
+        transform.translate(-tileCenter.x(), -tileCenter.y());
+    }
+
+    return transform;
+}
+
 void TileDelegate::paint(QPainter *painter,
                          const QStyleOptionViewItem &option,
                          const QModelIndex &index) const
@@ -282,20 +316,25 @@ void TileDelegate::paint(QPainter *painter,
         painter->restore();
     }
 
+    const auto highlight = option.palette.highlight();
+
     // Overlay with highlight color when selected
     if (option.state & QStyle::State_Selected) {
         const qreal opacity = painter->opacity();
         painter->setOpacity(0.5);
-        painter->fillRect(targetRect, option.palette.highlight());
+        painter->fillRect(targetRect, highlight);
         painter->setOpacity(opacity);
     }
 
     if (mTilesetView->isEditTerrain()) {
+        painter->save();
+        painter->setTransform(tilesetGridTransform(*tile->tileset(), targetRect.center()), true);
+
         const unsigned terrain = tile->terrain();
 
         paintTerrainOverlay(painter, terrain,
                             mTilesetView->terrainId(), targetRect,
-                            option.palette.highlight().color());
+                            highlight.color());
 
         // Overlay with terrain corner indication when hovered
         if (index == mTilesetView->hoveredIndex()) {
@@ -311,13 +350,14 @@ void TileDelegate::paint(QPainter *painter,
             painter->setBrush(option.palette.highlight());
             painter->setClipRect(targetRect);
             painter->setRenderHint(QPainter::Antialiasing);
-            QPen pen(option.palette.highlight().color().darker(), 2);
-            painter->setPen(pen);
+            setCosmeticPen(painter, highlight.color().darker(), 2);
             painter->drawEllipse(pos,
                                  targetRect.width() / 3,
                                  targetRect.height() / 3);
             painter->restore();
         }
+
+        painter->restore();
     }
 }
 
@@ -354,12 +394,12 @@ QSize TileDelegate::sizeHint(const QStyleOptionViewItem & /* option */,
 
 TilesetView::TilesetView(QWidget *parent)
     : QTableView(parent)
-    , mZoomable(nullptr)
-    , mMapDocument(nullptr)
+    , mZoomable(new Zoomable(this))
+    , mTilesetDocument(nullptr)
     , mMarkAnimatedTiles(true)
     , mEditTerrain(false)
     , mEraseTerrain(false)
-    , mTerrainId(-1)
+    , mTerrain(nullptr)
     , mHoveredCorner(0)
     , mTerrainChanged(false)
     , mHandScrolling(false)
@@ -369,6 +409,7 @@ TilesetView::TilesetView(QWidget *parent)
     setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
     setItemDelegate(new TileDelegate(this, this));
     setShowGrid(false);
+    setTabKeyNavigation(false);
 
     QHeaderView *hHeader = horizontalHeader();
     QHeaderView *vHeader = verticalHeader();
@@ -388,18 +429,23 @@ TilesetView::TilesetView(QWidget *parent)
 
     grabGesture(Qt::PinchGesture);
 
-    connect(prefs, SIGNAL(showTilesetGridChanged(bool)),
-            SLOT(setDrawGrid(bool)));
+    connect(prefs, &Preferences::showTilesetGridChanged,
+            this, &TilesetView::setDrawGrid);
+
+    connect(StyleHelper::instance(), &StyleHelper::styleApplied,
+            this, &TilesetView::updateBackgroundColor);
+
+    connect(mZoomable, SIGNAL(scaleChanged(qreal)), SLOT(adjustScale()));
 }
 
-void TilesetView::setMapDocument(MapDocument *mapDocument)
+void TilesetView::setTilesetDocument(TilesetDocument *tilesetDocument)
 {
-    mMapDocument = mapDocument;
+    mTilesetDocument = tilesetDocument;
 }
 
 QSize TilesetView::sizeHint() const
 {
-    return QSize(130, 100);
+    return Utils::dpiScaled(QSize(130, 100));
 }
 
 int TilesetView::sizeHintForColumn(int column) const
@@ -428,21 +474,15 @@ int TilesetView::sizeHintForRow(int row) const
     return qRound(tileHeight * scale()) + (mDrawGrid ? 1 : 0);
 }
 
-void TilesetView::setZoomable(Zoomable *zoomable)
-{
-    if (mZoomable)
-        mZoomable->disconnect(this);
-
-    if (zoomable)
-        connect(zoomable, SIGNAL(scaleChanged(qreal)), SLOT(adjustScale()));
-
-    mZoomable = zoomable;
-    adjustScale();
-}
-
 qreal TilesetView::scale() const
 {
-    return mZoomable ? mZoomable->scale() : 1;
+    return mZoomable->scale();
+}
+
+void TilesetView::setModel(QAbstractItemModel *model)
+{
+    QTableView::setModel(model);
+    updateBackgroundColor();
 }
 
 void TilesetView::setMarkAnimatedTiles(bool enabled)
@@ -456,15 +496,40 @@ void TilesetView::setMarkAnimatedTiles(bool enabled)
 
 bool TilesetView::event(QEvent *event)
 {
-    if (mZoomable && event->type() == QEvent::Gesture) {
+    if (event->type() == QEvent::Gesture) {
         QGestureEvent *gestureEvent = static_cast<QGestureEvent *>(event);
         if (QGesture *gesture = gestureEvent->gesture(Qt::PinchGesture))
             mZoomable->handlePinchGesture(static_cast<QPinchGesture *>(gesture));
+    } else if (event->type() == QEvent::ShortcutOverride) {
+        auto keyEvent = static_cast<QKeyEvent*>(event);
+        if (Utils::isZoomInShortcut(keyEvent) ||
+                Utils::isZoomOutShortcut(keyEvent) ||
+                Utils::isResetZoomShortcut(keyEvent)) {
+            event->accept();
+            return true;
+        }
     }
 
     return QTableView::event(event);
 }
 
+void TilesetView::keyPressEvent(QKeyEvent *event)
+{
+    if (Utils::isZoomInShortcut(event)) {
+        mZoomable->zoomIn();
+        return;
+    }
+    if (Utils::isZoomOutShortcut(event)) {
+        mZoomable->zoomOut();
+        return;
+    }
+    if (Utils::isResetZoomShortcut(event)) {
+        mZoomable->resetZoom();
+        return;
+    }
+    return QTableView::keyPressEvent(event);
+}
+
 void TilesetView::setEditTerrain(bool enabled)
 {
     if (mEditTerrain == enabled)
@@ -475,12 +540,21 @@ void TilesetView::setEditTerrain(bool enabled)
     viewport()->update();
 }
 
-void TilesetView::setTerrainId(int terrainId)
+/**
+ * The id of the terrain currently being specified. Returns -1 when no terrain
+ * is set (used for erasing terrain info).
+ */
+int TilesetView::terrainId() const
 {
-    if (mTerrainId == terrainId)
+     return mTerrain ? mTerrain->id() : -1;
+}
+
+void TilesetView::setTerrain(const Terrain *terrain)
+{
+    if (mTerrain == terrain)
         return;
 
-    mTerrainId = terrainId;
+    mTerrain = terrain;
     if (mEditTerrain)
         viewport()->update();
 }
@@ -536,9 +610,12 @@ void TilesetView::mouseMoveEvent(QMouseEvent *event)
     if (hoveredIndex.isValid()) {
         const QPoint center = visualRect(hoveredIndex).center();
 
-        if (pos.x() > center.x())
+        const auto t = tilesetGridTransform(*tilesetDocument()->tileset(), center);
+        const auto mappedPos = t.inverted().map(pos);
+
+        if (mappedPos.x() > center.x())
             hoveredCorner += 1;
-        if (pos.y() > center.y())
+        if (mappedPos.y() > center.y())
             hoveredCorner += 2;
     }
 
@@ -589,8 +666,7 @@ void TilesetView::leaveEvent(QEvent *event)
  */
 void TilesetView::wheelEvent(QWheelEvent *event)
 {
-    if (mZoomable &&
-            event->modifiers() & Qt::ControlModifier &&
+    if (event->modifiers() & Qt::ControlModifier &&
             event->orientation() == Qt::Vertical)
     {
         mZoomable->handleWheelDelta(event->delta());
@@ -612,7 +688,6 @@ void TilesetView::contextMenuEvent(QContextMenuEvent *event)
 
     Tile *tile = model->tileAt(index);
 
-    const bool isExternal = model->tileset()->isExternal();
     QMenu menu;
 
     QIcon propIcon(QLatin1String(":images/16x16/document-properties.png"));
@@ -626,21 +701,40 @@ void TilesetView::contextMenuEvent(QContextMenuEvent *event)
                                               QItemSelectionModel::Clear);
 
             QAction *addTerrain = menu.addAction(tr("Add Terrain Type"));
-            addTerrain->setEnabled(!isExternal);
-            connect(addTerrain, SIGNAL(triggered()), SLOT(createNewTerrain()));
+            connect(addTerrain, SIGNAL(triggered()), SLOT(addTerrainType()));
 
-            if (mTerrainId != -1) {
+            if (mTerrain) {
                 QAction *setImage = menu.addAction(tr("Set Terrain Image"));
-                setImage->setEnabled(!isExternal);
                 connect(setImage, SIGNAL(triggered()), SLOT(selectTerrainImage()));
             }
-        } else {
+        } else if (mTilesetDocument) {
             QAction *tileProperties = menu.addAction(propIcon,
                                                      tr("Tile &Properties..."));
-            tileProperties->setEnabled(!isExternal);
             Utils::setThemeIcon(tileProperties, "document-properties");
             connect(tileProperties, SIGNAL(triggered()),
                     SLOT(editTileProperties()));
+        } else {
+            // Assuming we're used in the MapEditor
+
+            // Enable "swap" if there are exactly 2 tiles selected
+            bool exactlyTwoTilesSelected =
+                    (selectionModel()->selectedIndexes().size() == 2);
+
+            QAction *swapTilesAction = menu.addAction(tr("&Swap Tiles"));
+            swapTilesAction->setEnabled(exactlyTwoTilesSelected);
+            connect(swapTilesAction, SIGNAL(triggered()),
+                    SLOT(swapTiles()));
+
+            bool onlyOneTileSelected =
+                    (selectionModel()->selectedIndexes().size() == 1);
+
+            // TODO: check that at least one object is selected
+
+            QAction *changeSelectedMapObjectsTileAction =
+                    menu.addAction(tr("&Replace Tile of Selected Objects"));
+            changeSelectedMapObjectsTileAction->setEnabled(onlyOneTileSelected);
+            connect(changeSelectedMapObjectsTileAction, &QAction::triggered,
+                    this, &TilesetView::changeSelectedMapObjectsTile);
         }
 
         menu.addSeparator();
@@ -657,7 +751,7 @@ void TilesetView::contextMenuEvent(QContextMenuEvent *event)
     menu.exec(event->globalPos());
 }
 
-void TilesetView::createNewTerrain()
+void TilesetView::addTerrainType()
 {
     if (Tile *tile = currentTile())
         emit createNewTerrain(tile);
@@ -671,12 +765,38 @@ void TilesetView::selectTerrainImage()
 
 void TilesetView::editTileProperties()
 {
+    Q_ASSERT(mTilesetDocument);
+
     Tile *tile = currentTile();
     if (!tile)
         return;
 
-    mMapDocument->setCurrentObject(tile);
-    mMapDocument->emitEditCurrentObject();
+    mTilesetDocument->setCurrentObject(tile);
+    emit mTilesetDocument->editCurrentObject();
+}
+
+void TilesetView::swapTiles()
+{
+    const QModelIndexList selectedIndexes = selectionModel()->selectedIndexes();
+    if (selectedIndexes.size() != 2)
+        return;
+
+    const TilesetModel *model = tilesetModel();
+    Tile *tile1 = model->tileAt(selectedIndexes[0]);
+    Tile *tile2 = model->tileAt(selectedIndexes[1]);
+
+    if (!tile1 || !tile2)
+        return;
+
+    emit swapTilesRequested(tile1, tile2);
+}
+
+void TilesetView::changeSelectedMapObjectsTile()
+{
+    const QModelIndexList selectedIndexes = selectionModel()->selectedIndexes();
+    const TilesetModel *model = tilesetModel();
+    Tile *tile = model->tileAt(selectedIndexes[0]);
+    emit changeSelectedMapObjectsTileRequested(tile);
 }
 
 void TilesetView::setDrawGrid(bool drawGrid)
@@ -703,13 +823,13 @@ void TilesetView::applyTerrain()
 
     unsigned terrain = setTerrainCorner(tile->terrain(),
                                         mHoveredCorner,
-                                        mEraseTerrain ? 0xFF : mTerrainId);
+                                        mEraseTerrain ? 0xFF : terrainId());
 
     if (terrain == tile->terrain())
         return;
 
-    QUndoCommand *command = new ChangeTileTerrain(mMapDocument, tile, terrain);
-    mMapDocument->undoStack()->push(command);
+    QUndoCommand *command = new ChangeTileTerrain(mTilesetDocument, tile, terrain);
+    mTilesetDocument->undoStack()->push(command);
     mTerrainChanged = true;
 }
 
@@ -719,7 +839,7 @@ void TilesetView::finishTerrainChange()
         return;
 
     // Prevent further merging since mouse was released
-    mMapDocument->undoStack()->push(new ChangeTileTerrain);
+    mTilesetDocument->undoStack()->push(new ChangeTileTerrain);
     mTerrainChanged = false;
 }
 
@@ -741,3 +861,18 @@ void TilesetView::setHandScrolling(bool handScrolling)
     else
         unsetCursor();
 }
+
+void TilesetView::updateBackgroundColor()
+{
+    QColor base = QApplication::palette().dark().color();
+
+    if (TilesetModel *model = tilesetModel()) {
+        Tileset *tileset = model->tileset();
+        if (tileset->backgroundColor().isValid())
+            base = tileset->backgroundColor();
+    }
+
+    QPalette p = palette();
+    p.setColor(QPalette::Base, base);
+    setPalette(p);
+}
