@@ -150,8 +150,9 @@ QSize TileStamp::maxSize() const
 {
     QSize size;
     for (const TileStampVariation &variation : d->variations) {
-        size.setWidth(qMax(size.width(), variation.map->width()));
-        size.setHeight(qMax(size.height(), variation.map->height()));
+        const QSize variationSize = variation.tileLayer()->size();
+        size.setWidth(qMax(size.width(), variationSize.width()));
+        size.setHeight(qMax(size.height(), variationSize.height()));
     }
     return size;
 }
@@ -232,7 +233,24 @@ TileStamp TileStamp::flipped(FlipDirection direction) const
 
     for (const TileStampVariation &variation : flipped.variations()) {
         TileLayer *layer = variation.tileLayer();
-        layer->flip(direction);
+
+        if (variation.map->isStaggered()) {
+            Map::StaggerAxis staggerAxis = variation.map->staggerAxis();
+
+            if (staggerAxis == Map::StaggerY) {
+                if ((direction == FlipVertically && !(layer->height() & 1)) || direction == FlipHorizontally)
+                    variation.map->invertStaggerIndex();
+
+            } else {
+                if ((direction == FlipHorizontally && !(layer->width() & 1)) || direction == FlipVertically)
+                    variation.map->invertStaggerIndex();
+            }
+        }
+
+        if (variation.map->orientation() == Map::Hexagonal)
+            layer->flipHexagonal(direction);
+        else
+            layer->flip(direction);
     }
 
     return flipped;
@@ -249,7 +267,10 @@ TileStamp TileStamp::rotated(RotateDirection direction) const
 
     for (const TileStampVariation &variation : rotated.variations()) {
         TileLayer *layer = variation.tileLayer();
-        layer->rotate(direction);
+        if (variation.map->orientation() == Map::Hexagonal)
+            layer->rotateHexagonal(direction, variation.map);
+        else
+            layer->rotate(direction);
 
         variation.map->setWidth(layer->width());
         variation.map->setHeight(layer->height());
@@ -280,7 +301,7 @@ QJsonObject TileStamp::toJson(const QDir &dir) const
     QJsonArray variations;
     for (const TileStampVariation &variation : d->variations) {
         MapToVariantConverter converter;
-        QVariant mapVariant = converter.toVariant(variation.map, dir);
+        QVariant mapVariant = converter.toVariant(*variation.map, dir);
         QJsonValue mapJson = QJsonValue::fromVariant(mapVariant);
 
         QJsonObject variationJson;
