@@ -27,6 +27,7 @@
 #include "mapdocument.h"
 #include "map.h"
 #include "preferences.h"
+#include "savefile.h"
 #include "stampbrush.h"
 #include "tilelayer.h"
 #include "tileselectiontool.h"
@@ -39,7 +40,6 @@
 #include <QDirIterator>
 #include <QJsonDocument>
 #include <QRegularExpression>
-#include <QSaveFile>
 
 using namespace Tiled;
 using namespace Tiled::Internal;
@@ -111,7 +111,7 @@ static TileStamp stampFromContext(AbstractTool *selectedTool)
     } else if (BucketFillTool *fillTool = dynamic_cast<BucketFillTool*>(selectedTool)) {
         // take the stamp from the fill tool
         stamp = fillTool->stamp();
-    } else if (MapDocument *mapDocument = DocumentManager::instance()->currentDocument()) {
+    } else if (MapDocument *mapDocument = qobject_cast<MapDocument*>(DocumentManager::instance()->currentDocument())) {
         // try making a stamp from the current tile selection
         const TileLayer *tileLayer =
                 dynamic_cast<TileLayer*>(mapDocument->currentLayer());
@@ -165,7 +165,7 @@ void TileStampManager::addVariation(const TileStamp &targetStamp)
     if (stamp == targetStamp) // avoid easy mistake of adding duplicates
         return;
 
-    foreach (const TileStampVariation &variation, stamp.variations())
+    for (const TileStampVariation &variation : stamp.variations())
         mTileStampModel->addVariation(targetStamp, variation);
 }
 
@@ -252,7 +252,7 @@ void TileStampManager::loadStamps()
             QJsonParseError error;
             document = QJsonDocument::fromJson(data, &error);
             if (error.error != QJsonParseError::NoError) {
-                qDebug() << "Failed to parse stamp file:" << qPrintable(error.errorString());
+                qDebug().noquote() << "Failed to parse stamp file:" << error.errorString();
                 continue;
             }
         }
@@ -325,14 +325,14 @@ void TileStampManager::saveStamp(const TileStamp &stamp)
     }
 
     QString filePath = stampsDir.filePath(stamp.fileName());
-    QSaveFile file(filePath);
+    SaveFile file(filePath);
     if (!file.open(QIODevice::WriteOnly)) {
         qDebug() << "Failed to open stamp file for writing" << filePath;
         return;
     }
 
     QJsonObject stampJson = stamp.toJson(QFileInfo(filePath).dir());
-    file.write(QJsonDocument(stampJson).toJson(QJsonDocument::Compact));
+    file.device()->write(QJsonDocument(stampJson).toJson(QJsonDocument::Compact));
 
     if (!file.commit())
         qDebug() << "Failed to write stamp" << filePath;
