@@ -18,8 +18,7 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef UNDOCOMMANDS_H
-#define UNDOCOMMANDS_H
+#pragma once
 
 /**
  * These undo command IDs are used by Qt to determine whether two undo commands
@@ -28,11 +27,8 @@
 enum UndoCommands {
     Cmd_EraseTiles,
     Cmd_PaintTileLayer,
-    Cmd_MoveTileset,
     Cmd_ChangeLayerOffset,
     Cmd_ChangeLayerOpacity,
     Cmd_ChangeTileTerrain,
     Cmd_ChangeTilesetTileOffset
 };
-
-#endif // UNDOCOMMANDS_H
