@@ -31,6 +31,7 @@
 #include <QGuiApplication>
 #include <QDebug>
 #include <QStringList>
+#include <QUrl>
 
 namespace {
 
@@ -40,7 +41,9 @@ struct CommandLineOptions {
         , showVersion(false)
         , scale(1.0)
         , tileSize(0)
+        , size(0)
         , useAntiAliasing(false)
+        , smoothImages(true)
         , ignoreVisibility(false)
     {}
 
@@ -50,7 +53,9 @@ struct CommandLineOptions {
     QString fileToSave;
     qreal scale;
     int tileSize;
+    int size;
     bool useAntiAliasing;
+    bool smoothImages;
     bool ignoreVisibility;
     QStringList layersToHide;
 };
@@ -70,7 +75,10 @@ static void showHelp()
             "  -s --scale SCALE        : The scale of the output image (default: 1)\n"
             "  -t --tilesize SIZE      : The requested size in pixels at which a tile is rendered\n"
             "                            Overrides the --scale option\n"
-            "  -a --anti-aliasing      : Smooth the output image using anti-aliasing\n"
+            "     --size SIZE          : The output image fits within a SIZE x SIZE square\n"
+            "                            Overrides the --scale and --tilesize options\n"
+            "  -a --anti-aliasing      : Antialias edges of primitives\n"
+            "     --no-smoothing       : Use nearest neighbour instead of smooth blending of pixels\n"
             "     --ignore-visibility  : Ignore all layer visibility flags in the map file, and render all\n"
             "                            layers in the output (default is to omit invisible layers)\n"
             "     --hide-layer         : Specifies a layer to omit from the output image\n"
@@ -79,8 +87,8 @@ static void showHelp()
 
 static void showVersion()
 {
-    qWarning() << "TMX Map Rasterizer"
-            << qPrintable(QCoreApplication::applicationVersion());
+    qWarning().noquote() << "TMX Map Rasterizer"
+                         << QCoreApplication::applicationVersion();
 }
 
 static void parseCommandLineArguments(CommandLineOptions &options)
@@ -120,6 +128,18 @@ static void parseCommandLineArguments(CommandLineOptions &options)
                     options.showHelp = true;
                 }
             }
+        } else if (arg == QLatin1String("--size")) {
+            i++;
+            if (i >= arguments.size()) {
+                options.showHelp = true;
+            } else {
+                bool sizeIsInt;
+                options.size = arguments.at(i).toInt(&sizeIsInt);
+                if (!sizeIsInt) {
+                    qWarning() << arguments.at(i) << ": the specified size is not an integer.";
+                    options.showHelp = true;
+                }
+            }
         } else if (arg == QLatin1String("--hide-layer")) {
             i++;
             if (i >= arguments.size()) {
@@ -130,6 +150,8 @@ static void parseCommandLineArguments(CommandLineOptions &options)
         } else if (arg == QLatin1String("--anti-aliasing")
                 || arg == QLatin1String("-a")) {
             options.useAntiAliasing = true;
+        } else if (arg == QLatin1String("--no-smoothing")) {
+            options.smoothImages = false;
         } else if (arg == QLatin1String("--ignore-visibility")) {
             options.ignoreVisibility = true;
         } else if (arg.isEmpty()) {
@@ -138,7 +160,11 @@ static void parseCommandLineArguments(CommandLineOptions &options)
             qWarning() << "Unknown option" << arg;
             options.showHelp = true;
         } else if (options.fileToOpen.isEmpty()) {
-            options.fileToOpen = arg;
+            const QUrl url(arg);
+            if (url.isLocalFile())
+                options.fileToOpen = url.toLocalFile();
+            else
+                options.fileToOpen = arg;
         } else if (options.fileToSave.isEmpty()) {
             options.fileToSave = arg;
         } else {
@@ -174,11 +200,13 @@ int main(int argc, char *argv[])
 
     TmxRasterizer w;
     w.setAntiAliasing(options.useAntiAliasing);
+    w.setSmoothImages(options.smoothImages);
     w.setIgnoreVisibility(options.ignoreVisibility);
     w.setLayersToHide(options.layersToHide);
 
-
-    if (options.tileSize > 0) {
+    if (options.size > 0) {
+        w.setSize(options.size);
+    } else if (options.tileSize > 0) {
         w.setTileSize(options.tileSize);
     } else if (options.scale > 0.0) {
         w.setScale(options.scale);
@@ -186,4 +214,3 @@ int main(int argc, char *argv[])
 
     return w.render(options.fileToOpen, options.fileToSave);
 }
-
