@@ -46,7 +46,9 @@ using namespace Tiled;
 TmxRasterizer::TmxRasterizer():
     mScale(1.0),
     mTileSize(0),
-    mUseAntiAliasing(true),
+    mSize(0),
+    mUseAntiAliasing(false),
+    mSmoothImages(true),
     mIgnoreVisibility(false)
 {
 }
@@ -55,9 +57,9 @@ TmxRasterizer::~TmxRasterizer()
 {
 }
 
-bool TmxRasterizer::shouldDrawLayer(Layer *layer)
+bool TmxRasterizer::shouldDrawLayer(const Layer *layer)
 {
-    if (layer->isObjectGroup())
+    if (layer->isObjectGroup() || layer->isGroupLayer())
         return false;
 
     if (mLayersToHide.contains(layer->name(), Qt::CaseInsensitive)) 
@@ -66,7 +68,7 @@ bool TmxRasterizer::shouldDrawLayer(Layer *layer)
     if (mIgnoreVisibility) 
         return true;
 
-    return layer->isVisible();
+    return !layer->isHidden();
 }
 
 int TmxRasterizer::render(const QString &mapFileName,
@@ -77,8 +79,9 @@ int TmxRasterizer::render(const QString &mapFileName,
     MapReader reader;
     map = reader.readMap(mapFileName);
     if (!map) {
-        qWarning().nospace() << "Error while reading " << mapFileName << ":\n"
-                             << qPrintable(reader.errorString());
+        qWarning("Error while reading \"%s\":\n%s",
+                 qUtf8Printable(mapFileName),
+                 qUtf8Printable(reader.errorString()));
         return 1;
     }
 
@@ -98,17 +101,20 @@ int TmxRasterizer::render(const QString &mapFileName,
         break;
     }
 
+    QSize mapSize = renderer->mapSize();
     qreal xScale, yScale;
 
-    if (mTileSize > 0) {
+    if (mSize > 0) {
+        xScale = (qreal) mSize / mapSize.width();
+        yScale = (qreal) mSize / mapSize.height();
+        xScale = yScale = qMin(1.0, qMin(xScale, yScale));
+    } else if (mTileSize > 0) {
         xScale = (qreal) mTileSize / map->tileWidth();
         yScale = (qreal) mTileSize / map->tileHeight();
     } else {
         xScale = yScale = mScale;
     }
 
-    QSize mapSize = renderer->mapSize();
-
     QMargins margins = map->computeLayerOffsetMargins();
     mapSize.setWidth(mapSize.width() + margins.left() + margins.right());
     mapSize.setHeight(mapSize.height() + margins.top() + margins.bottom());
@@ -120,23 +126,22 @@ int TmxRasterizer::render(const QString &mapFileName,
     image.fill(Qt::transparent);
     QPainter painter(&image);
 
-    if (xScale != qreal(1) || yScale != qreal(1)) {
-        if (mUseAntiAliasing) {
-            painter.setRenderHints(QPainter::SmoothPixmapTransform |
-                                   QPainter::Antialiasing);
-        }
-        painter.setTransform(QTransform::fromScale(xScale, yScale));
-    }
+    painter.setRenderHint(QPainter::Antialiasing, mUseAntiAliasing);
+    painter.setRenderHint(QPainter::SmoothPixmapTransform, mSmoothImages);
+    painter.setTransform(QTransform::fromScale(xScale, yScale));
 
     painter.translate(margins.left(), margins.top());
 
     // Perform a similar rendering than found in exportasimagedialog.cpp
-    for (Layer *layer : map->layers()) {
-        if (!shouldDrawLayer(layer)) 
+    LayerIterator iterator(map);
+    while (const Layer *layer = iterator.next()) {
+        if (!shouldDrawLayer(layer))
             continue;
 
-        painter.setOpacity(layer->opacity());
-        painter.translate(layer->offset());
+        const auto offset = layer->totalOffset();
+
+        painter.setOpacity(layer->effectiveOpacity());
+        painter.translate(offset);
 
         const TileLayer *tileLayer = dynamic_cast<const TileLayer*>(layer);
         const ImageLayer *imageLayer = dynamic_cast<const ImageLayer*>(layer);
@@ -147,7 +152,7 @@ int TmxRasterizer::render(const QString &mapFileName,
             renderer->drawImageLayer(&painter, imageLayer);
         }
 
-        painter.translate(-layer->offset());
+        painter.translate(-offset);
     }
 
     delete renderer;
@@ -155,9 +160,14 @@ int TmxRasterizer::render(const QString &mapFileName,
 
     // Save image
     QImageWriter imageWriter(imageFileName);
+
+    if (!imageWriter.canWrite())
+        imageWriter.setFormat("png");
+
     if (!imageWriter.write(image)) {
-        qWarning().nospace() << "Error while writing " << imageFileName << ": "
-                             << qPrintable(imageWriter.errorString());
+        qWarning("Error while writing \"%s\": %s",
+                 qUtf8Printable(imageFileName),
+                 qUtf8Printable(imageWriter.errorString()));
         return 1;
     }
 
