@@ -26,8 +26,7 @@
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef TMXRASTERIZER_H
-#define TMXRASTERIZER_H
+#pragma once
 
 #include "layer.h"
 
@@ -46,11 +45,14 @@ public:
     qreal scale() const { return mScale; }
     int tileSize() const { return mTileSize; }
     bool useAntiAliasing() const { return mUseAntiAliasing; }
+    bool smoothImages() const { return mSmoothImages; }
     bool IgnoreVisibility() const { return mIgnoreVisibility; }
 
     void setScale(qreal scale) { mScale = scale; }
     void setTileSize(int tileSize) { mTileSize = tileSize; }
+    void setSize(int size) { mSize = size; }
     void setAntiAliasing(bool useAntiAliasing) { mUseAntiAliasing = useAntiAliasing; }
+    void setSmoothImages(bool smoothImages) { mSmoothImages = smoothImages; }
     void setIgnoreVisibility(bool IgnoreVisibility) { mIgnoreVisibility = IgnoreVisibility; }
 
     void setLayersToHide(QStringList layersToHide) { mLayersToHide = layersToHide; }
@@ -60,12 +62,12 @@ public:
 private:
     qreal mScale;
     int mTileSize;
+    int mSize;
     bool mUseAntiAliasing;
+    bool mSmoothImages;
     bool mIgnoreVisibility;
     QStringList mLayersToHide;
 
-    bool shouldDrawLayer(Layer *layer);
+    bool shouldDrawLayer(const Layer *layer);
 
 };
-
-#endif // TMXRASTERIZER_H
